using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

using matrix;
using utilities;
using DifferentiableFunction;
using Dynamics;

namespace BSplineMechanism
{
    public class BSpline1
    {
        FArray P; //controlPoints
        Function fTangent;
        Function fAcceleration;
        Function fCurveValue;

        public BSpline1(params Function[] controlPoints)
        {
            P = Function.array(controlPoints);
        }
        
        public Function curveValue(Function t)
        {
            IntegerValue index = Function.floor(t + 3);
            t = t - Function.floor(t);
            Function B3 = ((1 - t) ^ 3);
            Function B2 = 3 * (t ^ 3) - 6 * (t ^ 2) + 4;
            Function B1 = 3 * (-(t ^ 3) + (t ^ 2) + t) + 1;
            Function B0 = (t ^ 3);
            fCurveValue = (1.0 / 6.0) * (B0 * P.get(index) +
            B1 * P.get(index - 1) +
            B2 * P.get(index - 2) +
            B3 * P.get(index - 3));
            return fCurveValue;
        }

        public Function tangent(Function t)
        {
            fTangent = Function.D(fCurveValue, t);
            fTangent = Function.derivative(fTangent);
            return fTangent;
        }

        public Function acceleration(Function t)
        {
            if (fTangent == null)
            {
                fTangent = tangent(t);
            }
            fAcceleration = Function.D(fTangent, t);
            fAcceleration = Function.derivative(fAcceleration);
            return fAcceleration;
        }
    }
    
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        #region Camera Model
        // The next set of variables are used to control the camera used in the sample. 
        // It is an arc ball camera, so it can rotate in a sphere around the target, and
        // zoom in and out.
        float cameraArc = -0.0f;
        float cameraRotation = 0.0f;
        float cameraDistance = 12.5f;
        float cameraMaxDistance = 100.0f;
        float cameraMinDistance = 0.2f;

        Matrix viewMatrix;
        Matrix projectionMatrix;

        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        #endregion

        MultiBodySystemXNA rbs;

        //spline
        BSpline1 spline;
        Function[] controlPoints;
        RuntimeFunction splineFun;
        Model cylinder; //for drawing spline
        Matrix[] cylinderABT; //absolute bone transforms
        Vector3[] splinePoints; //sample points of spline
        int numSegment = 100;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // define projection matrix
            float aspectRatio = graphics.GraphicsDevice.Viewport.Width /
                (float)graphics.GraphicsDevice.Viewport.Height;
            projectionMatrix = Matrix.CreatePerspectiveFieldOfView(
                MathHelper.ToRadians(45.0f), aspectRatio, .01f, 1000);

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here
            rbs = buildSplineMechanism(Content);

            rbs.setInitialState(new double[] { 1.0 }, new double[] { 0 });

            cylinder = Content.Load<Model>("unitCylinderRed");
            cylinderABT = new Matrix[cylinder.Bones.Count];
            cylinder.CopyAbsoluteBoneTransformsTo(cylinderABT);

            Variable x = new Variable("x");
            BSpline1 spline = new BSpline1(controlPoints);
            Function f = spline.curveValue(x);
            splineFun = f.compile();

            //make splinePoints
            double qmin = 1;
            double qmax = 9;
            double[] y = new double[1];
            splinePoints = new Vector3[numSegment + 1];
            for (int i = 0; i < numSegment + 1; ++i)
            {
                double q = qmin + i * (qmax - qmin) / numSegment;
                splineFun.eval(y, new double[] { q });
                splinePoints[i] = new Vector3((float)q, (float)y[0], 0);
            }

        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here

            NextStep(gameTime);

            UpdateCamera();

            base.Update(gameTime);
        }

        protected void NextStep(GameTime gameTime)
        {
            //proceed to the next time step
            double timeStep = 0.016;
            int stepMult = Math.Max(1, (int)Math.Round(gameTime.ElapsedGameTime.TotalSeconds / timeStep));
            for (int t = 0; t < stepMult; t++)
            {
                //define applied forces
                rbs.setZeroAllForces();
                double gravity = -9.8;
                for (int i = 0; i < rbs.numberOfNodes; ++i)
                {
                    double[] gforce = new double[] { 0, gravity * rbs.allNodes[i].mass(), 0 };
                    rbs.addExternalForce(i, gforce);
                }
                //damping
                double Kd = 2.6; //damping coefficient
                for (int i = 0; i < rbs.numberOfCoords; ++i)
                {
                    rbs.addGenForce(i, -Kd * rbs.dq[i]);
                }

                rbs.nextStepForwardEulerMethod(timeStep);
            }
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);
            GraphicsDevice.RenderState.DepthBufferEnable = true;

            //draw spline
            DrawSpline();

            //draw mechanism
            rbs.PrepareToDraw();
            rbs.Draw(viewMatrix, projectionMatrix);

            base.Draw(gameTime);
        }

        void DrawLineSegment(Vector3 start, Vector3 end, float width)
        {
            Vector3 center = (start + end) / 2;
            Vector3 x = end - start;
            x.Normalize();
            Vector3 z = Vector3.Cross(x, Vector3.Up);
            z.Normalize();
            Vector3 y = Vector3.Cross(z, x);
            Matrix worldXform = new Matrix(x.X, x.Y, x.Z, 0, y.X, y.Y, y.Z, 0, z.X, z.Y, z.Z, 0, center.X, center.Y, center.Z, 1);
            worldXform = Matrix.CreateScale(width, (end - start).Length(), width) * Matrix.CreateRotationZ((float)Math.PI / 2) * worldXform;
            DrawModel(cylinder, worldXform, cylinderABT);
        }

        void DrawSpline()
        {
            for (int i = 0; i < numSegment; ++i)
            {
                DrawLineSegment(splinePoints[i], splinePoints[i + 1], 0.15f);
            }

        }

        /// <summary>
        /// Handles input for moving the camera.
        /// </summary>
        void UpdateCamera()
        {
            // clamp the camera distance so it doesn't get too close or too far away.
            cameraDistance = MathHelper.Clamp(cameraDistance, cameraMinDistance, cameraMaxDistance);

            Matrix unrotatedView = Matrix.CreateLookAt(
                new Vector3(5, 3, cameraDistance), new Vector3(5, 3, 0), Vector3.Up);

            viewMatrix = Matrix.CreateRotationY(MathHelper.ToRadians(cameraRotation)) *
                          Matrix.CreateRotationX(MathHelper.ToRadians(cameraArc)) *
                          unrotatedView;
        }

        /// <summary>
        /// DrawModel is a helper function that takes a model, world matrix, and
        /// bone transforms. It does just what its name implies, and draws the model.
        /// </summary>
        /// <param name="model">the model to draw</param>
        /// <param name="worldTransform">where to draw the model</param>
        /// <param name="absoluteBoneTransforms">the model's bone transforms. this can
        /// be calculated using the function Model.CopyAbsoluteBoneTransformsTo</param>
        private void DrawModel(Model model, Matrix worldTransform, Matrix[] absoluteBoneTransforms)
        {
            // nothing tricky in here; this is the same model drawing code that we see
            // everywhere. we'll loop over all of the meshes in the model, set up their
            // effects, and then draw them.
            foreach (ModelMesh mesh in model.Meshes)
            {
                foreach (BasicEffect effect in mesh.Effects)
                {
                    effect.EnableDefaultLighting();
                    effect.PreferPerPixelLighting = true;

                    effect.View = viewMatrix;
                    effect.Projection = projectionMatrix;
                    effect.World = absoluteBoneTransforms[mesh.ParentBone.Index] * worldTransform;
                }
                mesh.Draw();
            }
        }

        public MultiBodySystemXNA buildSplineMechanism(ContentManager Content)
        {
            Function.newContext();
            List<LinkXNA> alljoints = new List<LinkXNA>();
            Variable t = new Variable("t");
            Graph<Link> g = null;

            //spline
            controlPoints = new Function[] { 5, 5, 5, 1, 5, 0, 4, -1, 3, -2, 5, 5, 5 };
            spline = new BSpline1(controlPoints);

            //dimension of a link
            float mass = 1.0f;
            float len = 0.8f;
            Function[,] centerOfMass = { { 0 }, { 0 }, { 0 }, { 1 } };
            Function[,] inertia = MultiBodySystem.inertiaMatrixBox(mass, len, len, len);

            //link0
            UnspecifiedFunction q0 = UnspecifiedFunction.functionOf("q0", t);

            Function f = spline.curveValue(q0);
            Function df = spline.tangent(q0);

            Function[,] A0 = VM.rotZ(Function.atan(df), q0, f, 0); 

            LinkXNA rigidBody0 = new LinkXNA(A0, new Function[] { q0 }, centerOfMass, inertia, mass, "link0", t);
            rigidBody0.SetShape(Content.Load<Model>("unitBoxChamGreen"));
            rigidBody0.scale = new Vector3(len);

            alljoints.Add(rigidBody0);

            g = new Graph<Link>(rigidBody0, alljoints.ToArray());

            MultiBodySystemXNA RBSTemp = new MultiBodySystemXNA(g, t, null);

            return RBSTemp;
        }


    }
}
