using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

using matrix;
using utilities;
using DifferentiableFunction;
using Dynamics;

namespace WindowsGame1
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        #region Camera Model
        // The next set of variables are used to control the camera used in the sample. 
        // It is an arc ball camera, so it can rotate in a sphere around the target, and
        // zoom in and out.
        float cameraArc = -30.0f;
        float cameraRotation = 30.0f;
        float cameraDistance = 10.5f;
        float cameraMaxDistance = 20.0f;
        float cameraMinDistance = 0.2f;
        
        Matrix viewMatrix;
        Matrix projectionMatrix;

        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        #endregion

        MultiBodySystemXNA rbs;
 
        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";            

       }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // define projection matrix
            float aspectRatio = graphics.GraphicsDevice.Viewport.Width /
                (float)graphics.GraphicsDevice.Viewport.Height;
            projectionMatrix = Matrix.CreatePerspectiveFieldOfView(
                MathHelper.ToRadians(45.0f), aspectRatio, .01f, 1000);

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here
            rbs = buildTwoLinkMechanism(Content);
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here
            
            NextStep(gameTime);

            UpdateCamera();

            base.Update(gameTime);
        }

        protected void NextStep(GameTime gameTime)
        {
            //proceed to the next time step
            double timeStep = 0.016;
            int stepMult = Math.Max(1, (int)Math.Round(gameTime.ElapsedGameTime.TotalSeconds / timeStep));
            for (int t = 0; t < stepMult; t++)
            {
                //define applied forces
                rbs.setZeroAllForces();
                double gravity = -9.8;
                for (int i = 0; i < rbs.numberOfNodes; ++i)
                {
                    double[] gforce = new double[] { 0, gravity * rbs.allNodes[i].mass(), 0 };
                    rbs.addExternalForce(i, gforce );                
                }
                //damping
                double Kd = 1; //damping coefficient
                for (int i = 0; i < rbs.numberOfCoords; ++i)
                {
                    rbs.addGenForce(i, -Kd * rbs.dq[i]);
                }

                ////inv dyn test
                //double[] qddt = new double[rbs.numberOfCoords];
                //double[] tau;
                //rbs.inverseDynamics(qddt, out tau);
                //rbs.setGenForce(tau);

                rbs.nextStepForwardEulerMethod(timeStep);
            }            
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);
            GraphicsDevice.RenderState.DepthBufferEnable = true;

            rbs.PrepareToDraw();
            rbs.Draw(viewMatrix, projectionMatrix);

            base.Draw(gameTime);
        }

        /// <summary>
        /// Handles input for moving the camera.
        /// </summary>
        void UpdateCamera()
        {            
            // clamp the camera distance so it doesn't get too close or too far away.
            cameraDistance = MathHelper.Clamp(cameraDistance, cameraMinDistance, cameraMaxDistance);

            Matrix unrotatedView = Matrix.CreateLookAt(
                new Vector3(0, 0, -cameraDistance), Vector3.Zero, Vector3.Up);

            viewMatrix = Matrix.CreateRotationY(MathHelper.ToRadians(cameraRotation)) *
                          Matrix.CreateRotationX(MathHelper.ToRadians(cameraArc)) *
                          unrotatedView;
        }

        /// <summary>
        /// DrawModel is a helper function that takes a model, world matrix, and
        /// bone transforms. It does just what its name implies, and draws the model.
        /// </summary>
        /// <param name="model">the model to draw</param>
        /// <param name="worldTransform">where to draw the model</param>
        /// <param name="absoluteBoneTransforms">the model's bone transforms. this can
        /// be calculated using the function Model.CopyAbsoluteBoneTransformsTo</param>
        private void DrawModel(Model model, Matrix worldTransform, Matrix[] absoluteBoneTransforms)
        {
            // nothing tricky in here; this is the same model drawing code that we see
            // everywhere. we'll loop over all of the meshes in the model, set up their
            // effects, and then draw them.
            foreach (ModelMesh mesh in model.Meshes)
            {
                foreach (BasicEffect effect in mesh.Effects)
                {
                    effect.EnableDefaultLighting();
                    effect.PreferPerPixelLighting = true;

                    effect.View = viewMatrix;
                    effect.Projection = projectionMatrix;
                    effect.World = absoluteBoneTransforms[mesh.ParentBone.Index] * worldTransform;
                }
                mesh.Draw();
            }
        }

        public MultiBodySystemXNA buildTwoLinkMechanism(ContentManager Content)
        {
            Function.newContext();
            List<LinkXNA> alljoints = new List<LinkXNA>();
            Variable t = new Variable("t");
            Graph<Link> g = null;

            //dimension of a link
            float mass = 1.0f;
            float lenX = 2.0f;
            float lenY = 0.5f;
            float lenZ = 1.0f;
            Function[,] centerOfMass = { { 0.5*lenX }, { 0 }, { 0 }, { 1 } };
            Function[,] inertia = MultiBodySystem.inertiaMatrixBox(mass, lenX, lenY, lenZ);

            //link0
            UnspecifiedFunction q0 = UnspecifiedFunction.functionOf("q0", t);
            Function[,] A0 = VM.translate(0, 0, 0); 
            A0 = VM.mult(A0, VM.rotZ(q0, 0, 0, 0));
            LinkXNA rigidBody0 = new LinkXNA(A0, new Function[] { q0 }, centerOfMass, inertia, mass, "link0", t);
            alljoints.Add(rigidBody0);

            //link1
            UnspecifiedFunction q1 = UnspecifiedFunction.functionOf("q1", t);
            Function[,] A1 = VM.translate(lenX, 0, 0);
            A1 = VM.mult(A1, VM.rotZ(q1, 0, 0, 0));
            LinkXNA rigidBody1 = new LinkXNA(A1, new Function[] { q1 }, centerOfMass, inertia, mass, "link1", t);
            alljoints.Add(rigidBody1);
            rigidBody0.c(rigidBody1);  // declare rigidBody1 as a child of rigidBody0

            g = new Graph<Link>(rigidBody0, alljoints.ToArray());

            //Function[,] explicitConstraints = new Function[,] { { q0, q1} };
            MultiBodySystemXNA RBSTemp = new MultiBodySystemXNA(g, t, null);

            //visualization
            rigidBody0.SetShape(Content.Load<Model>("unitBoxChamRed"));
            rigidBody0.scale = new Vector3(lenX, lenY, lenZ);
            rigidBody0.translation = new Vector3(0.5f * lenX, 0, 0);
            rigidBody1.SetShape(Content.Load<Model>("unitBoxChamGreen"));
            rigidBody1.scale = new Vector3(lenX, lenY, lenZ);
            rigidBody1.translation = new Vector3(0.5f * lenX, 0, 0);
            return RBSTemp;
        }


    }
}
