using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

using matrix;
using utilities;
using DifferentiableFunction;
using Dynamics;

namespace Humanoid
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        #region Camera Model
        // The next set of variables are used to control the camera used in the sample. 
        // It is an arc ball camera, so it can rotate in a sphere around the target, and
        // zoom in and out.
        float cameraArc = -30.0f;
        float cameraRotation = 30.0f;
        float cameraDistance = 5.5f;
        float cameraMaxDistance = 40.0f;
        float cameraMinDistance = 0.2f;

        Matrix viewMatrix;
        Matrix projectionMatrix;

        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        SpriteFont spriteFont;
        #endregion

        MultiBodySystemXNA rbs;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // define projection matrix
            float aspectRatio = graphics.GraphicsDevice.Viewport.Width /
                (float)graphics.GraphicsDevice.Viewport.Height;
            projectionMatrix = Matrix.CreatePerspectiveFieldOfView(
                MathHelper.ToRadians(45.0f), aspectRatio, .01f, 1000);

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            spriteFont = Content.Load<SpriteFont>("SpriteFont1");

            // TODO: use this.Content to load your game content here
            rbs = buildTwoLinkMechanism(Content);
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        int wind = 0;

        protected void KeyboardInput()
        {
            wind = 0;
            KeyboardState keyState = Keyboard.GetState();
            if (keyState.IsKeyDown(Keys.Space))
            {
                for (int i = 0; i < rbs.numberOfCoords; ++i) { rbs.q[i] = rbs.dq[i] = 0; }
                rbs.kinematicsUpdate(rbs.q,out rbs.W);
            }
            if (keyState.IsKeyDown(Keys.Right))
            {
                wind = 1;
            }
            if (keyState.IsKeyDown(Keys.Left))
            {
                wind = 2;
            }
            if (keyState.IsKeyDown(Keys.Up))
            {
                wind = 3;
            }
            if (keyState.IsKeyDown(Keys.Down))
            {
                wind = 4;
            }

        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here
            KeyboardInput();

            NextStep(gameTime);

            UpdateCamera();

            base.Update(gameTime);
        }

        protected void NextStep(GameTime gameTime)
        {
            //proceed to the next time step
            double timeStep = 0.016/5;
            int stepMult = Math.Max(1, (int)Math.Round(gameTime.ElapsedGameTime.TotalSeconds / timeStep));
            double[] Ks = new double[] { 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 30, 30, 30, 30, 30, 30, 30, 30, 1, 1, 1, 1, 1, 1, 1, 1 };
            for (int t = 0; t < stepMult; t++)
            {
                //define applied forces
                rbs.setZeroAllForces();

                //gravity
                double gravity = -9.8;
                for (int i = 0; i < rbs.numberOfNodes; ++i)
                {
                    double[] gforce = new double[] { 0, gravity * rbs.allNodes[i].mass(), 0 };
                    rbs.addExternalForce(i, gforce);
                }

                //user defined force
                if (wind == 1)
                {
                    for (int i = 0; i < rbs.numberOfNodes; ++i)
                    {
                        rbs.addExternalForce(i, new double[] { -10, 0, 0 });
                    }
                }
                else if (wind == 2)
                {
                    for (int i = 0; i < rbs.numberOfNodes; ++i)
                    {
                        rbs.addExternalForce(i, new double[] { 10, 0, 0 });
                    }
                }
                else if (wind == 3)
                {
                    for (int i = 0; i < rbs.numberOfNodes; ++i)
                    {
                        rbs.addExternalForce(i, new double[] { 0, 0, 10 });
                    }
                }
                else if (wind == 4)
                {
                    for (int i = 0; i < rbs.numberOfNodes; ++i)
                    {
                        rbs.addExternalForce(i, new double[] { 0, 0, -10 });
                    }
                }

                //spring and damping
                double Kd = 3; //damping coefficient
                for (int i = 0; i < rbs.numberOfCoords; ++i)
                {
                    rbs.addGenForce(i,-Ks[i]*rbs.q[i] -Kd * rbs.dq[i]);
                }

                rbs.nextStepForwardEulerMethod(timeStep);
            }
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);
            GraphicsDevice.RenderState.DepthBufferEnable = true;

            rbs.PrepareToDraw();
            rbs.Draw(viewMatrix, projectionMatrix);

            spriteBatch.Begin();
            DrawText();
            spriteBatch.End();
            
            base.Draw(gameTime);
        }
       
        void DrawText()
        {
            spriteBatch.DrawString(spriteFont, "Arrows: apply external forces. Spacebar: return to initial state.", new Vector2(20, 45), Color.White);
        }



        /// <summary>
        /// Handles input for moving the camera.
        /// </summary>
        void UpdateCamera()
        {
            // clamp the camera distance so it doesn't get too close or too far away.
            cameraDistance = MathHelper.Clamp(cameraDistance, cameraMinDistance, cameraMaxDistance);

            Matrix unrotatedView = Matrix.CreateLookAt(
                new Vector3(0, 0, -cameraDistance), new Vector3(0,-1,0), Vector3.Up);

            viewMatrix = Matrix.CreateRotationY(MathHelper.ToRadians(cameraRotation)) *
                          Matrix.CreateRotationX(MathHelper.ToRadians(cameraArc)) *
                          unrotatedView;
        }

        /// <summary>
        /// DrawModel is a helper function that takes a model, world matrix, and
        /// bone transforms. It does just what its name implies, and draws the model.
        /// </summary>
        /// <param name="model">the model to draw</param>
        /// <param name="worldTransform">where to draw the model</param>
        /// <param name="absoluteBoneTransforms">the model's bone transforms. this can
        /// be calculated using the function Model.CopyAbsoluteBoneTransformsTo</param>
        private void DrawModel(Model model, Matrix worldTransform, Matrix[] absoluteBoneTransforms)
        {
            // nothing tricky in here; this is the same model drawing code that we see
            // everywhere. we'll loop over all of the meshes in the model, set up their
            // effects, and then draw them.
            foreach (ModelMesh mesh in model.Meshes)
            {
                foreach (BasicEffect effect in mesh.Effects)
                {
                    effect.EnableDefaultLighting();
                    effect.PreferPerPixelLighting = true;

                    effect.View = viewMatrix;
                    effect.Projection = projectionMatrix;
                    effect.World = absoluteBoneTransforms[mesh.ParentBone.Index] * worldTransform;
                }
                mesh.Draw();
            }
        }

        public MultiBodySystemXNA buildTwoLinkMechanism(ContentManager Content)
        {
            Function.newContext();
            List<LinkXNA> alljoints = new List<LinkXNA>();
            Variable t = new Variable("t");
            Graph<Link> g = null;

            //dimension of a link
            float len = 1.0f, rad = 0.05f;
            float mass = 1;// density * (float)Math.PI * rad * rad * len;
            Function[,] com = new Function[,]{{0},{len/-2},{0},{1}};
            Function[,] inertia = MultiBodySystem.inertiaMatrixCylinder(mass, rad, len);

            //root
            UnspecifiedFunction q00 = UnspecifiedFunction.functionOf("qr0", t);
            UnspecifiedFunction q01 = UnspecifiedFunction.functionOf("qr1", t);
            //UnspecifiedFunction q02 = UnspecifiedFunction.functionOf("qr2", t);
            Function[,] Aroot = VM.translate(0, 0, 0);
            Aroot = VM.mult(VM.mult(VM.mult(Aroot, VM.rotX(q00, 0, 0, 0)), VM.rotY(0, 0, 0, 0)), VM.rotZ(q01, 0, 0, 0));
            LinkXNA root = new LinkXNA(Aroot, new Function[] { q00, q01 }, com, inertia, mass, "root", t);
            root.SetShape(Content.Load<Model>("unitCylinderGreen"));
            root.scale = new Vector3(rad * 2, len, rad * 2);
            root.translation = new Vector3(0, len/-2, 0);
            alljoints.Add(root);

            int n = 4;
            
            // level 1
            UnspecifiedFunction[] q = new UnspecifiedFunction[n];
            LinkXNA[] rod = new LinkXNA[n];
            for (int i = 0; i < n; ++i)
            {
                //rod i
                q[i] = UnspecifiedFunction.functionOf("q"+i.ToString(), t);
                
                Function[,] A = VM.mult(VM.rotY(i * Math.PI * 2.0f / n, 0, -len, 0), VM.rotZ(-Math.PI / 2, 0, 0, 0));
                A = VM.mult(A, VM.rotZ(q[i], 0, 0, 0));
                rod[i] = new LinkXNA(A, new Function[] { q[i] }, com, inertia, mass, "rod"+i.ToString(), t);
                rod[i].SetShape(Content.Load<Model>("unitCylinderRed"));
                rod[i].scale = new Vector3(rad * 2, len, rad * 2);
                rod[i].translation = new Vector3(0, len / -2, 0);
                alljoints.Add(rod[i]);
                root.c(rod[i]);
            }

            // level 2
            UnspecifiedFunction[] qq = new UnspecifiedFunction[n * 2];
            LinkXNA[] rod2 = new LinkXNA[n];
            for (int i = 0; i < n; ++i)
            {
                //rod i
                qq[i * 2] = UnspecifiedFunction.functionOf("qq" + i.ToString() + "0", t);
                qq[i * 2 + 1] = UnspecifiedFunction.functionOf("qq" + i.ToString() + "1", t);

                Function[,] A = VM.rotZ(Math.PI / 2, 0, -len, 0);
                A = VM.mult(VM.mult(A, VM.rotX(qq[i * 2], 0, 0, 0)), VM.rotZ(qq[i * 2 + 1], 0, 0, 0));
                rod2[i] = new LinkXNA(A, new Function[] { qq[i * 2], qq[i * 2 + 1] }, com, inertia, mass, "rod2" + i.ToString(), t);
                rod2[i].SetShape(Content.Load<Model>("unitCylinderGreen"));
                rod2[i].scale = new Vector3(rad * 2, len, rad * 2);
                rod2[i].translation = new Vector3(0, len / -2, 0);
                alljoints.Add(rod2[i]);
                rod[i].c(rod2[i]);
            }

            //level3
            UnspecifiedFunction[] qqq = new UnspecifiedFunction[n];
            LinkXNA[] obj = new LinkXNA[n];
            for (int i = 0; i < n; ++i)
            {
                //rod i
                qqq[i] = UnspecifiedFunction.functionOf("qqq" + i.ToString(), t);
                
                Function[,] A = VM.translate(0, -len, 0);
                A = VM.mult(A, VM.rotX(qqq[i], 0, 0, 0));

                double mass1 = 3;
                Function[,] com1 = new Function[,] { { 0 }, { -0.25 }, { 0 }, { 1 } };
                Function[,] inertia1 = MultiBodySystem.inertiaMatrixBox(mass1, 0.5, 0.5, 0.5);

                obj[i] = new LinkXNA(A, new Function[] { qqq[i] }, com1, inertia1, mass1, "obj" + i.ToString(), t);
                switch (i)
                {
                    case 0:
                        obj[i].SetShape(Content.Load<Model>("unitSphere"));
                        break;
                    case 1:
                        obj[i].SetShape(Content.Load<Model>("unitBoxChamPurple"));
                        break;
                    case 2:
                        obj[i].SetShape(Content.Load<Model>("unitCylinderCham"));
                        break;
                    case 3:
                        obj[i].SetShape(Content.Load<Model>("unitBoxChamRed"));
                        break;
                }
                obj[i].scale = new Vector3(0.5f);
                obj[i].translation = new Vector3(0, -0.25f, 0);
                alljoints.Add(obj[i]);
                rod2[i].c(obj[i]);
            }

            g = new Graph<Link>(root, alljoints.ToArray());

            MultiBodySystemXNA RBSTemp = new MultiBodySystemXNA(g, t, null);

            return RBSTemp;
        }


    }
}
