﻿//Written by Leonid Velikovich and Benjamin Hernandez Arreguin

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Windows.Forms;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using matrix;
using utilities;
using DifferentiableFunction;
using Interval;

using Float = ParametricModelEditor.FloatingPointMathHelper;

namespace ParametricModelEditor
{
    class ParametricModelDStar3DVisualizer
    {
        private const int c_triangleCount = 30000;
        
        public ParametricModelDStar3DVisualizer (GraphicsDevice device, int xSamples, int ySamples)
        {
            _device = device;
            _strEffect = File.ReadAllText("ParametricSurfaceEffect.fx");
            Resample(2 * xSamples * ySamples);
            
        }

        public void OnDisposeGraphicsDevice()
        {
            if (_hardwareEffect != null)
            {
                _hardwareEffect.Dispose();
                _hardwareEffect = null;
            }

            _device = null;
        }

        
        public void OnResetGraphicsDevice(GraphicsDevice device)
        {
            _device = device;
                                    
            CompiledEffect compEffect = Effect.CompileEffectFromSource(_strEffect,
                null, null, CompilerOptions.Debug, TargetPlatform.Windows);

            Debug.Assert(compEffect.Success == true, compEffect.ErrorsAndWarnings);

            byte[] effectCode = compEffect.GetEffectCode();

            _hardwareEffect = new Effect(device, effectCode, CompilerOptions.Debug, null);

            _hardwareEffect.CurrentTechnique = _hardwareEffect.Techniques["SoftwareTechnique"];

            /*
            _vertexBufferPositionNormal = new VertexBuffer(device, typeof(VertexPositionNormal), _numVertices,
                                             ResourceUsage.Dynamic | ResourceUsage.WriteOnly,
                                             ResourceManagementMode.Manual);
             * */
            
            _vertexBufferPositionNormal = new VertexBuffer(device, typeof(VertexPositionNormal), _numVertices,
                                             BufferUsage.WriteOnly);
            
            

        }

        public void OnRender(GraphicsDevice device, float aspectRatio)
        {
            _device = device;
            Vector3 eyeLocation = new Vector3(-3.0f, 0.0f, 1f) * _distance;
            Vector3 upVector = new Vector3(0, 0, 1);
            Matrix leanTransform = Matrix.CreateRotationY(_leanAngle);
            eyeLocation = Vector3.Transform(eyeLocation, leanTransform);
            upVector = Vector3.Transform(upVector, leanTransform);

            _hardwareEffect.Parameters["World"].SetValue( Matrix.CreateRotationZ(_rotation));
            _hardwareEffect.Parameters["View"].SetValue(
                Matrix.CreateLookAt(eyeLocation,
                                    Vector3.Zero,
                                    upVector));
            _hardwareEffect.Parameters["Projection"].SetValue(
                Matrix.CreatePerspectiveFieldOfView(1.25f * MathHelper.Pi / 4.0f,
                                                    aspectRatio,
                                                    .25f, 100.0f));
            _hardwareEffect.Parameters["EyeLocation"].SetValue(eyeLocation);

            _hardwareEffect.Parameters["AmbientFactor"].SetValue(_materialAmbientFactor);
            _hardwareEffect.Parameters["DiffuseFactor"].SetValue(_materialDiffuseFactor);
            _hardwareEffect.Parameters["SpecularFactor"].SetValue(_materialSpecularFactor);
            _hardwareEffect.Parameters["MaterialColor"].SetValue(_materialColor);

            _hardwareEffect.CommitChanges();


            if (_eqCalculated)
                GenerateMesh();
            
            EffectTechnique technique;
            technique = _hardwareEffect.Techniques["SoftwareTechnique"];
            _device.VertexDeclaration = new VertexDeclaration(device, VertexPositionNormal.Elements);
            _device.Vertices[0].SetSource(_vertexBufferPositionNormal, 0, VertexPositionNormal.SizeInBytes);
            _hardwareEffect.Begin();
            technique.Passes[0].Begin();
            _device.DrawUserIndexedPrimitives(PrimitiveType.TriangleStrip, _verticesPositionNormal, 0, _numVertices, _indices, 0, _numTriangles);
            technique.Passes[0].End();
            _hardwareEffect.End();
  

        }

        private void ResetGeometry()
        {
            if (_vertexBufferPositionNormal != null)
            {
                _vertexBufferPositionNormal.Dispose();
                _vertexBufferPositionNormal = null;
            }
        }

        public ParametricModel3DData GenerateParametricModel3DData() {
            ParametricModel3DData parametricModel3DData = new ParametricModel3DData();
            parametricModel3DData.MaterialProperties.InternalColor = _materialColor;
            parametricModel3DData.MaterialProperties.AmbientFactor = _materialAmbientFactor;
            parametricModel3DData.MaterialProperties.DiffuseFactor = _materialDiffuseFactor;
            parametricModel3DData.MaterialProperties.SpecularFactor = _materialSpecularFactor;

            //if (_parametricSurfaceEffect != null) {
                parametricModel3DData.ParametricEffectType = _parametricSurfaceEffect.ParametricEffectType;
                parametricModel3DData.ParametricCurves = _parametricSurfaceEffect.GetSerialization();
            //}
            return parametricModel3DData;
        }


        public void ApplySurfaceOfRevolutionEffect(ParametricCurve2D profileCurve)
        {
            /*
             Add DStar Code
             * */

            // HLSL Code
            //PARAMETRICCURVE2D_OUTPUT profileCurve = BSpline_Evaluate(input.x,
            //                                                              0,
            //                                                              BSpline1_ControlPointCount);
            //profileCurve.Position *= .004;

            //float theta = TWOPI * input.y;

            //PARAMETRICSURFACE3D_OUTPUT output;

            ////
            //// Compute position
            ////    
            //output.Position.x = cos(theta) * profileCurve.Position.x;
            //output.Position.y = -sin(theta) * profileCurve.Position.x;
            //output.Position.z = .7 - profileCurve.Position.y;

            ////
            //// Compute 1st-order partials
            ////
            //output.dF_du.x = cos(theta) * profileCurve.Tangent.x;
            //output.dF_dv.x = -TWOPI * sin(theta) * profileCurve.Position.x;

            //output.dF_du.y = -sin(theta) * profileCurve.Tangent.x;
            //output.dF_dv.y = -TWOPI * cos(theta) * profileCurve.Position.x;

            //output.dF_du.z = -profileCurve.Tangent.y;
            //output.dF_dv.z = 0;

            ////
            //// Compute 2nd-order partials
            ////
            //output.ddF_duu.x = cos(theta) * profileCurve.Acceleration.x;
            //output.ddF_dvv.x = -TWOPI * TWOPI * cos(theta) * profileCurve.Position.x;

            //output.ddF_duu.y = -sin(theta) * profileCurve.Acceleration.x;
            //output.ddF_dvv.y = TWOPI * TWOPI * sin(theta) * profileCurve.Position.x;

            //output.ddF_duu.z = -profileCurve.Acceleration.y;
            //output.ddF_dvv.z = 0;

            //return output;
            //GenerateMesh(); 
        }
        
        private void Resample (int totalTriangles)
        {
            float root = Float.Sqrt(.5f * (float)totalTriangles);  // # of grid squares = 1/2 * #triangles


            _numUSamples = (int)(root + .5f);
            _numVSamples = (int)(root + .5f);

            InitializeVertexBuffer();
            InitializeIndices();

        }

        private Function cross(Function a, Function b)
        {
            return new Function(a[1] * b[2] - a[2] * b[1],
                                a[2] * b[0] - a[0] * b[2],
                                a[0] * b[1] - a[1] * b[0]);
        }
        public void upDateCtrlPoints(ParametricCurve2D profileCurve, ParametricCurve2D crossSectionCurve)
        {
            int i;
            Vector2[] ctrlPointsProfileCurve = profileCurve.GetControlPoints();
            Vector2[] ctrlPointsCrossSectionCurve = crossSectionCurve.GetControlPoints();
            _ctrlPointsProfileCurveX = new double
                                                [ctrlPointsProfileCurve.Length];
            _ctrlPointsProfileCurveY = new double
                                                [ctrlPointsProfileCurve.Length];
            _ctrlCrossSectionCurveX = new double
                                                [ctrlPointsCrossSectionCurve.Length];
            _ctrlCrossSectionCurveY = new double
                                                [ctrlPointsCrossSectionCurve.Length];

            for (i = 0; i < ctrlPointsProfileCurve.Length; i++)
            {
                _ctrlPointsProfileCurveX[i] = ctrlPointsProfileCurve[i].X;
                _ctrlPointsProfileCurveY[i] = ctrlPointsProfileCurve[i].Y;
            }
            for (i = 0; i < ctrlPointsCrossSectionCurve.Length; i++)
            {
                _ctrlCrossSectionCurveX[i] = ctrlPointsCrossSectionCurve[i].X;
                _ctrlCrossSectionCurveY[i] = ctrlPointsCrossSectionCurve[i].Y;
            }
        }

        public void ApplyProfileProductEffect(ParametricCurve2D profileCurve, ParametricCurve2D crossSectionCurve)
        {

            
            int i;
            Function.newContext();
            //Function.printCompilerSource = true;
            Variable[] t = { new Variable("t"), new Variable("tetha")};
            Variable[] inputVars;

            //_isClosedCrossSectionCurve = crossSectionCurve.

            Vector2[] ctrlPointsProfileCurve = profileCurve.GetControlPoints();
            Vector2[] ctrlPointsCrossSectionCurve = crossSectionCurve.GetControlPoints();
            _ctrlPointsProfileCurveX = new double
                                                [ctrlPointsProfileCurve.Length];
            _ctrlPointsProfileCurveY = new double
                                                [ctrlPointsProfileCurve.Length];
            _ctrlCrossSectionCurveX = new double
                                                [ctrlPointsCrossSectionCurve.Length];
            _ctrlCrossSectionCurveY = new double
                                                [ctrlPointsCrossSectionCurve.Length];
            
            Variable[] fCtrlPointsProfileCurveX = new Variable[4];
            Variable[] fCtrlPointsProfileCurveY = new Variable[4];
            Variable[] fCtrlCrossSectionCurveX  = new Variable[4];
            Variable[] fCtrlCrossSectionCurveY  = new Variable[4];

            for (i = 0; i < 4; i++)
            {
                fCtrlPointsProfileCurveX[i] = new Variable("profileX" + i);
                fCtrlPointsProfileCurveY[i] = new Variable("profileY" + i);
                fCtrlCrossSectionCurveX[i] = new Variable("crossSectionX" + i);
                fCtrlCrossSectionCurveY[i] = new Variable("crossSectionY" + i);
            }            
            for (i = 0; i < ctrlPointsProfileCurve.Length; i++)
            {
                _ctrlPointsProfileCurveX[i] = ctrlPointsProfileCurve[i].X;
                _ctrlPointsProfileCurveY[i] = ctrlPointsProfileCurve[i].Y;
            }
            for (i = 0; i < ctrlPointsCrossSectionCurve.Length; i++)
            {
                _ctrlCrossSectionCurveX[i] = ctrlPointsCrossSectionCurve[i].X;
                _ctrlCrossSectionCurveY[i] = ctrlPointsCrossSectionCurve[i].Y;
            }

            
            //PROFILE CURVE             PROFILE CURVE
            BSpline profileCurveX = new BSpline(fCtrlPointsProfileCurveX);          
            Function fProfileCurvePositionX = profileCurveX.curveValue(t[0])*0.004;
            Function fProfileCurveTangentX = profileCurveX.tangent(t[0]);
            Function fProfileCurveAccelerationX = profileCurveX.acceleration(t[0]);
            
            BSpline profileCurveY = new BSpline(fCtrlPointsProfileCurveY);
            Function fProfileCurvePositionY = profileCurveY.curveValue(t[0])*0.004;
            Function fProfileCurveTangentY = profileCurveY.tangent(t[0]);
            Function fProfileCurveAccelerationY = profileCurveY.acceleration(t[0]); 

            //CROSS SECTION CURVE           CROSS SECTION CURVE 
            BSpline crossSectionCurveX = new BSpline(fCtrlCrossSectionCurveX);
            Function fCrossSectionCurvePositionX = crossSectionCurveX.curveValue(t[1])*0.01;
            Function fCrossSectionCurveTangentX = crossSectionCurveX.tangent(t[1]);
            Function fCrossSectionCurveAccelerationX = crossSectionCurveX.acceleration(t[1]);
            
            BSpline crossSectionCurveY = new BSpline(fCtrlCrossSectionCurveY);
            Function fCrossSectionCurvePositionY = crossSectionCurveY.curveValue(t[1])*0.01;
            Function fCrossSectionCurveTangentY = crossSectionCurveY.tangent(t[1]);
            Function fCrossSectionCurveAccelerationY = crossSectionCurveY.acceleration(t[1]);
            
            Function[] fposition = { fCrossSectionCurvePositionX * fProfileCurvePositionX, fCrossSectionCurvePositionY * fProfileCurvePositionX,
                                        0.7 - fProfileCurvePositionY };


            

            Function[] fdu = { fCrossSectionCurvePositionX * fProfileCurveTangentX, fCrossSectionCurvePositionY * fProfileCurveTangentX, 
                                        -fProfileCurveTangentY };
            Function[] fdv = { fCrossSectionCurveTangentX * fProfileCurvePositionX, fCrossSectionCurveTangentY * fProfileCurvePositionX,
                                        0 };
            
            
            Function[] fduu = { fCrossSectionCurvePositionX * fProfileCurveAccelerationX, fCrossSectionCurvePositionY * fProfileCurveAccelerationX,
                                        -fProfileCurveAccelerationY};
            Function[] fdvv = {  fCrossSectionCurveAccelerationX * fProfileCurvePositionX, fCrossSectionCurveAccelerationY * fProfileCurvePositionX,
                                        0 };
            Function fPosition = new Function(fposition);
            inputVars = VM.concatenate<Variable>(t, fCtrlPointsProfileCurveX, fCtrlPointsProfileCurveY, fCtrlCrossSectionCurveX, fCtrlCrossSectionCurveY);
            fPosition.orderVariablesInDomain(inputVars);

            _rtPosition = fPosition.compile();

            Function fDu = new Function(fdu);
            Function fDv = new Function(fdv);
            Function fNormal = cross(fDu, fDv);
            fNormal.orderVariablesInDomain(inputVars);
            _rtNormal = fNormal.compile();
            
            
            /*
            
            fDu.orderVariablesInDomain(inputVars);
            _rtDu = fDu.compile();

            
            inputVars = VM.concatenate<Variable>(t, fCtrlPointsProfileCurveX, fCtrlCrossSectionCurveX, fCtrlCrossSectionCurveY);
            fDv.orderVariablesInDomain(inputVars);
            _rtDv = fDv.compile();

            Function fDuu = new Function(fduu);
            inputVars = VM.concatenate<Variable>(t, fCtrlPointsProfileCurveX, fCtrlPointsProfileCurveY, fCtrlCrossSectionCurveX, fCtrlCrossSectionCurveY);
            fDuu.orderVariablesInDomain(inputVars);
            _rtDuu = fDuu.compile();

            Function fDvv = new Function(fdvv);
            inputVars = VM.concatenate<Variable>(t, fCtrlPointsProfileCurveX, fCtrlCrossSectionCurveX, fCtrlCrossSectionCurveY);
            fDvv.orderVariablesInDomain(inputVars);
            _rtDvv = fDvv.compile();
             * */

            #region PROFILE SHADER
            //PARAMETRICCURVE2D_OUTPUT profileCurve = BSpline_Evaluate(input.x,
            //                                                              0,
            //                                                              BSpline1_ControlPointCount);
            //profileCurve.Position *= .004;

            //PARAMETRICCURVE2D_OUTPUT crossSectionCurve = BSpline_Evaluate(input.y,
            //                                                               BSpline2_ControlPointOffset,
            //                                                               BSpline2_ControlPointCount);
            //crossSectionCurve.Position *= .01;

            //PARAMETRICSURFACE3D_OUTPUT output;

            ////
            //// Compute position
            ////    
            //output.Position.x = crossSectionCurve.Position.x * profileCurve.Position.x;
            //output.Position.y = crossSectionCurve.Position.y * profileCurve.Position.x;
            //output.Position.z = .7 - profileCurve.Position.y;

            ////
            //// Compute 1st-order partials
            ////
            //output.dF_du.x = crossSectionCurve.Position.x * profileCurve.Tangent.x;
            //output.dF_dv.x = crossSectionCurve.Tangent.x * profileCurve.Position.x;

            //output.dF_du.y = crossSectionCurve.Position.y * profileCurve.Tangent.x;
            //output.dF_dv.y = crossSectionCurve.Tangent.y * profileCurve.Position.x;

            //output.dF_du.z = -profileCurve.Tangent.y;
            //output.dF_dv.z = 0;

            ////
            //// Compute 2nd-order partials
            ////
            //output.ddF_duu.x = crossSectionCurve.Position.x * profileCurve.Acceleration.x;
            //output.ddF_dvv.x = crossSectionCurve.Acceleration.x * profileCurve.Position.x;

            //output.ddF_duu.y = crossSectionCurve.Position.y * profileCurve.Acceleration.x;
            //output.ddF_dvv.y = crossSectionCurve.Acceleration.y * profileCurve.Position.x;

            //output.ddF_duu.z = -profileCurve.Acceleration.y;
            //output.ddF_dvv.z = 0;

            //return output;
            #endregion

            //double[] res = new double[3];
            //_rtPosition.eval(res, dInputVars); 

            //Resample(c_triangleCount);
            InitializeVertexBuffer();
            InitializeIndices();
            GenerateMesh();

            _eqCalculated = true;
        }

        private void GenerateMesh()
        {
            int i, j;
            double dt = 1.0 / (_numUSamples-1);
            double dQ = 1.0 / (_numVSamples-1);
            double t=0, theta=0;
            double scaledT, scaledTetha;
            double[] position3D = new double[3]; 
            double[] normal3D = new double[3];

            double[] inputVars;
            double[] profileCX = new double[4];
            double[] profileCY = new double[4];
            double[] crossSectionCX = new double[4];
            double[] crossSectionCY = new double[4];
            double[] domain = new double[2];

            int n = 0;

            //if the curve is closed then the first three control points are copied onto the end of the control point array at a previous point in the processing. A closed curve with 8 control points
            //will be turned into a curve with 11 control points
            int appliedControlPointProfile = _ctrlPointsProfileCurveX.Length ;
            int appliedControlPointCrossSection = _ctrlCrossSectionCurveX.Length ;


            int currentSegmentU = 0;
            int currentSegmentV = 0;

            for (i = 0; i < _numUSamples; i++)
            {
                scaledT = (appliedControlPointProfile - 3) *t;   
                currentSegmentU = (int)scaledT;

                if (scaledT > 1)
                {
                    domain[0] = scaledT - Math.Floor(scaledT);
                }
                else
                {
                    domain[0] = scaledT;
                }
                profileCX[0] = _ctrlPointsProfileCurveX[currentSegmentU % _ctrlPointsProfileCurveX.Length];
                profileCX[1] = _ctrlPointsProfileCurveX[(currentSegmentU + 1) % _ctrlPointsProfileCurveX.Length];
                profileCX[2] = _ctrlPointsProfileCurveX[(currentSegmentU + 2) % _ctrlPointsProfileCurveX.Length];
                profileCX[3] = _ctrlPointsProfileCurveX[(currentSegmentU + 3) % _ctrlPointsProfileCurveX.Length];
                profileCY[0] = _ctrlPointsProfileCurveY[currentSegmentU % _ctrlPointsProfileCurveY.Length];
                profileCY[1] = _ctrlPointsProfileCurveY[(currentSegmentU + 1) % _ctrlPointsProfileCurveY.Length];
                profileCY[2] = _ctrlPointsProfileCurveY[(currentSegmentU + 2) % _ctrlPointsProfileCurveY.Length];
                profileCY[3] = _ctrlPointsProfileCurveY[(currentSegmentU + 3) % _ctrlPointsProfileCurveY.Length];

                for (j = 0; j < _numVSamples; j++)
                {
                    scaledTetha = (appliedControlPointCrossSection -3) * theta;

                    currentSegmentV = (int)scaledTetha;
                    if (scaledTetha > 1)
                    {
                        domain[1] = scaledTetha - Math.Floor(scaledTetha);
                    }
                    else
                    {
                        domain[1] = scaledTetha;
                    }
                    crossSectionCX[0] = _ctrlCrossSectionCurveX[currentSegmentV % _ctrlCrossSectionCurveX.Length];
                    crossSectionCX[1] = _ctrlCrossSectionCurveX[(currentSegmentV + 1) % _ctrlCrossSectionCurveX.Length];
                    crossSectionCX[2] = _ctrlCrossSectionCurveX[(currentSegmentV + 2) % _ctrlCrossSectionCurveX.Length];
                    crossSectionCX[3] = _ctrlCrossSectionCurveX[(currentSegmentV + 3) % _ctrlCrossSectionCurveX.Length];
                    crossSectionCY[0] = _ctrlCrossSectionCurveY[(currentSegmentV) % _ctrlCrossSectionCurveY.Length];
                    crossSectionCY[1] = _ctrlCrossSectionCurveY[(currentSegmentV + 1) % _ctrlCrossSectionCurveY.Length];
                    crossSectionCY[2] = _ctrlCrossSectionCurveY[(currentSegmentV + 2) % _ctrlCrossSectionCurveY.Length];
                    crossSectionCY[3] = _ctrlCrossSectionCurveY[(currentSegmentV + 3) % _ctrlCrossSectionCurveY.Length];



                    inputVars = VM.concatenate<double>(domain, profileCX, profileCY, crossSectionCX, crossSectionCY);
                    _rtPosition.eval(position3D, inputVars);


                    _verticesPositionNormal[n].Position.X = (float)position3D[0];
                    _verticesPositionNormal[n].Position.Y = (float)position3D[1];
                    _verticesPositionNormal[n].Position.Z = (float)position3D[2];

                    _rtNormal.eval(normal3D, inputVars);

                    _verticesPositionNormal[n].Normal.X = (float)normal3D[0];
                    _verticesPositionNormal[n].Normal.Y = (float)normal3D[1];
                    _verticesPositionNormal[n].Normal.Z = (float)normal3D[2];
                    _verticesPositionNormal[n++].Normal.Normalize();
                    
                    theta += dQ;   
                }
                t += dt;
                theta = 0;
            }
           
            /*
            double[] position3D = new double[3];
            double[] normal3D = new double[3];
            double[] inputVars;
            double[] domain = new double[2];
            double[] profileCX = new double[4];
            double[] profileCY = new double[4];
            double[] crossSectionCX = new double[4];
            double[] crossSectionCY = new double[4];
                       
            int n=0;

            for (int t = 0; t < _numVSamples; t++)
            {
                domain[0] = t * 1.0 / (_numVSamples);
                for (int tetha = 0; tetha < _numUSamples; tetha++)
                {
                    domain[1] = tetha * 1.0 / (_numUSamples);

                    // For a closed curve, we act like we append the first 3 points at the end of the list
                    int appliedControlPointCountProfile = _ctrlPointsProfileCurveX.Length + (_isClosedProfileCurve ? 3 : 0);
                    int appliedControlPointCountCrossSection = _ctrlCrossSectionCurveX.Length + (_isClosedCrossSectionCurve ? 3 : 0);

                    double scaledT = (appliedControlPointCountProfile - 3) * domain[0];
                    double scaledTetha = (appliedControlPointCountCrossSection - 3) * domain[1];
                    int currentSegmentU = (int)scaledT;
                    int currentSegmentV = (int)scaledTetha;
                    
                    if (domain[0] == 1)  // The last point on the curve
                    {
                        currentSegmentU--;  // This is the only case when relativeT will be set to 1
                    }
                    if (domain[1] == 1)
                    {
                        currentSegmentV--;
                    }
                    profileCX[0] = _ctrlPointsProfileCurveX[(currentSegmentU) % _ctrlPointsProfileCurveX.Length];
                    profileCX[1] = _ctrlPointsProfileCurveX[(currentSegmentU + 1) % _ctrlPointsProfileCurveX.Length];
                    profileCX[2] = _ctrlPointsProfileCurveX[(currentSegmentU + 2) % _ctrlPointsProfileCurveX.Length];
                    profileCX[3] = _ctrlPointsProfileCurveX[(currentSegmentU + 3) % _ctrlPointsProfileCurveX.Length];
                    profileCY[0] = _ctrlPointsProfileCurveY[(currentSegmentU) % _ctrlPointsProfileCurveY.Length];
                    profileCY[1] = _ctrlPointsProfileCurveY[(currentSegmentU + 1) % _ctrlPointsProfileCurveY.Length];
                    profileCY[2] = _ctrlPointsProfileCurveY[(currentSegmentU + 2) % _ctrlPointsProfileCurveY.Length];
                    profileCY[3] = _ctrlPointsProfileCurveY[(currentSegmentU + 3) % _ctrlPointsProfileCurveY.Length];

                    crossSectionCX[0] = _ctrlCrossSectionCurveX[(currentSegmentV) % _ctrlCrossSectionCurveX.Length];
                    crossSectionCX[1] = _ctrlCrossSectionCurveX[(currentSegmentV + 1) % _ctrlCrossSectionCurveX.Length];
                    crossSectionCX[2] = _ctrlCrossSectionCurveX[(currentSegmentV + 2) % _ctrlCrossSectionCurveX.Length];
                    crossSectionCX[3] = _ctrlCrossSectionCurveX[(currentSegmentV + 3) % _ctrlCrossSectionCurveX.Length];
                    crossSectionCY[0] = _ctrlCrossSectionCurveY[(currentSegmentV) % _ctrlCrossSectionCurveY.Length];
                    crossSectionCY[1] = _ctrlCrossSectionCurveY[(currentSegmentV + 1) % _ctrlCrossSectionCurveY.Length];
                    crossSectionCY[2] = _ctrlCrossSectionCurveY[(currentSegmentV + 2) % _ctrlCrossSectionCurveY.Length];
                    crossSectionCY[3] = _ctrlCrossSectionCurveY[(currentSegmentV + 3) % _ctrlCrossSectionCurveY.Length];

                    inputVars = VM.concatenate<double>(domain, profileCX, profileCY, crossSectionCX, crossSectionCY);
                    _rtPosition.eval(position3D, inputVars);
                    _verticesPositionNormal[n].Position.X = (float)position3D[0];
                    _verticesPositionNormal[n].Position.Y = (float)position3D[1];
                    _verticesPositionNormal[n].Position.Z = (float)position3D[2];

                    _rtPosition.eval(normal3D, inputVars);

                    _verticesPositionNormal[n].Normal.X = (float)normal3D[0];
                    _verticesPositionNormal[n].Normal.Y = (float)normal3D[1];
                    _verticesPositionNormal[n].Normal.Z = (float)normal3D[2];
                    _verticesPositionNormal[n++].Normal.Normalize();
                }
            }

            /*
            // TEST: distance-based domain scaling
            //TEST float lodFactorInverse = 1.0f;
            float lodFactorInverse = Math.Min(Math.Max(3.0f * _distance, 1.0f), 10.0f);
            float lodFactorPow2 = Float.NearestPowerOf2(lodFactorInverse);
            float lodFraction = 2.0f - lodFactorInverse / lodFactorPow2;  // E.g. 2-15/8 yields 1/8 remaining till next power of 2

            _domainScalingFactor = new Vector2(2.0f / _numUSamples, 2.0f / _numVSamples) * lodFactorPow2;


            for (int n = 0; n < _numVertices; n++)
            {
                VertexPosition2D position2D = _verticesPosition2D[n];

                Vector2 inputPoint = position2D.Position * _domainScalingFactor;
                inputPoint = Vector2.Clamp(inputPoint, Vector2.Zero, Vector2.One);

                domain[0] = inputPoint.X;
                domain[1] = inputPoint.Y;


                inputVars = VM.concatenate<double>(domain, _ctrlPointsProfileCurveX, _ctrlPointsProfileCurveY, _ctrlCrossSectionCurveX, _ctrlCrossSectionCurveY);
                _rtPosition.eval(position3D, inputVars);
                _rtNormal.eval(normal3D, inputVars); 

                _verticesPositionNormal[n].Position.X = (float)position3D[0];
                _verticesPositionNormal[n].Position.Y = (float)position3D[1];
                _verticesPositionNormal[n].Position.Z = (float)position3D[2];

                _verticesPositionNormal[n].Normal.X = (float)normal3D[0];
                _verticesPositionNormal[n].Normal.Y = (float)normal3D[1];
                _verticesPositionNormal[n].Normal.Z = (float)normal3D[2];
                //Console.WriteLine(inputPoint.X.ToString() + ", " + inputPoint.Y.ToString());
            }
             * */
            
        }



        public void ApplyWireProductEffect(ParametricCurve2D wireCurve, ParametricCurve2D crossSectionCurve)
        {
            /*
             Add DStar Code
             * */
            //PARAMETRICCURVE2D_OUTPUT wireCurve = BSpline_Evaluate(input.x,
            //                                                           0,
            //                                                           BSpline1_ControlPointCount);
            //wireCurve.Position *= .005;

            //PARAMETRICCURVE2D_OUTPUT crossSectionCurve = BSpline_Evaluate(input.y,
            //                                                               BSpline2_ControlPointOffset,
            //                                                               BSpline2_ControlPointCount);
            //crossSectionCurve.Position *= .0005;

            //float2 normVector = normalize(float2(-wireCurve.Tangent.y, -wireCurve.Tangent.x));

            //PARAMETRICSURFACE3D_OUTPUT output;

            ////
            //// Compute position
            ////    
            //output.Position.x = wireCurve.Position.x + normVector.x * crossSectionCurve.Position.x;
            //output.Position.y = -wireCurve.Position.y + normVector.y * crossSectionCurve.Position.x;
            //output.Position.z = -crossSectionCurve.Position.y;

            ////
            //// Compute 1st-order partials
            ////
            //output.dF_du.x = wireCurve.Tangent.x;  // + d/dU(normVector.X) * crossSectionCurvePoint2D.X;
            //output.dF_dv.x = normVector.x * crossSectionCurve.Tangent.x;

            //output.dF_du.y = -wireCurve.Tangent.y;  // + d/dU(normVector.Y) * crossSectionCurvePoint2D.X;
            //output.dF_dv.y = normVector.y * crossSectionCurve.Tangent.x;

            //output.dF_du.z = 0;
            //output.dF_dv.z = -crossSectionCurve.Tangent.y;

            ////
            //// Compute 2nd-order partials
            ////
            //output.ddF_duu.x = wireCurve.Acceleration.x;  // + d/dU( d/dU(normVector.X) * crossSectionCurvePoint2D.X );
            //output.ddF_dvv.x = normVector.x * crossSectionCurve.Acceleration.x;

            //output.ddF_duu.y = -wireCurve.Acceleration.y;  // + d/DU( d/dU(normVector.Y) * crossSectionCurvePoint2D.X );
            //output.ddF_dvv.y = normVector.y * crossSectionCurve.Acceleration.x;

            //output.ddF_duu.z = 0;
            //output.ddF_dvv.z = -crossSectionCurve.Acceleration.y;

            //return output;
            //GenerateMesh();
        }



        public void ApplyNullEffect()
        {
            _parametricSurfaceEffect = null;
        }


        public float Rotation
        {
            set { _rotation = value; }
        }

        public float Distance
        {
            set { _distance = value; }
        }

        public float LeanAngle
        {
            set { _leanAngle = value; }
        }

        public float AmbientFactor
        {
            set { _materialAmbientFactor = value; }
        }

        public float DiffuseFactor
        {
            set { _materialDiffuseFactor = value; }
        }

        public float SpecularFactor
        {
            set { _materialSpecularFactor = value; }
        }

        public Vector2 DomainScalingFactor
        {
            set { _domainScalingFactor = value; }
        }

        public float DomainLodFraction
        {
            set { _domainLodFraction = value; }
        }

        public Vector4 MaterialColor
        {
            get { return _materialColor; }
            set { _materialColor = value; }
        }

        private void InitializeVertexBuffer()
        {
            _numVertices = (_numUSamples+1) * (_numVSamples+1);
            _verticesPosition2D = new VertexPosition2D[_numVertices];
            _verticesPositionNormal = new VertexPositionNormal[_numVertices];

            int index = 0;
            for (int y = 0; y < _numVSamples; y++)
            {
                for (int x = 0; x <= _numUSamples; x++)
                {
                    float fx = .5f * (float)x;
                    float fy = .5f * (float)(_numVSamples - y);
                    _verticesPosition2D[index++].Position = new Vector2(fx, fy);  // Left-right, top-down vertex order
                }
            }

            //subtle incompatibility between D* version and non D* version. non D* version has numSamples+1 samples along the u,v directions, and D* version has numSamples, one less. 
            //In generate mesh this causes various scaling parameters to be 1 less than expected. 
            _vertexBufferPositionNormal = new VertexBuffer(_device, typeof(VertexPositionNormal), _numVertices,
                                 BufferUsage.WriteOnly);
        }

        // Using triangle strips
        private void InitializeIndices()
        {

            //setup indices for TriangleList
            _indices = new short[2 * 3 * (_numVSamples - 1) * (_numUSamples - 1)];
            int cnt = 0;
            for (int j = 0; j < _numVSamples - 1; j++)
            {
                for (int i = 0; i < _numUSamples - 1; i++)
                {
                    int idx = j * _numUSamples + i;
                    if (true)
                    {
                        _indices[cnt++] = (short)idx;
                        _indices[cnt++] = (short)(idx + _numUSamples);
                        _indices[cnt++] = (short)(idx + _numUSamples + 1);

                        _indices[cnt++] = (short)(idx);
                        _indices[cnt++] = (short)(idx + _numUSamples + 1);
                        _indices[cnt++] = (short)(idx + 1);
                    }
                    else
                    {
                        _indices[cnt++] = (short)idx;
                        _indices[cnt++] = (short)(idx + _numUSamples + 1);
                        _indices[cnt++] = (short)(idx + _numUSamples);


                        _indices[cnt++] = (short)idx;
                        _indices[cnt++] = (short)(idx + 1);
                        _indices[cnt++] = (short)(idx + _numUSamples + 1);
                    }
                }
            }
            _numTriangles = cnt - 2;
          
            
           /* 
            _numIndices = _numVSamples * 2 * (_numUSamples+2 );  // Conservative estimate
            Debug.Assert(_numIndices <= EditorForm.c_maxIndexBufferSize);  // Max legal index buffer size
            _indices = new short[_numIndices];

            int index = 0;

            for (int y = 0; y < _numVSamples; y++)
            {
                if (y % 2 == 0)
                {
                    for (int x = 0; x < _numUSamples; x++)
                    {
                        _indices[index++] = (short)((y + 1) * (_numUSamples + 1) + x);
                        _indices[index++] = (short)((y) * (_numUSamples + 1) + x);
                    }

                    // Degenerate point on the tristrip
                    _indices[index++] = (short)((y + 2) * (_numUSamples + 1) - 1);
                    _indices[index++] = (short)((y + 2) * (_numUSamples + 1) - 1);
                }
                else
                {
                    for (int x = _numUSamples; x > 0; x--)
                    {
                        _indices[index++] = (short)((y) * (_numUSamples + 1) + x);
                        _indices[index++] = (short)((y + 1) * (_numUSamples + 1) + x);
                    }

                    // Degenerate points on the tristrip
                    _indices[index++] = (short)((y + 1) * (_numUSamples + 1));
                    _indices[index++] = (short)((y + 2) * (_numUSamples + 1));
                }
            }
            _numTriangles = index - 2;
             */
             
        }


        public bool _isClosedProfileCurve = false;
        public bool _isClosedCrossSectionCurve = true;
        public const float c_singularityTolerance = .01f;

        bool _eqCalculated = false;
        private GraphicsDevice _device;
        private Effect _hardwareEffect;

        private int _numUSamples;  // We have a rectangular numU x numV grid domain
        private int _numVSamples;

        private int _numVertices;
        private int _numIndices;
        private int _numTriangles;

        private VertexPosition2D[] _verticesPosition2D;
        private VertexPositionNormal[] _verticesPositionNormal;
        private VertexBuffer _vertexBufferPositionNormal;
                
        private short[] _indices;


        // Control Points read from the GUI/file 
        private double[] _ctrlPointsProfileCurveX;
        private double[] _ctrlPointsProfileCurveY;
        private double[] _ctrlCrossSectionCurveX;
        private double[] _ctrlCrossSectionCurveY;

        private RuntimeFunction _rtPosition;
        private RuntimeFunction _rtNormal;
        /*
        private RuntimeFunction _rtDu;
        private RuntimeFunction _rtDv;
        private RuntimeFunction _rtDuu;
        private RuntimeFunction _rtDvv;
        */
        private ParametricSurfaceEffect _parametricSurfaceEffect = null;
        private string _strEffect;

        private float _rotation = 0.0f;
        private float _distance = 1.0f;
        private float _leanAngle = 0.0f;
        private Vector2 _domainScalingFactor = new Vector2(1.0f, 1.0f);
        private float _domainLodFraction = 1.0f;

        private float _materialAmbientFactor = .05f;
        private float _materialDiffuseFactor = .90f;
        private float _materialSpecularFactor = 4.0f;

        private Vector4 _materialColor = new Vector4(1.0f, .27f, .20f, 1.0f);

    }
}
