//+-----------------------------------------------------------------------------
//
// ParametricSurfaceEffect1 class.  A test implementation of a custom
// parametric surface (foam packaging pad) not based on B-Splines.
//
//------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

using Float = ParametricModelEditor.FloatingPointMathHelper;


namespace ParametricModelEditor
{
    public class ParametricSurfaceEffect1 : ParametricSurfaceEffect
    {
        protected const float COSTHETA = .9961647f;
        protected const float SINTHETA = .0871557f;

        public ParametricSurfaceEffect1()
        {
        }

        public override void VS_SurfaceFunction(Vector2 input, out Vector3 position,
                                                               out Vector3 dF_du,
                                                               out Vector3 dF_dv,
                                                               out Vector3 ddF_duu,
                                                               out Vector3 ddF_dvv)
        {
            position.X = .667f * input.X;
            position.Y = input.Y;
            position.Z = 0.15f * Float.Sin(PI * 4 * ( COSTHETA * input.X + SINTHETA * input.Y))
                               * Float.Sin(PI * 6 * (-SINTHETA * input.X + COSTHETA * input.Y));
            
            VS_SurfaceFunctionDU(input, out dF_du);
            VS_SurfaceFunctionDV(input, out dF_dv);

            // NOTE: Leaving 2nd-order partials unimplemented
            ddF_duu = ddF_dvv = Vector3.Zero;            
        }

        private void VS_SurfaceFunctionDU(Vector2 input, out Vector3 output)
        {
            output.X = 1;
            output.Y = 0;            
            output.Z = .15f * (
                               ( PI*4*COSTHETA * Float.Cos(PI * 4 * ( COSTHETA*input.X + SINTHETA*input.Y))
                               * Float.Sin(PI * 6 * (-SINTHETA*input.X + COSTHETA*input.Y)) )
                             +
                               ( Float.Sin(PI * 4 * ( COSTHETA*input.X + SINTHETA*input.Y))
                               * PI*6*-SINTHETA * Float.Cos(PI * 6 * (-SINTHETA*input.X + COSTHETA*input.Y)) )
                               );
        }

        private void VS_SurfaceFunctionDV(Vector2 input, out Vector3 output)
        {
            output.X = 0;
            output.Y = 1;            
            output.Z = .15f * (
                               ( PI*4*SINTHETA * Float.Cos(PI * 4 * ( COSTHETA*input.X + SINTHETA*input.Y))
                               * Float.Sin(PI * 6 * (-SINTHETA*input.X + COSTHETA*input.Y)) )
                             +
                               ( Float.Sin(PI * 4 * ( COSTHETA*input.X + SINTHETA*input.Y))
                               * PI*6*COSTHETA * Float.Cos(PI * 6 * (-SINTHETA*input.X + COSTHETA*input.Y)) )
                               );
        }

        public override List<ParametricCurve2D> GetSerialization() { return new List<ParametricCurve2D>(); }
        public override void SetSerialization(List<ParametricCurve2D> list) { }
    }
}
