float4x4 World;
float4x4 WorldViewProjection;
float Ambient;
float LightPower;
float3 LightColor = float3(1,1,1);
float3 LightPos;
float3 LightDirection;

float4 objectSelectedColor = float4(1,1,1,1);
float4 materialColor = float4(1,1,1,1);

float InvMaxDepth;
texture Texture;
bool TextureEnabled;
texture ShadowMap;
texture LightTexture;
float4x4 LightWorldViewProjection;
texture EnvironmentMap;
float4x4 View;
float Reflectance=1;

sampler TextureSampler = sampler_state
{
    Texture = (Texture);
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;    
    AddressU = Wrap;
    AddressV = Wrap;
};

sampler ShadowMapSampler = sampler_state 
{ 
	texture = (ShadowMap) ; 
	magfilter = LINEAR; 
	minfilter = LINEAR; 
	mipfilter = LINEAR; 
	AddressU = clamp; 
	AddressV = clamp;
};

sampler LightSampler = sampler_state
{
	texture = (LightTexture);
	magfilter = LINEAR; 
	minfilter = LINEAR; 
	mipfilter = LINEAR; 
	AddressU = clamp; 
	AddressV = clamp;
};

float DotProduct(float3 LightPos, float3 Pos3D, float3 Normal)
{
    float3 LightDir = normalize(LightPos - Pos3D);
    return dot(LightDir, Normal);
}

// Technique: DrawTexture
struct VS_OUT_DRAW_TEXTURE
{
    float4 Position     : POSITION;
    float2 TexCoords    : TEXCOORD0;
};

struct PS_OUT_DRAW_TEXTURE
{
    float4 Color : COLOR0;
};

VS_OUT_DRAW_TEXTURE DrawTextureVertexShader( float4 inPos : POSITION, float2 inTex : TEXCOORD0 )
{
    VS_OUT_DRAW_TEXTURE Output = (VS_OUT_DRAW_TEXTURE)0;
    Output.Position = inPos;
    Output.TexCoords = inTex;
    return Output;
}

PS_OUT_DRAW_TEXTURE DrawTexturePixelShader(VS_OUT_DRAW_TEXTURE PSIn)
{
    PS_OUT_DRAW_TEXTURE Output = (PS_OUT_DRAW_TEXTURE)0;
	float4 ColorComponent = tex2D(TextureSampler, PSIn.TexCoords);
    Output.Color = ColorComponent;    
    return Output;
}

technique DrawTexture
{
    pass Pass0
	{
        VertexShader = compile vs_3_0 DrawTextureVertexShader();
        PixelShader = compile ps_3_0 DrawTexturePixelShader();
    }
}


//
// technique : Basic
//

struct VS_OUTPUT_BASIC
{
    float4 Position   	: POSITION;    
    float2 TexCoords	: TEXCOORD0;
    float3 Normal		: TEXCOORD1;
    float3 Position3D	: TEXCOORD2;
};

struct PS_OUTPUT_BASIC
{
    float4 Color : COLOR0;
};

VS_OUTPUT_BASIC BasicVertexShader( float4 inPos : POSITION, float3 inNormal: NORMAL, float2 inTexCoords: TEXCOORD0)
{	
	VS_OUTPUT_BASIC Output = (VS_OUTPUT_BASIC)0;
	 
	Output.Position = mul(inPos, WorldViewProjection);	
	Output.TexCoords = inTexCoords;		
	Output.Normal = normalize(mul(normalize(inNormal), (float3x3)World));	 //this is correct one	
	//Output.Normal = normalize(mul(inNormal, (float3x3)World));	
	Output.Position3D = mul(inPos,World);
    
	return Output;    
}

PS_OUTPUT_BASIC BasicPixelShader(VS_OUTPUT_BASIC input) 
{
	PS_OUTPUT_BASIC Output = (PS_OUTPUT_BASIC)0;		
	
	float4 ColorComponent = tex2D(TextureSampler, input.TexCoords);
	float DiffuseLightingFactor = DotProduct(LightPos, input.Position3D, input.Normal);   
	Output.Color = ColorComponent*DiffuseLightingFactor*LightPower; 
	
	return Output;
}

technique Basic
{
	pass Pass0
    {   
    	VertexShader = compile vs_3_0 BasicVertexShader();
        PixelShader  = compile ps_3_0 BasicPixelShader();
    }
}

//
// technique: ShadowMap
//

struct VS_OUTPUT_SMAP
{
    float4 Position     : POSITION;    
    float  Depth		: TEXCOORD0;
};

VS_OUTPUT_SMAP ShadowMapVertexShader(float4 inPos : POSITION)
{
    VS_OUTPUT_SMAP Output = (VS_OUTPUT_SMAP)0;    
    Output.Position = mul(inPos, LightWorldViewProjection);
    Output.Depth = Output.Position.z*InvMaxDepth;
    return Output;
}

struct PS_OUTPUT_SMAP
{
    float4 Color : COLOR0;
};

PS_OUTPUT_SMAP ShadowMapPixelShader(VS_OUTPUT_SMAP input)
{
    PS_OUTPUT_SMAP Output = (PS_OUTPUT_SMAP)0;
    Output.Color = input.Depth;
    return Output;
}

technique ShadowMap
{
	pass Pass1
	{
		VertexShader = compile vs_3_0 ShadowMapVertexShader();
		PixelShader = compile ps_3_0 ShadowMapPixelShader();
	}
}


//
// technique: ShadowedScene
//

struct VS_INPUT_SSCENE
{
	float4 Position : POSITION;
	float2 TexCoords : TEXCOORD0;
	float3 Normal : NORMAL;
	
};

struct VS_OUTPUT_SSCENE
{
    float4 Position             : POSITION;
    float4 PositionLightSpace   : TEXCOORD0;    
    float4 RealDepth			: TEXCOORD1; 
	float2 TexCoords            : TEXCOORD2;
	float3 Normal				: TEXCOORD3;
	float3 Position3D           : TEXCOORD4;
};

VS_OUTPUT_SSCENE ShadowedSceneVertexShader( VS_INPUT_SSCENE input )
{
    VS_OUTPUT_SSCENE Output = (VS_OUTPUT_SSCENE)0;

    Output.Position = mul(input.Position, WorldViewProjection);
    Output.PositionLightSpace = mul(input.Position, LightWorldViewProjection);
    Output.RealDepth = Output.PositionLightSpace.z*InvMaxDepth;
    Output.Normal = normalize(mul(input.Normal, (float3x3)World));
    Output.Position3D = mul(input.Position, World);
    	
    Output.TexCoords = input.TexCoords;
    if (!TextureEnabled)
        Output.TexCoords = (0,0);

    return Output;
}

struct SScenePixelToFrame
{
    float4 Color : COLOR0;
};


SScenePixelToFrame ShadowedScenePixelShader(VS_OUTPUT_SSCENE input)
{
    SScenePixelToFrame Output = (SScenePixelToFrame)0;

    float2 ProjectedTexCoords;
    ProjectedTexCoords[0] = input.PositionLightSpace.x/input.PositionLightSpace.w/2.0f +0.5f;
    ProjectedTexCoords[1] = -input.PositionLightSpace.y/input.PositionLightSpace.w/2.0f +0.5f;
    float4 ColorComponent = tex2D(TextureSampler, input.TexCoords)*objectSelectedColor * materialColor;
  
    if ((saturate(ProjectedTexCoords.x) == ProjectedTexCoords.x) && (saturate(ProjectedTexCoords.y) == ProjectedTexCoords.y))
    {
        float StoredDepthInShadowMap = tex2D(ShadowMapSampler, ProjectedTexCoords).x;
        if ((input.RealDepth.x - 1.0f/100.0f) <= StoredDepthInShadowMap)
        {			
            float DiffuseLightingFactor = DotProduct(LightPos, input.Position3D, input.Normal);             
            float LightTextureFactor = tex2D(LightSampler, ProjectedTexCoords).r;

            Output.Color = ColorComponent*LightTextureFactor*DiffuseLightingFactor*LightPower;
        }
    }

	//ambient color
	Output.Color += ColorComponent*Ambient;

    return Output;
}

technique ShadowedScene
{
    pass Pass0
    {
        VertexShader = compile vs_3_0 ShadowedSceneVertexShader();
        PixelShader = compile ps_3_0 ShadowedScenePixelShader();
    }
}


//
// technique: ShadowedEnvMapScene
//

struct VS_INPUT_SHADOWED_ENVMAP_SCENE
{
	float4 Position : POSITION;
	float2 TexCoords : TEXCOORD0;
	float3 Normal : NORMAL;
};

struct VS_OUTPUT_SHADOWED_ENVMAP_SCENE
{
    float4 Position             : POSITION;
    float4 PositionLightSpace   : TEXCOORD0;    
    float4 RealDepth			: TEXCOORD1; 
	float2 TexCoords            : TEXCOORD2;
	float3 Normal				: TEXCOORD3;
	float3 Position3D           : TEXCOORD4;
	float3 Reflection : TEXCOORD5;
    float3 Fresnel : COLOR0;
    float3 Lighting : COLOR1;
};

VS_OUTPUT_SHADOWED_ENVMAP_SCENE ShadowedEnvMapSceneVertexShader( VS_INPUT_SHADOWED_ENVMAP_SCENE input )
{
    VS_OUTPUT_SHADOWED_ENVMAP_SCENE Output = (VS_OUTPUT_SHADOWED_ENVMAP_SCENE)0;

    Output.Position = mul(input.Position, WorldViewProjection);
    Output.PositionLightSpace = mul(input.Position, LightWorldViewProjection);
    Output.RealDepth = Output.PositionLightSpace.z*InvMaxDepth;
    Output.Normal = normalize(mul(input.Normal, (float3x3)World));
    Output.Position3D = mul(input.Position, World);
    	
    Output.TexCoords = input.TexCoords;
    //if (!TextureEnabled) Output.TexCoords = (0,0);        
   
    // Compute a reflection vector for the environment map.
    float3 eyePosition = mul(-View._m30_m31_m32, transpose(View));
    float3 viewVector = Output.Position3D - eyePosition;
    Output.Reflection = reflect(viewVector, Output.Normal);

    // Approximate a Fresnel coefficient for the environment map.
    // This makes the surface less reflective when you are looking
    // straight at it, and more reflective when it is viewed edge-on.
    Output.Fresnel = saturate(Reflectance + dot(normalize(viewVector), Output.Normal));
    
    // Compute lighting.
    float lightAmount = max(dot(Output.Normal, LightDirection), 0);
    Output.Lighting = LightPower * lightAmount * LightColor;

    return Output;
}

sampler EnvironmentMapSampler = sampler_state
{
    Texture = (EnvironmentMap);

    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
    
    AddressU = Wrap;
    AddressV = Wrap;
};

SScenePixelToFrame ShadowedEnvMapScenePixelShader(VS_OUTPUT_SHADOWED_ENVMAP_SCENE input)
{
    SScenePixelToFrame Output = (SScenePixelToFrame)0;

    float2 ProjectedTexCoords;
    ProjectedTexCoords[0] = input.PositionLightSpace.x/input.PositionLightSpace.w/2.0f +0.5f;
    ProjectedTexCoords[1] = -input.PositionLightSpace.y/input.PositionLightSpace.w/2.0f +0.5f;
    float3 ColorComponent = tex2D(TextureSampler, input.TexCoords);
    
    if ((saturate(ProjectedTexCoords.x) == ProjectedTexCoords.x) && (saturate(ProjectedTexCoords.y) == ProjectedTexCoords.y))
    {
        float StoredDepthInShadowMap = tex2D(ShadowMapSampler, ProjectedTexCoords).x;
        if ((input.RealDepth.x - 1.0f/1000.0f) <= StoredDepthInShadowMap)
        {			
            float DiffuseLightingFactor = DotProduct(LightPos, input.Position3D, input.Normal);     
            float LightTextureFactor = tex2D(LightSampler, ProjectedTexCoords).r;        
            
            float3 envmap = texCUBE(EnvironmentMapSampler, input.Reflection);
            // Use the Fresnel coefficient to interpolate between texture and environment map.
			ColorComponent = lerp(ColorComponent, envmap, input.Fresnel);
			//ColorComponent = envmap;
    
            Output.Color = float4(LightTextureFactor*ColorComponent*DiffuseLightingFactor*LightPower,1);
        }
    }

	//ambient color
	Output.Color += float4(ColorComponent*Ambient,1);

    return Output;
}

technique ShadowedEnvMapScene
{
    pass Pass0
    {
        VertexShader = compile vs_3_0 ShadowedEnvMapSceneVertexShader();
        PixelShader = compile ps_3_0 ShadowedEnvMapScenePixelShader();
    }
}

//
// technique : Colored
//

struct VS_OUTPUT_COLORED
{
    float4 Position   	: POSITION;    
    float4 Color		: COLOR0;
};

struct PS_OUTPUT_COLORED
{
    float4 Color : COLOR0;
};

VS_OUTPUT_COLORED ColoredVertexShader( float4 inPos : POSITION, float4 inColor : COLOR )
{	
	VS_OUTPUT_COLORED Output = (VS_OUTPUT_COLORED)0;
	 
	Output.Position = mul(inPos, WorldViewProjection);	
	Output.Color = inColor;
    
	return Output;    
}

PS_OUTPUT_COLORED ColoredPixelShader(VS_OUTPUT_COLORED input) 
{
	PS_OUTPUT_COLORED Output = (PS_OUTPUT_COLORED)0;		
	
	Output.Color = input.Color; 
	
	return Output;
}

technique Colored
{
	pass Pass0
    {   
    	VertexShader = compile vs_3_0 ColoredVertexShader();
        PixelShader  = compile ps_3_0 ColoredPixelShader();
    }
}