#region File Description
//-----------------------------------------------------------------------------
// MainForm.cs
//
// Microsoft XNA Community Game Platform
// Copyright (C) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------
#endregion

#region Using Statements
using System.Windows.Forms;
using System;
using System.Diagnostics;
using System.Timers;
using System.Collections.Generic;
using System.ComponentModel;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using DifferentiableFunction;
using Dynamics;
using utilities;
using matrix;
#endregion

namespace WinFormsGraphicsDevice
{
    // System.Drawing and the XNA Framework both define Color types.
    // To avoid conflicts, we define shortcut names for them both.
    using GdiColor = System.Drawing.Color;
    using XnaColor = Microsoft.Xna.Framework.Graphics.Color;
    /// <summary>
    /// Custom form provides the main user interface for the program.
    /// In this sample we used the designer to add a splitter pane to the form,
    /// which contains a SpriteFontControl and a SpinningTriangleControl.
    /// </summary>
    public partial class MainForm : Form
    {
        
        //private cRigidBodyUI lastBodySelected;
        private cRigidBodyUI sourceBody;
        private newFileDialogBox newFileDialog = new newFileDialogBox();
        public string RBSFileName; // Rigid Body System name
        private int newFileFlag = 0;

        private int numRB = 0;
        
        cRigidBodySystemIO rigidBodySystemIO = new cRigidBodySystemIO("WinFormsGraphicsDevice");        // RBS reader/writer
        iRigidBodySystemFunctions rb;

        private fmProgress m_fmProgress = null;
        

        public MainForm()
        {
            InitializeComponent();
 
            this.imageList = new System.Windows.Forms.ImageList();
            //
            // ImageList to be used with Treeview control
            //
            this.imageList.Images.Add(System.Drawing.Image.FromFile("..\\..\\..\\Content\\red.bmp"));
            this.imageList.Images.Add(System.Drawing.Image.FromFile("..\\..\\..\\Content\\green.bmp"));

            this.rigidBodyHierarchy.ImageList = imageList;
            
            this.openFileDialog.Filter = "Rigid Body System Files(*.cs)|*.cs|All Files(*.*)|*.*";
            this.openFileDialog.AddExtension = true;
            this.saveFileDialog.Filter = "Rigid Body System Files(*.cs)|*.cs|All Files(*.*)|*.*";
            this.saveFileDialog.AddExtension = true;

            
            backgroundWorker1.WorkerReportsProgress = true;
            backgroundWorker2.WorkerReportsProgress = true;

            //backgroundWorker1.WorkerSupportsCancellation = true;

            //backgroundWorker1.RunWorkerAsync((TreeView)rigidBodyHierarchy);

            
        }

        private void mainMenu_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
       
            MenuStrip menu = (MenuStrip)sender;
            if (menu.Name == "mainMenu")
            {
                
            }
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void treeView_AfterSelect(object sender, TreeViewEventArgs e)
        {
            TreeView treeView = (TreeView)sender;
            //cRigidBodyUI rbUIselected = null;
            if (treeView.Name == rigidBodyHierarchy.Name)
            {
                //Console.WriteLine();
                if (e.Node.Name != RBSFileName)
                {

                    propertyGrid1.SelectedObject = e.Node;
                    
                   // propertyGrid1.
                }
               
            }
        }

        private void treeView_AfterLabelEdit(object sender, NodeLabelEditEventArgs e)
        {
            TreeView treeView = (TreeView)sender;
            if (treeView.Name == rigidBodyHierarchy.Name)
            {

            }

        }

        private void rigidBodyHierarchy_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            TreeView treeView = (TreeView)sender;
            if (treeView.Name == rigidBodyHierarchy.Name)
            {/*
                Console.WriteLine ("rigidBodyHierarchy_NodeMouseClick " + e.Node.Name);
                if(rigidBodyHierarchy.Nodes[0].Name!= e.Node.Name)
                    lastBodySelected = (cRigidBodyUI)e.Node;
                else
                    lastBodySelected = null;
                propertyGrid1.ExpandAllGridItems();*/
                
            }

        }

     
        private void rigidBodyHierarchy_NodeMouseDoubleClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            TreeView treeView = (TreeView)sender;
            if (treeView.Name == rigidBodyHierarchy.Name)
            {
                //e.Node.BeginEdit();
                /*
                if (e.Node.Index == 0)
                    lastBodySelected = (cRigidBodyUI)e.Node;*/
            }
        }
        ///<summary>
        /// The ItemDrag event is called when the item drag begins. Here is
        /// where you can perform any tracking, or validate if the drag
        /// operation should occur, and so on.
        ///</summary>
        private void rigidBodyHierarchy_ItemDrag(object sender, ItemDragEventArgs e)
        {
            sourceBody = (cRigidBodyUI)e.Item;
            rigidBodyHierarchy.SelectedNode = sourceBody;
            DoDragDrop(e.Item.ToString(), DragDropEffects.Move | DragDropEffects.Copy);
            
        }

        ///<summary>
        /// Define the event that occurs while the dragging happens
        ///</summary>
        private void rigidBodyHierarchy_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.Text))
                e.Effect = DragDropEffects.Move;
            else
                e.Effect = DragDropEffects.None;
        }

        
        ///<summary>
        /// Determine if target node is inside source node 
        /// </summary>
        
        private int isChild(TreeNode source, TreeNode target)
        {
            if (source == null)
                return 2;
            if (source.Name == target.Name)
                return 1;
            else
            {
                for (int i = 0; i < source.Nodes.Count; i++)
                    if (isChild(source.Nodes[i], target)==1)
                        return 1;
            }
            return 0;
            
        }

        ///<summary>
        /// Determine what node in the tree we are dropping on to (target),
        /// copy the drag source (sourceNode), make the new node and delete
        /// the old one.
        /// </summary>

        private void rigidBodyHierarchy_DragDrop(object sender, DragEventArgs e)
        {
            System.Drawing.Point pos = rigidBodyHierarchy.PointToClient(new System.Drawing.Point(e.X, e.Y));
            cRigidBodyUI targetBody = (cRigidBodyUI)rigidBodyHierarchy.GetNodeAt(pos);
            cRigidBodyUI bodyCopy = null;
            cRigidBodyUI[] children = null;

            //Object p =  e.Data.GetData(typeof(TreeNode));

            if (targetBody != null && targetBody.Name != sourceBody.Name && (isChild(sourceBody, targetBody)==0))
            {
                    if (sourceBody.Nodes.Count > 0)
                    {
                        children = new cRigidBodyUI[sourceBody.Nodes.Count];
                        for (int i = 0; i < sourceBody.Nodes.Count; ++i)
                        {
                            children[i] = (cRigidBodyUI)sourceBody.Nodes[i];
                        }
                        sourceBody.Nodes.Clear();
                    }
                    try
                    {
                        if (children != null)
                        {
                            bodyCopy = new cRigidBodyUI(sourceBody.Text, sourceBody.ImageIndex, sourceBody.SelectedImageIndex, children);
                            
                        }
                        else
                            bodyCopy = new cRigidBodyUI(sourceBody.Text, sourceBody.ImageIndex, sourceBody.SelectedImageIndex);
                        
                        bodyCopy.setGUIValues(sourceBody);
                    }
                    catch (System.Exception ex)
                    {
                        Console.WriteLine("{0} Exception caught.", ex);
                    };
                    try
                    {
                        targetBody.Nodes.Insert(targetBody.Nodes.Count + 1, bodyCopy);
                    }
                    catch (System.Exception ex)
                    {
                        Console.WriteLine("{0} Exception caught.", ex);
                    };
                sourceBody.Remove();
                rigidBodyHierarchy.Invalidate();
                rigidBodyHierarchy.ExpandAll();
                //backgroundWorker1.RunWorkerAsync((TreeView)rigidBodyHierarchy);
                m_fmProgress = new fmProgress();
                backgroundWorker1.RunWorkerAsync((TreeView)rigidBodyHierarchy);
                m_fmProgress.ShowDialog(this);
                m_fmProgress = null;

//                windowsFormXNA1.initializeRBSystem(rigidBodyHierarchy);
                windowsFormXNA1.setFocusRigidBody(rigidBodyHierarchy.SelectedNode.Name);
                

            }
        }

        private void rigidBodyHierarchy_DragOver(object sender, DragEventArgs e)
        {

        }

        private void saveYesNoDialog()
        {
            string fileName;
            MessageBoxButtons buttons = MessageBoxButtons.YesNoCancel;
            string message = "Do you want to save changes?";

            DialogResult result = MessageBox.Show(message, "RBS", buttons);
            if (result == DialogResult.Yes)
            {
                if (this.saveFileDialog.ShowDialog(this) == DialogResult.OK)
                {
                    //this.saveFileDialog.FileName = rigidBodyHierarchy.Nodes[0].Name + ".cs";
                    fileName = this.saveFileDialog.FileName.ToString();
                    if (rigidBodyHierarchy.Nodes.Count > 0)
                    {
                        if (rigidBodyHierarchy.Nodes[0].Nodes.Count > 0)
                        {
                            rigidBodySystemIO.save(rigidBodyHierarchy, windowsFormXNA1.GraphicsDevice, windowsFormXNA1.textureMaps, fileName);
                        }
                        else
                        {
                            MessageBox.Show("The system should have at least one body");
                        }
                    }
                    else
                    {
                        MessageBox.Show("The system should have at least one body");
                    }

                }
            }
            else if (result == DialogResult.No)
            {
                this.newFileDialog.clearTextBoxText();
                this.newFileDialog.setTextFocus();
                this.newFileDialog.ShowDialog();

                if (newFileDialog.okClicked == 1)
                {
                    rigidBodyHierarchy.Nodes.Clear();
                    RBSFileName = newFileDialog.getTextBoxText();
                    createRBSTreeView();
                    newFileDialog.okClicked = 0;
                    newFileFlag = 1;
                    addBody.Enabled = true;
                    deleteBody.Enabled = true;
                    rigidBodyHierarchy.Invalidate();
                    rigidBodyHierarchy.ExpandAll();
                    if (windowsFormXNA1.rigidBodySystemUI != null)
                        windowsFormXNA1.rigidBodySystemUI = null;
                    playSimToolStripButton.Enabled = false;
                    pauseSimToolStripButton.Enabled = false;
                }
            }
        }

        private void toolBar_Click(object sender, EventArgs e)
        {
            ToolStripButton item = (ToolStripButton)sender;
            string p;
            string fileName,fileName2, rbName;
            if (item.Name == newToolStripButton.Name)
            {
                if (newFileFlag == 0)
                {
                    this.newFileDialog.ShowDialog();
                    this.newFileDialog.setTextFocus();
                    if (newFileDialog.okClicked == 1)
                    {
                        rigidBodyHierarchy.Nodes.Clear();
                        RBSFileName = newFileDialog.getTextBoxText();
                        createRBSTreeView();
                        newFileDialog.okClicked = 0;
                        newFileFlag = 1;
                        rigidBodyHierarchy.Invalidate();
                        rigidBodyHierarchy.ExpandAll();
                        addBody.Enabled = true;
                        deleteBody.Enabled = true;
                        if (windowsFormXNA1.rigidBodySystemUI != null)
                            windowsFormXNA1.rigidBodySystemUI = null;
                    }
                }
                else
                {
                    saveYesNoDialog();                    
                }

            }
            else if (item.Name == openToolStripButton.Name)
            {
                if (newFileFlag == 0)
                {
                    if (this.openFileDialog.ShowDialog(this) == DialogResult.OK)
                    {
                        if ((rb = rigidBodySystemIO.load(this.openFileDialog.FileName.ToString(), out fileName)) != null)
                        {
/*
                            fileName2 = System.IO.Path.GetFileName (this.openFileDialog.FileName.ToString()); 
                            fileName = fileName2.Substring(0, fileName2.LastIndexOf(".cs"));
*/
                            rigidBodyHierarchy.Nodes.Clear();
                            rb.populateTreeView(rigidBodyHierarchy, fileName);
                            RBSFileName = fileName;
                            //windowsFormXNA1.initializeRBSystem(rigidBodyHierarchy);
                            m_fmProgress = new fmProgress();
                            backgroundWorker1.RunWorkerAsync((TreeView)rigidBodyHierarchy);
                            m_fmProgress.ShowDialog(this);
                            m_fmProgress = null;
                            rigidBodyHierarchy.Name.ToString();
                            rigidBodyHierarchy.Invalidate();
                            rigidBodyHierarchy.ExpandAll();
                            addBody.Enabled = true;
                            deleteBody.Enabled = true;
                            playSimToolStripButton.Enabled = false;
                            pauseSimToolStripButton.Enabled = false;
                            

                        }
                    }
                }
                else
                {
                    
                    MessageBoxButtons buttons = MessageBoxButtons.YesNoCancel;
                    string message = "Do you want to save changes?";

                    DialogResult result = MessageBox.Show(message, "RBS", buttons);
                    if (result == DialogResult.Yes)
                    {
                        if (this.saveFileDialog.ShowDialog(this) == DialogResult.OK)
                        {
                            //this.saveFileDialog.FileName = rigidBodyHierarchy.Nodes[0].Name + ".cs";
                            fileName = this.saveFileDialog.FileName.ToString();
                            if (rigidBodyHierarchy.Nodes.Count > 0)
                            {
                                if (rigidBodyHierarchy.Nodes[0].Nodes.Count > 0)
                                {
                                    rigidBodySystemIO.save(rigidBodyHierarchy, windowsFormXNA1.GraphicsDevice, windowsFormXNA1.textureMaps, fileName);
                                }
                                else
                                {
                                    MessageBox.Show("The system should have at least one body");
                                }
                            }
                            else
                            {
                                MessageBox.Show("The system should have at least one body");
                            }

                        }
                    }
                    else if (result == DialogResult.No)
                    {
                        if (this.openFileDialog.ShowDialog(this) == DialogResult.OK)
                        {
                            if ((rb = rigidBodySystemIO.load(this.openFileDialog.FileName.ToString(), out fileName)) != null)
                            {

                                rigidBodyHierarchy.Nodes.Clear();
                                rb.populateTreeView(rigidBodyHierarchy, fileName);
                                RBSFileName = fileName;
                                m_fmProgress = new fmProgress();
                                backgroundWorker1.RunWorkerAsync((TreeView)rigidBodyHierarchy);
                                m_fmProgress.ShowDialog(this);
                                m_fmProgress = null;
                                rigidBodyHierarchy.Invalidate();
                                rigidBodyHierarchy.ExpandAll();
                                addBody.Enabled = true;
                                deleteBody.Enabled = true;
                                playSimToolStripButton.Enabled = false;
                                pauseSimToolStripButton.Enabled = false;


                            }
                        }                     
                    }                    
                }
            }
            else if (item.Name == saveToolStripButton.Name)
            {
                
                if (this.saveFileDialog.ShowDialog(this) == DialogResult.OK)
                {
                   // saveFileDialog.FileName = rigidBodyHierarchy.Nodes[0].Name + ".cs";
                    fileName = this.saveFileDialog.FileName.ToString();
                    if (rigidBodyHierarchy.Nodes.Count > 0)
                    {
                        if (rigidBodyHierarchy.Nodes[0].Nodes.Count > 0)
                        {
                            rigidBodySystemIO.save(rigidBodyHierarchy, windowsFormXNA1.GraphicsDevice, windowsFormXNA1.textureMaps, fileName);
                        }
                        else
                        {
                            MessageBox.Show("The system should have at least one body");
                        }
                    }
                    else
                    {
                        MessageBox.Show("The system should have at least one body");
                    }
                }
            }
            else if (item.Name == axisStripButton.Name)
            {
                if (rigidBodyHierarchy.Nodes.Count > 0)
                {
                    if (rigidBodyHierarchy.Nodes[0].Nodes.Count > 0)
                    {
                        windowsFormXNA1.displayAxis(rigidBodyHierarchy.SelectedNode.Name);
                    }
                }
                else
                {
                    MessageBox.Show("The system should have at least one body");
                }
            }
            else if (item.Name == reloadSimToolStripButton.Name)
            {
                if (rigidBodyHierarchy.Nodes.Count > 0)
                {
                    if (rigidBodyHierarchy.Nodes[0].Nodes.Count > 0)
                    {
                        
                        
                        m_fmProgress = new fmProgress();
                        backgroundWorker2.RunWorkerAsync((TreeView)rigidBodyHierarchy);
                        m_fmProgress.ShowDialog(this);
                        m_fmProgress = null;
                        playSimToolStripButton.Enabled = true;
                        pauseSimToolStripButton.Enabled = true;
                    }
                }
                else
                {
                    MessageBox.Show("The system should have at least one body");
                }
                //pauseSimToolStripButton.Checked = false;
                //playSimToolStripButton.Checked = false;
            }
            else if (item.Name == playSimToolStripButton.Name)
            {
                windowsFormXNA1.setSimulationState((int)WindowsFormXNA.simState.PLAY);
                //pauseSimToolStripButton.Checked = false;
            }
            else if (item.Name == pauseSimToolStripButton.Name)
            {
                windowsFormXNA1.setSimulationState((int)WindowsFormXNA.simState.PAUSE);
                //playSimToolStripButton.Checked = false;
            }
        }
        private void menuItem_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem item = (ToolStripMenuItem)sender;
            string p;
            string fileName, fileName2;
            if (item.Name == newFile.Name)
            {
                if (newFileFlag==0)
                {
                    this.newFileDialog.ShowDialog();
                    this.newFileDialog.setTextFocus();
                    if (newFileDialog.okClicked == 1)
                    {
                        rigidBodyHierarchy.Nodes.Clear();
                        RBSFileName = newFileDialog.getTextBoxText();
                        createRBSTreeView();
                        newFileDialog.okClicked = 0;
                        newFileFlag = 1;
                        addBody.Enabled = true;
                        deleteBody.Enabled = true;
                        rigidBodyHierarchy.Invalidate();
                        rigidBodyHierarchy.ExpandAll();
                        if (windowsFormXNA1.rigidBodySystemUI != null)
                            windowsFormXNA1.rigidBodySystemUI = null;
                        playSimToolStripButton.Enabled = false;
                        pauseSimToolStripButton.Enabled = false;
                    }
                }
                else
                {
                    saveYesNoDialog();
                }
            }
            else if (item.Name == openFile.Name)
            {
                if (this.openFileDialog.ShowDialog(this) == DialogResult.OK)
                {
                    if ((rb = rigidBodySystemIO.load(this.openFileDialog.FileName.ToString(), out fileName)) != null)
                    {
                        rigidBodyHierarchy.Nodes.Clear();
                        rb.populateTreeView(rigidBodyHierarchy, fileName);
                        RBSFileName = fileName;
                        m_fmProgress = new fmProgress();
                        backgroundWorker1.RunWorkerAsync((TreeView)rigidBodyHierarchy);
                        m_fmProgress.ShowDialog(this);
                        m_fmProgress = null;

                        rigidBodyHierarchy.Invalidate();
                        rigidBodyHierarchy.ExpandAll();
                        addBody.Enabled = true;
                        deleteBody.Enabled = true;
                        playSimToolStripButton.Enabled = false;
                        pauseSimToolStripButton.Enabled = false;

                    }
                }

            }
            else if (item.Name == saveFile.Name)
            {
                if (this.saveFileDialog.ShowDialog(this) == DialogResult.OK)
                {
                    //this.saveFileDialog.FileName = rigidBodyHierarchy.Nodes[0].Name + ".cs";
                    fileName = this.saveFileDialog.FileName.ToString();
                    if (rigidBodyHierarchy.Nodes.Count > 0)
                    {
                        if (rigidBodyHierarchy.Nodes[0].Nodes.Count > 0)
                        {
                            rigidBodySystemIO.save(rigidBodyHierarchy, windowsFormXNA1.GraphicsDevice, windowsFormXNA1.textureMaps, fileName);
                        }
                    }
                    else
                    {
                        MessageBox.Show("The system should have at least one body");
                    }

                }

            }
            else if (item.Name == exitFile.Name)
            {
                Close();
            }

        }
        /// <summary>
        /// Creates a hierarchical view for a new Rigid Body System
        /// </summary>
        private void createRBSTreeView()
        {
            TreeNode root = new TreeNode(RBSFileName);
            root.ImageIndex = 0;
            root.Name = RBSFileName;
            rigidBodyHierarchy.Nodes.Insert(0, root);

            
        }

        private void button_Click(object sender, EventArgs e)
        {
            
            Button button = (Button)sender;
            if (button.Name == addBody.Name)
            {
                string str; 
                cRigidBodyUI body = new cRigidBodyUI();
                body.ImageIndex = 1;
                
                if (rigidBodyHierarchy.Nodes[0].Nodes.Count == 0)
                {
                    str = "rootBody";
                    body.Text = str;
                    body.Name = str;
                    rigidBodyHierarchy.Nodes[0].Nodes.Insert(rigidBodyHierarchy.Nodes[0].Nodes.Count, body);
                }
                else
                {
                    str = "Body" + numRB;
                    body.Text = str;
                    body.Name = str;
                    rigidBodyHierarchy.Nodes[0].Nodes[0].Nodes.Insert(rigidBodyHierarchy.Nodes[0].Nodes[0].Nodes.Count, body);
                }
                numRB++;
                addBody.Enabled = false;
                m_fmProgress = new fmProgress();
                backgroundWorker1.RunWorkerAsync((TreeView)rigidBodyHierarchy);
                m_fmProgress.ShowDialog(this); 
                m_fmProgress = null;

                addBody.Enabled = true;
                rigidBodyHierarchy.Invalidate();
                rigidBodyHierarchy.ExpandAll();
                playSimToolStripButton.Enabled = false;
                pauseSimToolStripButton.Enabled = false;
                
            }
            else if (button.Name == deleteBody.Name)
            {
                rigidBodyHierarchy.SelectedNode.Remove();
                m_fmProgress = new fmProgress();
                backgroundWorker1.RunWorkerAsync((TreeView)rigidBodyHierarchy);
                m_fmProgress.ShowDialog(this);
                m_fmProgress = null;
            }
        }

        private void propertyGrid_Click(object sender, EventArgs e)
        {
            
            PropertyGrid propertyGrid = (PropertyGrid)sender;
            if (propertyGrid.Name == propertyGrid1.Name)
            {
                
                MessageBox.Show("1 You are in the PropertyGrid.SelectedObjectsChanged event.");
            }
        }

        private void propertyGrid_PropertyValueChanged(Object sender, PropertyValueChangedEventArgs e)
        {
            PropertyGrid propertyGrid = (PropertyGrid)sender;

            object obj = this.propertyGrid1.ActiveControl;
            if (propertyGrid.Name == propertyGrid1.Name)
            {
                
                GridItem item = e.ChangedItem;
                string pasd= item.Label;
                double aux;
                cRigidBodyUI body = (cRigidBodyUI)rigidBodyHierarchy.SelectedNode;
                
                
                //windowsFormXNA1.propertyChangeRBS(rigidBodyHierarchy, eAttributeChange.SELECTION);
                if (item.Label == "X" || item.Label == "Y" || item.Label == "Z")
                {
                    if (e.ChangedItem.Parent.Label.Equals("Position"))
                    {
                        windowsFormXNA1.updatePosition(rigidBodyHierarchy.SelectedNode.Name, new Vector3((float)body.Position.X, (float)body.Position.Y, (float)body.Position.Z));                //windowsFormXNA1.initializeRBSystem(rigidBodyHierarchy);
                        windowsFormXNA1.setSimulationState((int)WindowsFormXNA.simState.RESTART);
                        //pauseSimToolStripButton.Checked = false;
                        //playSimToolStripButton.Checked = false;
                    }
                    else if (e.ChangedItem.Parent.Label.Equals("Orientation"))
                    {
                        windowsFormXNA1.updateOrientation(rigidBodyHierarchy.SelectedNode.Name, new Vector3((float)body.Orientation.X, (float)body.Orientation.Y, (float)body.Orientation.Z));
                        windowsFormXNA1.setSimulationState((int)WindowsFormXNA.simState.RESTART);
                        //pauseSimToolStripButton.Checked = false;
                        //playSimToolStripButton.Checked = false;
                    }
                    else if (e.ChangedItem.Parent.Label.Equals("Forces"))
                    {
                        windowsFormXNA1.updateForces(rigidBodyHierarchy.SelectedNode.Name, new Vector3((float)body.Forces.X, (float)body.Forces.Y, (float)body.Forces.Z));
                        windowsFormXNA1.setSimulationState((int)WindowsFormXNA.simState.RESTART);
                    }
                    else if (e.ChangedItem.Parent.Label.Equals("Torque"))
                    {
                        windowsFormXNA1.updateTorque(rigidBodyHierarchy.SelectedNode.Name, new Vector3((float)body.Torque.X, (float)body.Torque.Y, (float)body.Torque.Z));
                        windowsFormXNA1.setSimulationState((int)WindowsFormXNA.simState.RESTART);
                    }
                    else if (e.ChangedItem.Parent.Label.Equals("CenterOfMass"))
                    {
                        windowsFormXNA1.updateCenterOfMass(rigidBodyHierarchy.SelectedNode.Name, new Vector3((float)body.CenterOfMass.X, (float)body.CenterOfMass.Y, (float)body.CenterOfMass.Z));
                        windowsFormXNA1.setSimulationState((int)WindowsFormXNA.simState.RESTART);
                    }

                }
                else if (item.Label == "Mass")
                {
                    //if ( key.KeyCode== System.Windows.Forms.Keys.A ) 
                    aux = (double)item.Value;
                    windowsFormXNA1.updateMass(rigidBodyHierarchy.SelectedNode.Name, aux);
                }
                else if ((item.Label == "Use_Cylinder" || item.Label == "Final_Angle" || item.Label == "Height" || item.Label == "Initial_Angle" || item.Label == "Radius") && item.Parent.Label == "Cylinder")
                {
                    if (body.Cylinder.Use_Cylinder == true)
                        windowsFormXNA1.updateCylinder(rigidBodyHierarchy.SelectedNode.Name, body.Cylinder.Initial_Angle * Math.PI / 180.0,
                                        body.Cylinder.Final_Angle * Math.PI / 180.0, body.Cylinder.Height, body.Cylinder.Radius);
                }
                else if (item.Label == "Use_sphere" || item.Label == "Radius")
                {
                    if (body.Sphere.Use_sphere == true)
                        windowsFormXNA1.updateSphere(rigidBodyHierarchy.SelectedNode.Name, body.Sphere.Radius);
                }
                else if (item.Label == "Use_Torus" || item.Label == "Inner_Radius" || item.Label == "Outer_Radius")
                {
                    if (body.Torus.Use_Torus == true)
                        windowsFormXNA1.updateTorus(rigidBodyHierarchy.SelectedNode.Name, body.Torus.Inner_Radius, body.Torus.Outer_Radius);
                }
                else if ((item.Label == "File" || item.Label == "Width" || item.Label == "Height") && item.Parent.Label == "ParametricModel")
                {
                    if (body.ParametricModel.Width <= 0)
                    {
                        MessageBox.Show("Width must be non-zero and non-negative", "Warning");
                        body.ParametricModel.Width = 1.0;
                    }
                    if (body.ParametricModel.Height <= 0)
                    {
                        MessageBox.Show("Height must be non-zero and non-negative", "Warning");
                        body.ParametricModel.Height = 1.0;
                    }
                    windowsFormXNA1.updateShape(rigidBodyHierarchy.SelectedNode.Name, body.ParametricModel.File, body.ParametricModel.Width, body.ParametricModel.Height);
                    body.Cylinder.Use_Cylinder = false;
                    body.Sphere.Use_sphere = false;
                    body.Torus.Use_Torus = false;
                }
                
                else if (item.Label == "q0")
                {
                    windowsFormXNA1.updateGenCoords(rigidBodyHierarchy.SelectedNode.Name, body.genCoords.q0, item.Label);
                    windowsFormXNA1.setSimulationState((int)WindowsFormXNA.simState.RESTART);
                }
                else if (item.Label == "q1")
                {
                    windowsFormXNA1.updateGenCoords(rigidBodyHierarchy.SelectedNode.Name, body.genCoords.q1, item.Label);
                    windowsFormXNA1.setSimulationState((int)WindowsFormXNA.simState.RESTART);
                }
                else if (item.Label == "q2")
                {
                    windowsFormXNA1.updateGenCoords(rigidBodyHierarchy.SelectedNode.Name, body.genCoords.q2, item.Label);
                    windowsFormXNA1.setSimulationState((int)WindowsFormXNA.simState.RESTART);
                }
                else if (item.Label == "q3")
                {
                    windowsFormXNA1.updateGenCoords(rigidBodyHierarchy.SelectedNode.Name, body.genCoords.q3, item.Label);
                    windowsFormXNA1.setSimulationState((int)WindowsFormXNA.simState.RESTART);
                }
                else if (item.Label == "q4")
                {
                    windowsFormXNA1.updateGenCoords(rigidBodyHierarchy.SelectedNode.Name, body.genCoords.q4, item.Label);
                    windowsFormXNA1.setSimulationState((int)WindowsFormXNA.simState.RESTART);
                }
                else if (item.Label == "q5")
                {
                    windowsFormXNA1.updateGenCoords(rigidBodyHierarchy.SelectedNode.Name, body.genCoords.q5, item.Label);
                    windowsFormXNA1.setSimulationState((int)WindowsFormXNA.simState.RESTART);
                }
                else if (item.Label == "Name")
                {
                    
                }
                else
                {
                    m_fmProgress = new fmProgress();
                    backgroundWorker1.RunWorkerAsync((TreeView)rigidBodyHierarchy);
                    m_fmProgress.ShowDialog(this);
                    m_fmProgress = null;
                    playSimToolStripButton.Enabled = false;
                    pauseSimToolStripButton.Enabled = false;
                    //windowsFormXNA1.upDatePackedMatricesFunction();
                    /*
                    windowsFormXNA1.initializeRBSystem(rigidBodyHierarchy);
                     * */
                }
            }
        } 

        private void propertyGrid_SelectedObjectsChanged(Object sender, EventArgs e)
        {
            PropertyGrid propertyGrid = (PropertyGrid)sender;
           

            if (propertyGrid.Name == propertyGrid1.Name)
            {
               
                //Console.WriteLine("propertyGrid_SelectedObjectsChanged ");
                //windowsFormXNA1.propertyChangeRBS(rigidBodyHierarchy);
                // propertyGrid.ge
                //MessageBox.Show("2 You are in the PropertyGrid.SelectedObjectsChanged event.");
                windowsFormXNA1.setFocusRigidBody(rigidBodyHierarchy.SelectedNode.Name);
                cRigidBodyUI body = (cRigidBodyUI)rigidBodyHierarchy.SelectedNode;

              //  PropertyDescriptor pd = pdc.Find("PropertyC", false);
               // pdc.Remove(pd); 



               // if (myAttribute.Browsable)
                {
                    // Insert code here.
                }


                //body.genCoords.
                propertyGrid1.Invalidate();
                propertyGrid1.ExpandAllGridItems();
                
            }

            
        }

        private void onLoad(object sender, EventArgs e)
        {

        }

        private void propertyGrid_KeyDown(object sender, KeyEventArgs e)
        {
            PropertyGrid propertyGrid = (PropertyGrid)sender;
        }

       

        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            
            windowsFormXNA1.initializeRBSystem((TreeView) e.Argument);
        }

        private void backgroundWorker2_DoWork(object sender, DoWorkEventArgs e)
        {
            windowsFormXNA1.initializeRBSystem((TreeView)e.Argument);
            windowsFormXNA1.generateDynamics();
        }



        private void backgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            // The background process is complete. First we should hide the
            // modal Progress Form to unlock the UI. The we need to inspect our
            // response to see if an error occured, a cancel was requested or
            // if we completed succesfully.

            // Hide the Progress Form
            if (m_fmProgress != null)
            {
                m_fmProgress.Hide();
                //m_fmProgress = null;
            }

            // Check to see if an error occured in the 
            // background process.
            if (e.Error != null)
            {
                MessageBox.Show(e.Error.Message);
                return;
            }

            // Check to see if the background process was cancelled.
            if (e.Cancelled)
            {
                MessageBox.Show("Processing cancelled.");
                return;
            }

            // Everything completed normally.
            // process the response using e.Result
           // MessageBox.Show("Done!");
        }

      
        
    }
}
