//+-----------------------------------------------------------------------------
//
//  Class FloatingPointMathHelper.  Exposes Math.* functions for float types
//  (instead of double) and adds other useful macros for float types.
//
//------------------------------------------------------------------------------

using System;
using Microsoft.Xna.Framework;

namespace WinFormsGraphicsDevice
{
    static class FloatingPointMathHelper
    {
        public const float Pi = 3.141592654f;
        public const float TwoPi = 6.283185307f;

        /// <summary>
        /// Absolute value of a float
        /// </summary>
        /// <param name="n"></param>
        /// <returns></returns>
        public static float Abs(float n) { return (float)Math.Abs((double)n); }

        /// <summary>
        /// Square a float
        /// </summary>
        /// <param name="n"></param>
        /// <returns></returns>
        public static float Sq(float n) { return n * n; }

        /// <summary>
        /// Square root of a float
        /// </summary>
        /// <param name="n"></param>
        /// <returns></returns>
        public static float Sqrt(float n) { return (float)Math.Sqrt((float)n); }

        /// <summary>
        /// Raise a float to a float power
        /// </summary>
        /// <param name="n"></param>
        /// <param name="exp"></param>
        /// <returns></returns>
        public static float Pow(float n, float exp) { return (float)Math.Pow((double)n, (double)exp); }

        /// <summary>
        /// Arccos function
        /// </summary>
        /// <param name="n"></param>
        /// <returns></returns>
        public static float Acos(float n) { return (float)Math.Acos((double)n); }

        /// <summary>
        /// Sine function
        /// </summary>
        /// <param name="x"></param>
        /// <returns></returns>
        public static float Sin(float x) { return (float)Math.Sin((double)x); }

        /// <summary>
        /// Cosine function
        /// </summary>
        /// <param name="x"></param>
        /// <returns></returns>
        public static float Cos(float x) { return (float)Math.Cos((double)x); }

        /// <summary>
        /// Round function
        /// </summary>
        /// <param name="x"></param>
        /// <param name="decimals"></param>
        /// <returns></returns>
        public static float Round(float x, int decimals) { return (float)Math.Round((double)x, decimals); }

        /// <summary>
        /// Nearest power of 2.  Actually it returns the highest pow2 smaller
        /// </summary>
        /// <param name="x"></param>
        /// <returns></returns>
        public static float NearestPowerOf2(float x)
        {
            int n = (int)Math.Log((double)x, 2.0);
            return (float)Math.Pow(2.0, (double)n);
        }
    }
}
