//+-----------------------------------------------------------------------------
//
//  Class MaterialProperties.  Encapsulates data about the visual appearance of
//  a 3D model.
//
//------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;
using System.Xml.Serialization;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace WinFormsGraphicsDevice
{
    public class MaterialProperties
    {
        public MaterialProperties()
        {
        }

        [XmlElement("Red")]
        public float Red
        {
            get { return _color.X; }
            set { _color.X = value; }
        }
        [XmlElement("Green")]
        public float Green
        {
            get { return _color.Y; }
            set { _color.Y = value; }
        }
        [XmlElement("Blue")]
        public float Blue
        {
            get { return _color.Z; }
            set { _color.Z = value; }
        }

        [XmlElement("Alpha")]
        public float Alpha
        {
            get { return _color.W; }
            set { _color.W = value; }
        }

        internal Vector4 InternalColor
        {
            get { return _color; }
            set { _color = value; }
        }

        [XmlElement("AmbientFactor")]
        public float AmbientFactor
        {
            get { return _ambientFactor; }
            set { _ambientFactor = value; }
        }

        [XmlElement("DiffuseFactor")]
        public float DiffuseFactor
        {
            get { return _diffuseFactor; }
            set { _diffuseFactor = value; }
        }

        [XmlElement("SpecularFactor")]
        public float SpecularFactor
        {
            get { return _specularFactor; }
            set { _specularFactor = value; }
        }

        private Vector4 _color = -Vector4.One;
        private float _ambientFactor = -1.0f;
        private float _diffuseFactor = -1.0f;
        private float _specularFactor = -1.0f;
    }
}
