//+-----------------------------------------------------------------------------
//
//  Class ParametricModel3DData.  An encapsulation of model data primarily used
//  for saving and loading models using XML.
//
//------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;
using System.Xml.Serialization;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace WinFormsGraphicsDevice
{
    [XmlInclude(typeof(BSplineCurve2D))]
    //[XmlInclude(typeof(BSplineCurve2DRealization))]
    [XmlInclude(typeof(Vector4))]
    [XmlRoot("ParametricModel3DData")]
    public class ParametricModel3DData
    {
        public ParametricModel3DData()
        {
            _materialProperties = new MaterialProperties();
        }

        [XmlAttribute("Name")]
        public string Name
        {
            get { return _name; }
            set { _name = value; }
        }

        [XmlAttribute("ParametricEffectType")]
        public ParametricEffectType ParametricEffectType
        {
            get { return _parametricEffectType; }
            set { _parametricEffectType = value; }
        }

        [XmlElement("MaterialProperties")]
        public MaterialProperties MaterialProperties
        {
            get { return _materialProperties; }
            set { _materialProperties = value; }
        }

        [XmlElement("ParametricCurve")]
        public List<ParametricCurve2D> ParametricCurves
        {
            get { return _parametricCurves; }
            set { _parametricCurves = value; }
        }


        private string _name;
        private MaterialProperties _materialProperties;
        private ParametricEffectType _parametricEffectType = ParametricEffectType.None;
        private List<ParametricCurve2D> _parametricCurves = new List<ParametricCurve2D>();
    }
}
