﻿#region Using Statements
using System;
using System.Text;
using System.IO;
using System.Diagnostics;
using System.Timers;
using System.Windows.Forms;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using DifferentiableFunction;
using Dynamics;
using utilities;
using matrix;
using WinFormsGraphicsDevice.Geometries;
using System.CodeDom;
using System.CodeDom.Compiler;

#endregion

namespace WinFormsGraphicsDevice
{
    public interface iRigidBodySystemFunctions
    {
        void populateTreeView(TreeView rigidBodyHierarchy, string RBSFileName);
        RigidBodySystem build(GraphicsDevice device, Texture2D[] textureMaps);
        Function[] frictionForce(Variable t, params Function[] genCoords);
        Function[,] inertiaMatrixCylinder(double mass, double radius, double height);
        Function[,] inertiaMatrixTorus(double mass, double largeRadius, double smallRadius);
        Function[,] inertiaMatrixSphere(double mass, double radius);
    }

    public class cRigidBodySystemIO
    {
        public string nameSpace;
        private string code;
        private StringBuilder sb = null;
        private StringWriter output = null;

        public cRigidBodySystemIO(string _nameSpace)
        {
            nameSpace = _nameSpace;
            sb = new StringBuilder();
            output = new StringWriter(sb);

        }
        
        private string writeHeader(string RBSName)
        {
            string header = @"
using System;
using System.Diagnostics;
using System.IO;
using System.Windows.Forms;
using System.Collections.Generic;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using matrix;
using utilities;
using DifferentiableFunction;
using Dynamics;


namespace WinFormsGraphicsDevice {
    //class RigidBodySystem;
	public class _" + RBSName + "_ : iRigidBodySystemFunctions " + @"
    {" + @"
       
        public RigidBodySystem build(GraphicsDevice device, Texture2D[] textureMaps)
        {"
       + @"
        Function.newContext();
        List<RigidBody> alljoints = new List<RigidBody>();
		Variable t = new Variable(" + "\"t\"" + @");
		Graph<JointNode> g = null;
		
		
        ";

            return header;
        }

        private string writeMiscFunctions()
        {
            string str = @"
        public Function[] frictionForce(Variable t, params Function[] genCoords)
        {
            Function[] result = new Function[genCoords.Length];

            //for (int i = 0; i < genCoords.Length; i++) { result[i] = (- Math.PI/2.0 - genCoords[i] ) - .2 * Function.D(genCoords[i], t); } //this works only for one cylinder
            for (int i = 0; i < genCoords.Length; i++) { result[i] = -2 * Function.D(genCoords[i], t); }
            return result;
        }

        public Function[,] inertiaMatrixCylinder(double mass, double radius, double height)
        {
            // moment of inertia:
            return new Function[,]{{1/12.0*mass*(3*(radius*radius) + (height*height)), 0 ,0 ,0},
                         {0, .5*mass*(radius*radius), 0 ,0},
                         {0, 0, 1/12.0*mass*(3*(radius*radius) + (height*height)), 0},
                         {0,0,0,0}};
        }

        public Function[,] inertiaMatrixCylinder(Variable mass, double radius, double height)
        {
            // moment of inertia:
            return new Function[,]{{1/12.0*mass*(3*(radius*radius) + (height*height)), 0 ,0 ,0},
                         {0, .5*mass*(radius*radius), 0 ,0},
                         {0, 0, 1/12.0*mass*(3*(radius*radius) + (height*height)), 0},
                         {0,0,0,0}};
        }

        public Function[,] inertiaMatrixTorus(double mass, double largeRadius, double smallRadius)
        {
            return new Function[,]{{1/8.0*mass*(4*largeRadius*largeRadius + 5*smallRadius*smallRadius), 0 ,0 ,0},
                         {0, .25*mass*(4*largeRadius*largeRadius + 3*smallRadius*smallRadius), 0 ,0},
                         {0, 0, 1/8.0*mass*(4*largeRadius*largeRadius + 5*smallRadius*smallRadius), 0},
                         {0,0,0,0}};
        }

        public Function[,] inertiaMatrixTorus(Variable mass, double largeRadius, double smallRadius)
        {
            return new Function[,]{{1/8.0*mass*(4*largeRadius*largeRadius + 5*smallRadius*smallRadius), 0 ,0 ,0},
                         {0, .25*mass*(4*largeRadius*largeRadius + 3*smallRadius*smallRadius), 0 ,0},
                         {0, 0, 1/8.0*mass*(4*largeRadius*largeRadius + 5*smallRadius*smallRadius), 0},
                         {0,0,0,0}};
        }

        public Function[,] inertiaMatrixSphere(double mass, double radius)
        {
            return new Function[,]{ {0.4*mass*radius*radius, 0 ,0 ,0},
                                    {0, 0.4*mass*radius*radius, 0 ,0},
                                    {0, 0, 0.4*mass*radius*radius, 0},
                                    {0,0,0,0}};
        }

        public Function[,] inertiaMatrixSphere(Variable mass, double radius)
        {
            return new Function[,]{ {0.4*mass*radius*radius, 0 ,0 ,0},
                                    {0, 0.4*mass*radius*radius, 0 ,0},
                                    {0, 0, 0.4*mass*radius*radius, 0},
                                    {0,0,0,0}};
        }

    }
}
";
            return str;
        }

            

        public void save(TreeView rigidBodyHierarchy, GraphicsDevice device,  List<Texture2D> textureMaps, string fileName)
        {
            StreamWriter sw = new StreamWriter(fileName, false, Encoding.ASCII);
             
            sb = new StringBuilder();
            output = new StringWriter(sb);
            String header = writeHeader(rigidBodyHierarchy.Nodes[0].Name);
            output.Write(header);
            TraverseTreeView(rigidBodyHierarchy, output, device, textureMaps);
            saveTreeView (rigidBodyHierarchy, output);
            String miscFunctions = writeMiscFunctions();
            output.Write(miscFunctions);
            miscFunctions = output.ToString();

            sw.Write(miscFunctions);
            sw.Close();

           //compileFromFile(miscFunctions);
        }

        
        public iRigidBodySystemFunctions load(string fileName, out string rbsName)
        {
            int i;
            string aux = "public class"; 
            StreamReader sr = new StreamReader(fileName, Encoding.ASCII);
            string source = sr.ReadToEnd();
            sr.Close();
            int first = source.IndexOf(aux);
            int last = source.LastIndexOf(": iRigidBodySystemFunctions");
            string className = source.Substring(first, last - first);
            className = className.Substring(aux.Length);
            className = className.Trim();
            iRigidBodySystemFunctions result = (iRigidBodySystemFunctions)runTimeCompile(source, className, "/optimize");
            rbsName = className.Trim('_');
            return result;
        }

        public object runTimeCompile(string source, string className, string compilerOptions) { return dynamicCompile(source, className, compilerOptions); }

        public static object dynamicCompile(string source, string className, string compilerOptions)
        {
            CompilerParameters compparams = new CompilerParameters(new string[] { "mscorlib.dll", "ClassLibrary1.dll"});

            compparams.ReferencedAssemblies.Add("System.dll");
            compparams.ReferencedAssemblies.Add("System.Windows.Forms.dll");
            compparams.ReferencedAssemblies.Add("C:\\Program Files\\Microsoft XNA\\XNA Game Studio\\v3.0\\References\\Windows\\x86\\Microsoft.Xna.Framework.dll");
            //compparams.ReferencedAssemblies.Add("cRigidBodySystemIO.cs");
            compparams.ReferencedAssemblies.Add("WinFormsGraphicsDevice1.dll");
            
            //compparams.ReferencedAssemblies.Add("System.dll");
            compparams.TreatWarningsAsErrors = false;
            //debug statement
            compparams.IncludeDebugInformation = true;
            //compparams.IncludeDebugInformation = false;
            //end of debug
            compparams.CompilerOptions = compilerOptions;
            //Console.WriteLine(compparams.CompilerOptions);
            //debug to see if this affects round off stuff
            compparams.GenerateInMemory = true;
            Microsoft.CSharp.CSharpCodeProvider csharp = new Microsoft.CSharp.CSharpCodeProvider();
            CompilerResults compresult = csharp.CompileAssemblyFromSource(compparams, source);
            foreach (string s in compresult.Output) { Console.WriteLine(s); }
            foreach (CompilerError error in compresult.Errors) { 
                Console.WriteLine(error.ErrorText); 
            }
            if (compresult == null || compresult.Errors.Count > 0) { Console.WriteLine("Could not complete run time compile"); Console.Read(); Environment.Exit(1); }
            object result = compresult.CompiledAssembly.CreateInstance("WinFormsGraphicsDevice." + className);
            return result;
        }


        private RigidBodySystem TraverseTreeView(TreeView tview, StringWriter output, GraphicsDevice device, List<Texture2D> textureMaps)
        {
            Function.newContext();
            output.WriteLine();
            //Function.printCompilerSource = true;
            //output.WriteLine("Function.printCompilerSource = true;");
            
            Variable t = new Variable("t");
            List<RigidBody> allJoints = new List<RigidBody>();
            Graph<JointNode> g = null;
            long qCont = new long();
            long numRigidBodies = new long();


            //UnspecifiedFunction q0 = UnspecifiedFunction.functionOf("q0", t);

            //Create a TreeNode to hold the Parent Node
            cRigidBodyUI temp = new cRigidBodyUI();
            cRigidBodyUI temp2 = new cRigidBodyUI();
            string sTemp;

            List<double> pos = new List<double>();
            output.WriteLine("\t\tList<double> pos = new List<double>();");
            List<double> mass = new List<double>();
            output.WriteLine("\t\tList<double> mass = new List<double>();");
            List<double> angles = new List<double>();
            output.WriteLine("\t\tList<double> angles = new List<double>();");

            List<double> force = new List<double>();
            output.WriteLine("\t\tList<double> force = new List<double>();");
            List<double> torque = new List<double>();
            output.WriteLine("\t\tList<double> torque = new List<double>();");




            List<double> lCenterOfMass = new List<double>();
            output.WriteLine("\t\tList<double> lCenterOfMass = new List<double>();");




            //Loop through the Parent Nodes
            for (int k = 0; k < tview.Nodes[0].Nodes.Count; k++)
            {

                //Store the Parent Node in temp
                temp = (cRigidBodyUI)tview.Nodes[0].Nodes[k];
                temp.q0 = UnspecifiedFunction.functionOf("q" + qCont.ToString(), t);

                output.WriteLine("\t\tUnspecifiedFunction q" + qCont.ToString() + " = UnspecifiedFunction.functionOf( \"q" + qCont.ToString() + "\", t);");

                temp.vMass = new Variable("m" + qCont.ToString());
                output.WriteLine("\t\tVariable m" + qCont.ToString() + "= new Variable(\"m" + qCont.ToString() + "\");");

                Variable posX = new Variable("posX" + qCont);
                output.WriteLine("\t\tVariable posX" + qCont + " = new Variable(\"posX" + qCont + "\");");
                Variable posY = new Variable("posY" + qCont);
                output.WriteLine("\t\tVariable posY" + qCont + " = new Variable(\"posY" + qCont + "\");");
                Variable posZ = new Variable("posZ" + qCont);
                output.WriteLine("\t\tVariable posZ" + qCont + " = new Variable(\"posZ" + qCont + "\");");

                Variable angleX = new Variable("angleX" + qCont);
                output.WriteLine("\t\tVariable angleX" + qCont + " = new Variable(\"angleX" + qCont + "\");");
                Variable angleY = new Variable("angleY" + qCont);
                output.WriteLine("\t\tVariable angleY" + qCont + " = new Variable(\"angleY" + qCont + "\");");
                Variable angleZ = new Variable("angleZ" + qCont);
                output.WriteLine("\t\tVariable angleZ" + qCont + " = new Variable(\"angleZ" + qCont + "\");");

                Variable forceX = new Variable("forceX" + qCont);
                output.WriteLine("\t\tVariable forceX" + qCont + " = new Variable(\"forceX" + qCont + "\");");
                Variable forceY = new Variable("forceY" + qCont);
                output.WriteLine("\t\tVariable forceY" + qCont + " = new Variable(\"forceY" + qCont + "\");");
                Variable forceZ = new Variable("forceZ" + qCont);
                output.WriteLine("\t\tVariable forceZ" + qCont + " = new Variable(\"forceZ" + qCont + "\");");
                Variable torqueX = new Variable("torqueX" + qCont);
                output.WriteLine("\t\tVariable torqueX" + qCont + " = new Variable(\"torqueX" + qCont + "\");");
                Variable torqueY = new Variable("torqueY" + qCont);
                output.WriteLine("\t\tVariable torqueY" + qCont + " = new Variable(\"torqueY" + qCont + "\");");
                Variable torqueZ = new Variable("torqueZ" + qCont);
                output.WriteLine("\t\tVariable torqueZ" + qCont + " = new Variable(\"torqueZ" + qCont + "\");");




                Variable comX = new Variable("comX" + qCont);
                output.WriteLine("\t\tVariable comX" + qCont + " = new Variable(\"comX" + qCont + "\");");
                Variable comY = new Variable("comY" + qCont);
                output.WriteLine("\t\tVariable comY" + qCont + " = new Variable(\"comY" + qCont + "\");");
                Variable comZ = new Variable("comZ" + qCont);
                output.WriteLine("\t\tVariable comZ" + qCont + " = new Variable(\"comZ" + qCont + "\");");




                pos.Add(temp.position.X);
                output.WriteLine("\t\tpos.Add(" + temp.position.X.ToString() + ");");
                pos.Add(temp.position.Y);
                output.WriteLine("\t\tpos.Add(" + temp.position.Y.ToString() + ");");
                pos.Add(temp.position.Z);
                output.WriteLine("\t\tpos.Add(" + temp.position.Z.ToString() + ");");

                angles.Add(temp.Orientation.X * Math.PI / 180.0);
                output.WriteLine("\t\tangles.Add(" + (temp.Orientation.X * Math.PI / 180.0).ToString() + ");");
                angles.Add(temp.Orientation.Y * Math.PI / 180.0);
                output.WriteLine("\t\tangles.Add(" + (temp.Orientation.Y * Math.PI / 180.0).ToString() + ");");
                angles.Add(temp.Orientation.Z * Math.PI / 180.0);
                output.WriteLine("\t\tangles.Add(" + (temp.Orientation.Z * Math.PI / 180.0).ToString() + ");");
                
                mass.Add(temp.Mass);
                output.WriteLine("\t\tmass.Add(" + temp.Mass.ToString() + ");");

                force.Add(temp.Forces.X);
                output.WriteLine("\t\tforce.Add(" + temp.Forces.X.ToString() + ");");
                force.Add(temp.Forces.Y);
                output.WriteLine("\t\tforce.Add(" + temp.Forces.Y.ToString() + ");");
                force.Add(temp.Forces.Z);
                output.WriteLine("\t\tforce.Add(" + temp.Forces.Z.ToString() + ");");

                torque.Add(temp.Torque.X);
                output.WriteLine("\t\ttorque.Add(" + temp.Torque.X.ToString() + ");");
                torque.Add(temp.Torque.Y);
                output.WriteLine("\t\ttorque.Add(" + temp.Torque.Y.ToString() + ");");
                torque.Add(temp.Torque.Z);
                output.WriteLine("\t\ttorque.Add(" + temp.Torque.Z.ToString() + ");");
                




                lCenterOfMass.Add(temp.CenterOfMass.X);
                output.WriteLine("\t\tlCenterOfMass.Add(" + temp.CenterOfMass.X.ToString() + ");");
                lCenterOfMass.Add(temp.CenterOfMass.Y);
                output.WriteLine("\t\tlCenterOfMass.Add(" + temp.CenterOfMass.Y.ToString() + ");");
                lCenterOfMass.Add(temp.CenterOfMass.Z);
                output.WriteLine("\t\tlCenterOfMass.Add(" + temp.CenterOfMass.Z.ToString() + ");");
                Function[,] centerOfMass = { { comX }, { comY }, { comZ }, { 1 } }; //point so need 1 in 4th position
                output.WriteLine("\t\tFunction[,] centerOfMass = { { comX" + qCont +  "}, { comY" + qCont + "}, { comZ" + qCont + "}, { 1 } };");




                Function[,] A0 = { { 1, 0, 0, posX }, { 0, 1, 0, posY }, { 0, 0, 1, posZ }, { 0, 0, 0, 1 } };
                output.WriteLine("\t\tFunction[,] A" + numRigidBodies + " = {{ 1, 0, 0, posX" + qCont + "}, { 0, 1, 0, posY" + qCont + " }, { 0, 0, 1, posZ" + qCont + "}, { 0, 0, 0, 1 } };");

                Function[,] Aaux = { { 1, 0, 0, 0 }, { 0, 1, 0, 0 }, { 0, 0, 1, 0 }, { 0, 0, 0, 1 } };

                Function[,] rb = null;
                output.WriteLine("\t\tFunction[,] rb" + qCont.ToString() + "= null;");

                sTemp = output.ToString();

                A0 = VM.mult(A0, VM.rotX(angleX, 0, 0, 0)); //conversion to radians
                output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", VM.rotX(angleX" + qCont + ", 0, 0, 0));");
                sTemp = output.ToString();

                A0 = VM.mult(A0, VM.rotY(angleY, 0, 0, 0)); //conversion to radians
                output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", VM.rotY(angleY" + qCont + ", 0, 0, 0));");
                sTemp = output.ToString();

                A0 = VM.mult(A0, VM.rotZ(angleZ, 0, 0, 0)); //conversion to radians
                output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", VM.rotZ(angleZ" + qCont + ", 0, 0, 0));");
                sTemp = output.ToString();


                if (temp.Cylinder.Use_Cylinder == true)
                {
                    // TODO: validate the user input

                    if (temp.Cylinder.Radius <= 0)
                    {
                        MessageBox.Show("Radius should be higher than zero", "Error");
                        temp.Cylinder.Radius = 0.3;
                    }
                    if (temp.Cylinder.Initial_Angle > 360)
                        temp.Cylinder.Initial_Angle = 360;
                    else if (temp.Cylinder.Initial_Angle < 0)
                        temp.Cylinder.Initial_Angle = 0;

                    if (temp.Cylinder.Final_Angle > 360)
                        temp.Cylinder.Final_Angle = 360;
                    else if (temp.Cylinder.Final_Angle < 0)
                        temp.Cylinder.Final_Angle = 0;

                    if (temp.RotXAxis == true)
                    {
                        Aaux = VM.mult(A0, VM.rotX(temp.q0, 0, 0, 0));
                        output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", VM.rotX( q" + qCont + ", 0, 0, 0));");
                    }
                    else if (temp.RotYAxis == true)
                    {
                        Aaux = VM.mult(A0, VM.rotY(temp.q0, 0, 0, 0));
                        output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", VM.rotY( q" + qCont + ", 0, 0, 0));");
                    }
                    else if (temp.RotZAxis == true)
                    {
                        Aaux = VM.mult(A0, VM.rotZ(temp.q0, 0, 0, 0));
                        output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", VM.rotZ( q" + qCont + ", 0, 0, 0));");
                    }

                    else
                    {
                        Aaux = VM.mult(A0, VM.rotX(temp.q0, 0, 0, 0));
                        output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", VM.rotX( q" + qCont + ", 0, 0, 0));");
                    }

                    
                    //rb = VM.rotX(temp.q0, 0, 0, 0);
                    //output.WriteLine("\t\trb" + qCont + " = VM.rotX(q" + qCont + ", 0, 0, 0);");

                    sTemp = output.ToString();

                    Function[,] I1 = inertiaMatrixCylinder(temp.vMass, temp.Cylinder.Radius, temp.Cylinder.Height);
                    output.WriteLine("\t\tFunction[,] I = inertiaMatrixCylinder(m" + qCont + ", " + temp.Cylinder.Radius.ToString("F") + ", " + temp.Cylinder.Height.ToString("F") + ");");
                    sTemp = output.ToString();

                    Geometries.Cylinder cylinder = new Geometries.Cylinder(temp.Cylinder.Initial_Angle * Math.PI / 180.0, temp.Cylinder.Final_Angle * Math.PI / 180.0, 0.0, temp.Cylinder.Height, temp.Cylinder.Radius);
                    output.WriteLine("\t\tGeometries.Cylinder cylinder = new Geometries.Cylinder(" + temp.Cylinder.Initial_Angle.ToString("F") + " * Math.PI / 180.0, " + temp.Cylinder.Final_Angle.ToString("F") + " * Math.PI / 180.0, 0.0, " + temp.Cylinder.Height.ToString("F") + ", " + temp.Cylinder.Radius.ToString("F") + ");");
                    sTemp = output.ToString();

                    temp.rigidBody = new RigidBody(Aaux, new Function[] { temp.q0 }, frictionForce(t, temp.q0), centerOfMass, I1, temp.vMass, temp.Name, t);
                    output.WriteLine("\t\tRigidBody rigidBody" + numRigidBodies + " = new RigidBody(A" + numRigidBodies + " ,new Function[] { q" + qCont + " }, frictionForce(t, q" + qCont + "), centerOfMass, I, m" + qCont + ", \"" + temp.Name + "\", t);");

                    temp.rigidBody.CreateGeometry(device, cylinder, textureMaps[10]);
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".CreateGeometry(device, cylinder, textureMaps[10%4]);");

                    temp.rigidBody.geometry = cylinder;
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".geometry =  cylinder;");
                    sTemp = output.ToString();

                }
                else if (temp.Sphere.Use_sphere == true)
                {
                    // validate the user input
                    if (temp.Sphere.Radius <= 0)
                    {
                        MessageBox.Show("Radius should be higher than zero", "Error");
                        temp.Sphere.Radius = 0.5;
                    }

                    if (temp.RotXAxis == true)
                    {
                        Aaux = VM.mult(A0, VM.rotX(temp.q0, 0, 0, 0));
                        output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", VM.rotX( q" + qCont + ", 0, 0, 0));");
                    }
                    else if (temp.RotYAxis == true)
                    {
                        Aaux = VM.mult(A0, VM.rotY(temp.q0, 0, 0, 0));
                        output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", VM.rotY( q" + qCont + ", 0, 0, 0));");
                    }
                    else if (temp.RotZAxis == true)
                    {
                        Aaux = VM.mult(A0, VM.rotZ(temp.q0, 0, 0, 0));
                        output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", VM.rotZ( q" + qCont + ", 0, 0, 0));");
                    }

                    else
                    {
                        Aaux = VM.mult(A0, VM.rotX(temp.q0, 0, 0, 0));
                        output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", VM.rotX( q" + qCont + ", 0, 0, 0));");
                    }

                    sTemp = output.ToString();


                    Function[,] I1 = inertiaMatrixSphere(temp.vMass, temp.Sphere.Radius);
                    output.WriteLine("\t\tFunction[,] I = inertiaMatrixSphere(m" + qCont + ", " + temp.Sphere.Radius.ToString("F") + ");");
                    sTemp = output.ToString();

                    Geometries.Sphere sphere = new Geometries.Sphere(temp.Sphere.Radius);
                    output.WriteLine("\t\tGeometries.Sphere sphere = new Geometries.Sphere(" + temp.Sphere.Radius + ");");
                    sTemp = output.ToString();

                    temp.rigidBody = new RigidBody(Aaux, new Function[] { temp.q0 }, frictionForce(t, temp.q0), centerOfMass, I1, temp.vMass, temp.Name, t);
                    output.WriteLine("\t\tRigidBody rigidBody" + numRigidBodies + " = new RigidBody(A" + numRigidBodies + ", new Function[] { q" + qCont + " }, frictionForce(t, q" + qCont + "), centerOfMass, I, m" + qCont + ", \"" + temp.Name + "\", t);");


                    temp.rigidBody.CreateGeometry(device, sphere, textureMaps[10]);
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".CreateGeometry(device , sphere, textureMaps[10%4]);");

                    temp.rigidBody.geometry = sphere;
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".geometry =  sphere;");
                    sTemp = output.ToString();

                }
                else if (temp.Torus.Use_Torus == true)
                {
                    // TODO: validate the user input


                    if (temp.Torus.Outer_Radius <= 0)
                    {
                        MessageBox.Show("Outer Radius should be higher than zero", "Error");
                        temp.Torus.Outer_Radius = 0.8;
                    }
                    if (temp.Torus.Inner_Radius <= 0)
                    {
                        MessageBox.Show("Inner Radius should be higher than zero", "Error");
                        temp.Torus.Inner_Radius = 0.2;
                    }

                    if (temp.RotXAxis == true)
                    {
                        Aaux = VM.mult(A0, VM.rotX(temp.q0, 0, 0, 0));
                        output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", VM.rotX( q" + qCont + ", 0, 0, 0));");
                    }
                    else if (temp.RotYAxis == true)
                    {
                        Aaux = VM.mult(A0, VM.rotY(temp.q0, 0, 0, 0));
                        output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", VM.rotY( q" + qCont + ", 0, 0, 0));");
                    }
                    else if (temp.RotZAxis == true)
                    {
                        Aaux = VM.mult(A0, VM.rotZ(temp.q0, 0, 0, 0));
                        output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", VM.rotZ( q" + qCont + ", 0, 0, 0));");
                    }

                    else
                    {
                        Aaux = VM.mult(A0, VM.rotX(temp.q0, 0, 0, 0));
                        output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", VM.rotX( q" + qCont + ", 0, 0, 0));");
                    }

                    sTemp = output.ToString();

                    Function[,] I1 = inertiaMatrixTorus(temp.vMass, temp.Torus.Outer_Radius, temp.Torus.Inner_Radius);
                    output.WriteLine("\t\tFunction[,] I = inertiaMatrixTorus(m" + qCont + ", " + temp.Torus.Outer_Radius.ToString("F") + ", " + temp.Torus.Inner_Radius + ");");
                    sTemp = output.ToString();

                    Geometries.Torus torus = new Geometries.Torus(temp.Torus.Outer_Radius, temp.Torus.Inner_Radius);
                    output.WriteLine("\t\tGeometries.Torus torus = new Geometries.Torus(" + temp.Torus.Outer_Radius.ToString("F") + ", " + temp.Torus.Inner_Radius + ");");
                    sTemp = output.ToString();

                    temp.rigidBody = new RigidBody(Aaux, new Function[] { temp.q0 }, frictionForce(t, temp.q0), centerOfMass, I1, temp.vMass, temp.Name, t);
                    output.WriteLine("\t\tRigidBody rigidBody" + numRigidBodies + " = new RigidBody(A" + numRigidBodies + ", new Function[] { q" + qCont + " }, frictionForce(t, q" + qCont + "), centerOfMass, I, m" + qCont + ", \"" + temp.Name + "\", t);");

                    temp.rigidBody.CreateGeometry(device, torus, textureMaps[10]);
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".CreateGeometry(device, torus, textureMaps[1]);");

                    temp.rigidBody.geometry = torus;
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".geometry =  torus;");
                    sTemp = output.ToString();


                }

                temp.rigidBody.orientation = new Variable[] { angleX, angleY, angleZ };
                output.WriteLine("\t\trigidBody" + numRigidBodies + ".orientation = new Variable[] {angleX" + qCont + ", angleY" + qCont + ", angleZ" + qCont + "};");
                temp.rigidBody.position = new Variable[] {posX, posY, posZ};
                output.WriteLine("\t\trigidBody" + numRigidBodies + ".position = new Variable[] {posX" + qCont + ", posY" + qCont + ", posZ" + qCont + "};");

                temp.rigidBody.force = new Function[,] { { forceX }, { forceY }, { forceZ }, { 0 } }; //vector not point
                output.WriteLine("\t\trigidBody" + numRigidBodies + ".force = new Function[,] { { forceX" + qCont + " }, { forceY" + qCont + "}, { forceZ" + qCont + " }, { 0 }};");
                sTemp = output.ToString();

                temp.rigidBody.torque = new Function[,] { { torqueX }, { torqueY }, { torqueZ }, { 0 } };
                output.WriteLine("\t\trigidBody" + numRigidBodies + ".torque = new Function[,] { { torqueX" + qCont + " }, { torqueY" + qCont + " }, { torqueZ" + qCont + " }, { 0 }};");
                sTemp = output.ToString();
                //Display the Text of the Parent Node i.e. temp
                //MessageBox.Show(temp.Text);
                allJoints.Add(temp.rigidBody);
                output.WriteLine("\t\talljoints.Add(rigidBody" + numRigidBodies + ");");
                sTemp = output.ToString();
                temp.bodyNumber = numRigidBodies;
                qCont++;
                numRigidBodies++;
                //Now Loop through each of the child nodes in this parent node i.e.temp
                for (int i = 0; i < temp.Nodes.Count; i++)
                {
                    temp2 = (cRigidBodyUI)temp.Nodes[i];
                    visitChildNodes(temp2, temp, t, allJoints, output, ref numRigidBodies, ref qCont, device, textureMaps, mass, pos, angles, force, torque, lCenterOfMass); //send every child to the function for further traversal   
                }
            }
            sTemp = output.ToString();
            output.WriteLine();
            //Function.printCompilerSource = true;
            
            g = new Graph<JointNode>(temp.rigidBody, allJoints.ToArray());
            output.WriteLine("\t\tg = new Graph<JointNode>(rigidBody0, alljoints.ToArray());");

            sTemp = output.ToString();
            RigidBodySystem RBSTemp = new RigidBodySystem(g, t, device, mass.ToArray(), pos.ToArray(), angles.ToArray(), force.ToArray(), torque.ToArray(), lCenterOfMass.ToArray());

            output.WriteLine("\t\tRigidBodySystem RBSTemp = new RigidBodySystem(g, t, device, mass.ToArray(), pos.ToArray(), angles.ToArray(), force.ToArray(), torque.ToArray(), lCenterOfMass.ToArray());");
            RBSTemp.initialize();
            output.WriteLine("\t\tRBSTemp.initialize();");
            output.WriteLine("\t\treturn RBSTemp;");
            output.WriteLine("\t} ");
            output.WriteLine();
            sTemp = output.ToString();
            return RBSTemp;
            

        }

        private void visitChildNodes(cRigidBodyUI node, cRigidBodyUI father, Variable t, List<RigidBody> allJoints, StringWriter output, ref long numRigidBodies, ref long qCont, GraphicsDevice device, 
            List<Texture2D> textureMaps, List<double> mass, List<double> pos, List<double> angles, List<double> force, List<double> torque, List<double> lCenterOfMass)
        {
            string sTemp;
            output.WriteLine();
            
            sTemp = output.ToString();

            Function[,] rb = null; // stores the constraints
            output.WriteLine("\t\tFunction[,] rb" + numRigidBodies + " = null;");

            node.q0 = UnspecifiedFunction.functionOf("q" + qCont.ToString(), t);
            output.WriteLine("\t\tUnspecifiedFunction q" + qCont + " = UnspecifiedFunction.functionOf(\"q" + qCont + "\", t);");
            node.vMass = new Variable("m" + qCont.ToString());
            output.WriteLine("\t\tVariable m" + qCont + " = new Variable(\"m" + qCont + "\");");
            sTemp = output.ToString();

            Variable posX = new Variable("posX" + qCont);
            output.WriteLine("\t\tVariable posX" + qCont + " = new Variable(\"posX" + qCont + "\");");
            Variable posY = new Variable("posY" + qCont);
            output.WriteLine("\t\tVariable posY" + qCont + " = new Variable(\"posY" + qCont + "\");");
            Variable posZ = new Variable("posZ" + qCont);
            output.WriteLine("\t\tVariable posZ" + qCont + " = new Variable(\"posZ" + qCont + "\");");

            Variable angleX = new Variable("angleX" + qCont);
            output.WriteLine("\t\tVariable angleX" + qCont + " = new Variable(\"angleX" + qCont + "\");");
            Variable angleY = new Variable("angleY" + qCont);
            output.WriteLine("\t\tVariable angleY" + qCont + " = new Variable(\"angleY" + qCont + "\");");
            Variable angleZ = new Variable("angleZ" + qCont);
            output.WriteLine("\t\tVariable angleZ" + qCont + " = new Variable(\"angleZ" + qCont + "\");");

            Variable forceX = new Variable("forceX" + qCont);
            output.WriteLine("\t\tVariable forceX" + qCont + " = new Variable(\"forceX" + qCont + "\");");
            Variable forceY = new Variable("forceY" + qCont);
            output.WriteLine("\t\tVariable forceY" + qCont + " = new Variable(\"forceY" + qCont + "\");");
            Variable forceZ = new Variable("forceZ" + qCont);
            output.WriteLine("\t\tVariable forceZ" + qCont + " = new Variable(\"forceZ" + qCont + "\");");
            Variable torqueX = new Variable("torqueX" + qCont);
            output.WriteLine("\t\tVariable torqueX" + qCont + " = new Variable(\"torqueX" + qCont + "\");");
            Variable torqueY = new Variable("torqueY" + qCont);
            output.WriteLine("\t\tVariable torqueY" + qCont + " = new Variable(\"torqueY" + qCont + "\");");
            Variable torqueZ = new Variable("torqueZ" + qCont);
            output.WriteLine("\t\tVariable torqueZ" + qCont + " = new Variable(\"torqueZ" + qCont + "\");");




            Variable comX = new Variable("comX" + qCont);
            output.WriteLine("\t\tVariable comX" + qCont + " = new Variable(\"comX" + qCont + "\");");
            Variable comY = new Variable("comY" + qCont);
            output.WriteLine("\t\tVariable comY" + qCont + " = new Variable(\"comY" + qCont + "\");");
            Variable comZ = new Variable("comZ" + qCont);
            output.WriteLine("\t\tVariable comZ" + qCont + " = new Variable(\"comZ" + qCont + "\");");
            Function[,] centerOfMass = { { comX }, { comY }, { comZ }, { 1 } }; //point so need 1 in 4th position
            output.WriteLine("\t\tFunction[,] centerOfMass" + numRigidBodies + " = { { comX" + qCont + "}, { comY" + qCont + "}, { comZ" + qCont + "}, { 1 } };");


            pos.Add(node.position.X);
            output.WriteLine("\t\tpos.Add(" + node.position.X.ToString() + ");");
            pos.Add(node.position.Y);
            output.WriteLine("\t\tpos.Add(" + node.position.Y.ToString() + ");");
            pos.Add(node.position.Z);
            output.WriteLine("\t\tpos.Add(" + node.position.Z.ToString() + ");");

            angles.Add(node.Orientation.X * Math.PI / 180.0);
            output.WriteLine("\t\tangles.Add(" + (node.Orientation.X * Math.PI / 180.0).ToString() + ");");
            angles.Add(node.Orientation.Y * Math.PI / 180.0);
            output.WriteLine("\t\tangles.Add(" + (node.Orientation.Y * Math.PI / 180.0).ToString() + ");");
            angles.Add(node.Orientation.Z * Math.PI / 180.0);
            output.WriteLine("\t\tangles.Add(" + (node.Orientation.Z * Math.PI / 180.0).ToString() + ");");
            
            mass.Add(node.Mass);
            output.WriteLine("\t\tmass.Add(" + node.Mass.ToString() + ");");

            force.Add(node.Forces.X);
            output.WriteLine("\t\tforce.Add(" + node.Forces.X.ToString() + ");");
            force.Add(node.Forces.Y);
            output.WriteLine("\t\tforce.Add(" + node.Forces.Y.ToString() + ");");
            force.Add(node.Forces.Z);
            output.WriteLine("\t\tforce.Add(" + node.Forces.Z.ToString() + ");");

            torque.Add(node.Torque.X);
            output.WriteLine("\t\ttorque.Add(" + node.Torque.X.ToString() + ");");
            torque.Add(node.Torque.Y);
            output.WriteLine("\t\ttorque.Add(" + node.Torque.Y.ToString() + ");");
            torque.Add(node.Torque.Z);
            output.WriteLine("\t\ttorque.Add(" + node.Torque.Z.ToString() + ");");


            lCenterOfMass.Add(node.CenterOfMass.X);
            output.WriteLine("\t\tlCenterOfMass.Add(" + node.CenterOfMass.X.ToString() + ");");
            lCenterOfMass.Add(node.CenterOfMass.Y);
            output.WriteLine("\t\tlCenterOfMass.Add(" + node.CenterOfMass.Y.ToString() + ");");
            lCenterOfMass.Add(node.CenterOfMass.Z);
            output.WriteLine("\t\tlCenterOfMass.Add(" + node.CenterOfMass.Z.ToString() + ");");


            Function[,] A0 = { { 1, 0, 0, posX }, { 0, 1, 0, posY }, { 0, 0, 1, posZ }, { 0, 0, 0, 1 } };
            output.WriteLine("\t\tFunction[,] A" + numRigidBodies + " = { { 1, 0, 0, posX" + qCont + " }, { 0, 1, 0, posY" + qCont + "}, { 0, 0, 1, posZ" + qCont + "}, { 0, 0, 0, 1 } };");


            A0 = VM.mult(A0, VM.rotX(Math.PI * node.orientation.X / 180.0, 0, 0, 0)); //conversion to radians
            output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", VM.rotX(angleX" + qCont + ", 0, 0, 0));");

            A0 = VM.mult(A0, VM.rotY(Math.PI * node.orientation.Y / 180.0, 0, 0, 0)); //conversion to radians
            output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", VM.rotY(angleY" + qCont + ", 0, 0, 0));");

            A0 = VM.mult(A0, VM.rotZ(Math.PI * node.orientation.Z / 180.0, 0, 0, 0)); //conversion to radians
            output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", VM.rotZ(angleZ" + qCont + ", 0, 0, 0));");
            sTemp = output.ToString();

            if (node.Cylinder.Use_Cylinder == true)
            {
                if (node.Cylinder.Radius <= 0)
                {
                    MessageBox.Show("Radius should be higher than zero", "Error");
                    node.Cylinder.Radius = 0.3;
                }

                if (node.Cylinder.Initial_Angle > 360)
                    node.Cylinder.Initial_Angle = 360;
                else if (node.Cylinder.Initial_Angle < 0)
                    node.Cylinder.Initial_Angle = 0;

                if (node.Cylinder.Final_Angle > 360)
                    node.Cylinder.Final_Angle = 360;
                else if (node.Cylinder.Final_Angle < 0)
                    node.Cylinder.Final_Angle = 0;

                Function[,] I1 = inertiaMatrixCylinder(node.vMass, node.Cylinder.Radius, node.Cylinder.Height);
                output.WriteLine("\t\tFunction[,] I" + numRigidBodies + " = inertiaMatrixCylinder(m" + qCont + ", " + node.Cylinder.Radius.ToString("F") + ", " + node.Cylinder.Height.ToString("F") + ");");

                Geometries.Cylinder cylinder = new Geometries.Cylinder(node.Cylinder.Initial_Angle * Math.PI / 180.0, node.Cylinder.Final_Angle * Math.PI / 180.0, 0.0, node.Cylinder.Height, node.Cylinder.Radius);
                output.WriteLine("\t\tGeometries.Cylinder cylinder" + numRigidBodies + " = new Geometries.Cylinder(" + node.Cylinder.Initial_Angle.ToString("F") + " * Math.PI / 180.0, " + node.Cylinder.Final_Angle.ToString("F") + " * Math.PI / 180.0, 0.0, " + node.Cylinder.Height.ToString("F") + ", " + node.Cylinder.Radius.ToString("F") + ");");

                if (node.Surface == true)
                {
                    qCont++;
                    node.q1 = UnspecifiedFunction.functionOf("q" + qCont.ToString(), t);
                    output.WriteLine("\t\tUnspecifiedFunction q" + qCont + " = UnspecifiedFunction.functionOf(\"q" + qCont + "\", t);");
                    
                    rb = father.rigidBody.geometry.SurfaceTangentFrame(node.q0, node.q1);
                    output.WriteLine("\t\trb" + numRigidBodies + " = rigidBody" + father.bodyNumber + ".geometry.SurfaceTangentFrame(q" + (qCont - 1) + ", q" + qCont  + ");");

                    A0 = VM.mult(A0, rb);
                    output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", rb" + numRigidBodies + ");");

                    node.rigidBody = new RigidBody(A0, new Function[] { node.q0, node.q1 }, frictionForce(t, node.q0, node.q1), centerOfMass, I1, node.vMass, node.Name, t);
                    output.WriteLine("\t\tRigidBody rigidBody" + numRigidBodies + " = new RigidBody(A" + numRigidBodies + ", new Function[] { q" + (qCont - 1) + ", q" + qCont + " }, frictionForce(t, q" + (qCont - 1) + ", q" + qCont  + "), centerOfMass" + numRigidBodies + ", I" + numRigidBodies + ", m" + (qCont -1) + ", \"" + node.Name + "\", t);");

                    node.rigidBody.CreateGeometry(device, cylinder, textureMaps[11]);
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".CreateGeometry(device, cylinder" + numRigidBodies + ", textureMaps[2]);");

                    node.rigidBody.force = new Function[,] { { forceX /** rb[0, 0]*/ }, { forceY }, { forceZ /** rb[2, 0]*/ }, { 0 } }; //vector not point
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".force = new Function[,] { { forceX" + (qCont - 1) + /*" * rb" + numRigidBodies + "[0, 0]*/ "}, { forceY" + (qCont - 1) + " }, { forceZ" + (qCont - 1) + /*" * rb" + numRigidBodies + "[2, 0]*/ "}, { 0 } };");

                    node.rigidBody.orientation = new Variable[] { angleX, angleY, angleZ };
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".orientation = new Variable[] {angleX" + (qCont - 1) + ", angleY" + (qCont - 1) + ", angleZ" + (qCont - 1) + "};");
                    node.rigidBody.position = new Variable[] { posX, posY, posZ };
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".position = new Variable[] {posX" + (qCont - 1) + ", posY" + (qCont - 1) + ", posZ" + (qCont - 1) + "};");

                    node.rigidBody.torque = new Function[,] { { torqueX }, { torqueY }, { torqueZ }, { 0 } };
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".torque = new Function[,] { { torqueX" + (qCont - 1) + " }, { torqueY" + (qCont - 1) + " }, { torqueZ" + (qCont - 1) + " }, { 0 }};");

                    qCont++;
                    sTemp = output.ToString();
                }
                else
                {
                    if (node.RotXAxis== true)
                    {
                        A0 = VM.mult(A0, VM.rotX(node.q0, 0, 0, 0));
                        output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", VM.rotX(q" + qCont + ", 0, 0, 0));");
                    }
                    else if (node.RotYAxis == true)
                    {
                        A0 = VM.mult(A0, VM.rotY(node.q0, 0, 0, 0));
                        output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", VM.rotY(q" + qCont + ", 0, 0, 0));");
                    }
                    else if (node.RotZAxis == true)
                    {
                        A0 = VM.mult(A0, VM.rotZ(node.q0, 0, 0, 0));
                        output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", VM.rotZ(q" + qCont + ", 0, 0, 0));");
                    }
                    else
                    {
                        A0 = VM.mult(A0, VM.rotZ(node.q0, 0, 0, 0));
                        output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", VM.rotZ(q" + qCont + ", 0, 0, 0));");
                    }
                    
                    //rb = VM.rotZ(node.q0, 0, 0, 0);
                    //output.WriteLine("\t\trb" + qCont + " = VM.rotZ(q" + qCont + ", 0, 0, 0);");

                    node.rigidBody = new RigidBody(A0,  new Function[] { node.q0 }, frictionForce(t, node.q0), centerOfMass, I1, node.vMass, node.Name, t);
                    output.WriteLine("\t\tRigidBody rigidBody" + numRigidBodies + " = new RigidBody(A" + numRigidBodies + ", new Function[] { q" + qCont + " }, frictionForce(t, q" + qCont + "), centerOfMass" + numRigidBodies + ", I" + numRigidBodies + ", m" + qCont + ", \"" + node.Name + "\", t);");

                    node.rigidBody.CreateGeometry(device, cylinder, textureMaps[11]);
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".CreateGeometry(device, cylinder" + numRigidBodies + ", textureMaps[1]);");

                    node.rigidBody.force = new Function[,] { { forceX }, { forceY }, { forceZ }, { 0 } }; //vector not point                    
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".force = new Function[,] { { forceX" + qCont + "}, { forceY" + qCont + " }, { forceZ" + qCont + "}, { 0 } };");

                    node.rigidBody.orientation = new Variable[] { angleX, angleY, angleZ };
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".orientation = new Variable[] {angleX" + qCont + ", angleY" + qCont + ", angleZ" + qCont + "};");
                    node.rigidBody.position = new Variable[] { posX, posY, posZ };
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".position = new Variable[] {posX" + qCont + ", posY" + qCont + ", posZ" + qCont + "};");

                    node.rigidBody.torque = new Function[,] { { torqueX }, { torqueY }, { torqueZ }, { 0 } };
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".torque = new Function[,] { { torqueX" + qCont + " }, { torqueY" + qCont + " }, { torqueZ" + qCont + " }, { 0 }};");


                    sTemp = output.ToString();
                    qCont++;
                }
                node.rigidBody.geometry = cylinder;
                output.WriteLine("\t\trigidBody" + numRigidBodies + ".geometry = cylinder" + numRigidBodies + ";");
                sTemp = output.ToString();
            }
            else if (node.Sphere.Use_sphere == true)
            {
                // TODO: validate the user input

                if (node.Sphere.Radius <= 0)
                {
                    MessageBox.Show("Radius should be higher than zero", "Error");
                    node.Sphere.Radius = 0.5;
                }

                Function[,] I1 = inertiaMatrixSphere(node.vMass, node.Sphere.Radius);
                output.WriteLine("\t\tFunction[,] I" + numRigidBodies + " = inertiaMatrixSphere(m" + qCont + ", " + node.Sphere.Radius.ToString("F") + ");");
                Geometries.Sphere sphere = new Geometries.Sphere(node.Sphere.Radius);
                output.WriteLine("\t\tGeometries.Sphere sphere" + numRigidBodies + " = new Geometries.Sphere(" + node.Sphere.Radius.ToString("F") + ");");

                if (node.Surface == true)
                {
                    qCont++;
                    node.q1 = UnspecifiedFunction.functionOf("q" + qCont.ToString(), t);
                    output.WriteLine("\t\tUnspecifiedFunction q" + qCont + " = UnspecifiedFunction.functionOf(\"q" + qCont + "\", t);");
                    
                    rb = father.rigidBody.geometry.SurfaceTangentFrame(node.q0, node.q1);
                    output.WriteLine("\t\trb" + numRigidBodies + " = rigidBody" + father.bodyNumber + ".geometry.SurfaceTangentFrame(q" + (qCont - 1) + ", q" + (qCont ) + ");");

                    A0 = VM.mult(A0, rb);
                    output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", rb" + numRigidBodies + ");");

                    node.rigidBody = new RigidBody(A0, new Function[] { node.q0, node.q1 }, frictionForce(t, node.q0, node.q1), centerOfMass, I1, node.vMass, node.Name, t);
                    output.WriteLine("\t\tRigidBody rigidBody" + numRigidBodies + " = new RigidBody(A" + numRigidBodies + ", new Function[] { q" + (qCont - 1) + ", q" + qCont + " }, frictionForce(t, q" + (qCont - 1) + ", q" + qCont + "), centerOfMass" + numRigidBodies + ", I" + numRigidBodies + ", m" + (qCont - 1) + ", \"" + node.Name + "\", t);");

                    node.rigidBody.CreateGeometry(device, sphere, textureMaps[11]);
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".CreateGeometry(device, sphere" + numRigidBodies + ", textureMaps[1]);");

                    node.rigidBody.force = new Function[,] { { forceX /** rb[0, 0]*/ }, { forceY }, { forceZ /** rb[2, 0]*/ }, { 0 } }; //vector not point
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".force = new Function[,] { { forceX" + (qCont - 1) + /*" * rb" + numRigidBodies + "[0, 0]*/ "}, { forceY" + (qCont - 1) + " }, { forceZ" + (qCont - 1) + /*" * rb" + numRigidBodies + "[2, 0]*/ "}, { 0 } };");
                    
                    node.rigidBody.orientation = new Variable[] { angleX, angleY, angleZ };
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".orientation = new Variable[] {angleX" + (qCont - 1) + ", angleY" + (qCont - 1) + ", angleZ" + (qCont - 1) + "};");
                    node.rigidBody.position = new Variable[] { posX, posY, posZ };
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".position = new Variable[] {posX" + (qCont - 1) + ", posY" + (qCont - 1) + ", posZ" + (qCont - 1) + "};");

                    node.rigidBody.torque = new Function[,] { { torqueX }, { torqueY }, { torqueZ }, { 0 } };
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".torque = new Function[,] { { torqueX" + (qCont - 1) + " }, { torqueY" + (qCont - 1) + " }, { torqueZ" + (qCont - 1) + " }, { 0 }};");

                    qCont++;
                    sTemp = output.ToString();
                }
                else
                {
                    if (node.RotXAxis == true)
                    {
                        A0 = VM.mult(A0, VM.rotX(node.q0, 0, 0, 0));
                        output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", VM.rotX(q" + qCont + ", 0, 0, 0));");
                    }
                    else if (node.RotYAxis == true)
                    {
                        A0 = VM.mult(A0, VM.rotY(node.q0, 0, 0, 0));
                        output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", VM.rotY(q" + qCont + ", 0, 0, 0));");
                    }
                    else if (node.RotZAxis == true)
                    {
                        A0 = VM.mult(A0, VM.rotZ(node.q0, 0, 0, 0));
                        output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", VM.rotZ(q" + qCont + ", 0, 0, 0));");
                    }
                    else
                    {
                        A0 = VM.mult(A0, VM.rotZ(node.q0, 0, 0, 0));
                        output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", VM.rotZ(q" + qCont + ", 0, 0, 0));");
                    }
                    
                    //output.WriteLine("\t\trb" + qCont + " = VM.rotZ(q" + qCont + ", 0, 0, 0);");

                    node.rigidBody = new RigidBody(A0, new Function[] { node.q0 }, frictionForce(t, node.q0), centerOfMass, I1, node.vMass, node.Name, t);
                    output.WriteLine("\t\tRigidBody rigidBody" + numRigidBodies + " = new RigidBody(A" + numRigidBodies + ", new Function[] { q" + qCont + " }, frictionForce(t, q" + qCont + "), centerOfMass" + numRigidBodies + ", I" + numRigidBodies + ", m" + qCont + ", \"" + node.Name + "\", t);");

                    node.rigidBody.CreateGeometry(device, sphere, textureMaps[11]);
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".CreateGeometry(device, sphere" + numRigidBodies + ", textureMaps[1]);");

                    node.rigidBody.force = new Function[,] { { forceX }, { forceY }, { forceZ }, { 0 } }; //vector not point
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".force = new Function[,] { { forceX" + qCont + "}, { forceY" + qCont + " }, { forceZ" + qCont + "}, { 0 } };");

                    node.rigidBody.orientation = new Variable[] { angleX, angleY, angleZ };
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".orientation = new Variable[] {angleX" + qCont + ", angleY" + qCont + ", angleZ" + qCont + "};");
                    node.rigidBody.position = new Variable[] { posX, posY, posZ };
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".position = new Variable[] {posX" + qCont + ", posY" + qCont + ", posZ" + qCont + "};");

                    node.rigidBody.torque = new Function[,] { { torqueX }, { torqueY }, { torqueZ }, { 0 } };
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".torque = new Function[,] { { torqueX" + qCont + " }, { torqueY" + qCont + " }, { torqueZ" + qCont + " }, { 0 }};");

                    qCont++;

                }
                node.rigidBody.geometry = sphere;
                output.WriteLine("\t\trigidBody" + numRigidBodies + ".geometry = sphere" + numRigidBodies + ";");
                sTemp = output.ToString();

            }
            else if (node.Torus.Use_Torus == true)
            {
                // TODO: validate the user input

                if (node.Torus.Outer_Radius <= 0)
                {
                    MessageBox.Show("Outer Radius should be higher than zero", "Error");
                    node.Torus.Outer_Radius = 0.8;
                }
                if (node.Torus.Inner_Radius <= 0)
                {
                    MessageBox.Show("Inner Radius should be higher than zero", "Error");
                    node.Torus.Inner_Radius = 0.2;
                }

                //A0 = VM.mult (A0,VM.rotX(node.q0, node.Torus.Outer_Radius, 0, 0));

                Function[,] I1 = inertiaMatrixTorus(node.vMass, node.Torus.Outer_Radius, node.Torus.Inner_Radius);
                output.WriteLine("\t\tFunction[,] I" + numRigidBodies + " = inertiaMatrixTorus(m" + qCont + ", " + node.Torus.Outer_Radius.ToString("F") + ", " + node.Torus.Inner_Radius.ToString("F") + ");");
                Geometries.Torus torus = new Geometries.Torus(node.Torus.Outer_Radius, node.Torus.Inner_Radius);
                output.WriteLine("\t\tGeometries.Torus torus" + numRigidBodies + " = new Geometries.Torus(" + node.Torus.Outer_Radius.ToString("F") + ", " + node.Torus.Inner_Radius.ToString("F") + ");");

                if (node.Surface == true)
                {
                    qCont++;
                    node.q1 = UnspecifiedFunction.functionOf("q" + qCont.ToString(), t);
                    output.WriteLine("\t\tUnspecifiedFunction q" + qCont + " = UnspecifiedFunction.functionOf(\"q" + qCont + "\", t);");
                    
                    rb = father.rigidBody.geometry.SurfaceTangentFrame(node.q0, node.q1);
                    output.WriteLine("\t\trb" + numRigidBodies + " = rigidBody" + father.bodyNumber + ".geometry.SurfaceTangentFrame(q" + (qCont - 1) + ", q" + (qCont ) + ");");

                    A0 = VM.mult(A0, rb);
                    output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", rb"+ numRigidBodies + ");");

                    node.rigidBody = new RigidBody(A0, new Function[] { node.q0, node.q1 }, frictionForce(t, node.q0, node.q1), centerOfMass, I1, node.vMass, node.Name, t);
                    output.WriteLine("\t\tRigidBody rigidBody" + numRigidBodies + " = new RigidBody(A" + numRigidBodies + ", new Function[] { q" + (qCont - 1) + ", q" + (qCont) + " }, frictionForce(t, q" + (qCont - 1) + ", q" + (qCont) + "), centerOfMass" + numRigidBodies + ", I" + numRigidBodies + ", m" + (qCont-1) + ", \"" + node.Name + "\", t);");

                    node.rigidBody.CreateGeometry(device, torus, textureMaps[11]);
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".CreateGeometry(device, torus" + numRigidBodies + ", textureMaps[1]);");

                    node.rigidBody.force = new Function[,] { { forceX /** rb[0, 0]*/ }, { forceY }, { forceZ /** rb[2, 0]*/ }, { 0 } }; //vector not point
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".force = new Function[,] { { forceX" + (qCont - 1) + /*" * rb" + numRigidBodies + "[0, 0]*/ "}, { forceY" + (qCont - 1) + " }, { forceZ" + (qCont - 1) + /*" * rb" + numRigidBodies + "[2, 0]*/ "}, { 0 } };");

                    node.rigidBody.orientation = new Variable[] { angleX, angleY, angleZ };
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".orientation = new Variable[] {angleX" + (qCont - 1) + ", angleY" + (qCont - 1) + ", angleZ" + (qCont - 1) + "};");
                    node.rigidBody.position = new Variable[] { posX, posY, posZ };
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".position = new Variable[] {posX" + (qCont - 1) + ", posY" + (qCont - 1) + ", posZ" + (qCont - 1) + "};");

                    node.rigidBody.torque = new Function[,] { { torqueX }, { torqueY }, { torqueZ }, { 0 } };
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".torque = new Function[,] { { torqueX" + (qCont - 1) + " }, { torqueY" + (qCont - 1) + " }, { torqueZ" + (qCont - 1) + " }, { 0 }};");
                    qCont++;
                    sTemp = output.ToString();
                }
                else
                {
                    if (node.RotXAxis == true)
                    {
                        A0 = VM.mult(A0, VM.rotX(node.q0, 0, 0, 0));
                        output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", VM.rotX(q" + qCont + ", 0, 0, 0));");
                    }
                    else if (node.RotYAxis == true)
                    {
                        A0 = VM.mult(A0, VM.rotY(node.q0, 0, 0, 0));
                        output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", VM.rotY(q" + qCont + ", 0, 0, 0));");
                    }
                    else if (node.RotZAxis == true)
                    {
                        A0 = VM.mult(A0, VM.rotZ(node.q0, 0, 0, 0));
                        output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", VM.rotZ(q" + qCont + ", 0, 0, 0));");
                    }
                    else
                    {
                        A0 = VM.mult(A0, VM.rotZ(node.q0, 0, 0, 0));
                        output.WriteLine("\t\tA" + numRigidBodies + " = VM.mult(A" + numRigidBodies + ", VM.rotZ(q" + qCont + ", 0, 0, 0));");
                    }

                    node.rigidBody = new RigidBody(A0, new Function[] { node.q0 }, frictionForce(t, node.q0), centerOfMass, I1, node.vMass, node.Name, t);
                    output.WriteLine("\t\tRigidBody rigidBody" + numRigidBodies + " = new RigidBody(A" + numRigidBodies + ", new Function[] { q" + qCont + " }, frictionForce(t, q" + qCont + "), centerOfMass" + numRigidBodies + ", I" + numRigidBodies + ", m" + qCont + ", \"" + node.Name + "\", t);");

                    node.rigidBody.CreateGeometry(device, torus, textureMaps[11]);
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".CreateGeometry(device, torus" + numRigidBodies + ", textureMaps[2]);");

                    node.rigidBody.force = new Function[,] { { forceX }, { forceY }, { forceZ }, { 0 } }; //vector not point
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".force = new Function[,] { { forceX" + qCont + "}, { forceY" + qCont + " }, { forceZ" + qCont + "}, { 0 } };");

                    node.rigidBody.orientation = new Variable[] { angleX, angleY, angleZ };
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".orientation = new Variable[] {angleX" + qCont + ", angleY" + qCont + ", angleZ" + qCont + "};");
                    node.rigidBody.position = new Variable[] { posX, posY, posZ };
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".position = new Variable[] {posX" + qCont + ", posY" + qCont + ", posZ" + qCont + "};");

                    node.rigidBody.torque = new Function[,] { { torqueX }, { torqueY }, { torqueZ }, { 0 } };
                    output.WriteLine("\t\trigidBody" + numRigidBodies + ".torque = new Function[,] { { torqueX" + qCont + " }, { torqueY" + qCont + " }, { torqueZ" + qCont + " }, { 0 }};");

                    qCont++;
                }

                node.rigidBody.geometry = torus;
                output.WriteLine("\t\trigidBody" + numRigidBodies + ".geometry = torus" + numRigidBodies + ";");
                sTemp = output.ToString();
            }

            allJoints.Add(node.rigidBody);
            output.WriteLine("\t\talljoints.Add(rigidBody" + numRigidBodies + ");");
            father.rigidBody.c(node.rigidBody);
            output.WriteLine("\t\trigidBody" + father.bodyNumber + ".c(rigidBody" + numRigidBodies + ");");
            sTemp = output.ToString();
            node.bodyNumber = numRigidBodies;

            numRigidBodies++;
            //MessageBox.Show(node.Name);// + " Mass " + node.Mass.ToString());    
            //body.Mass = 2.0;
            //Loop Through this node and its childs recursively
            for (int j = 0; j < node.Nodes.Count; j++)
                visitChildNodes((cRigidBodyUI)node.Nodes[j], node, t, allJoints, output, ref numRigidBodies, ref qCont, device, textureMaps,mass, pos, angles, force, torque, lCenterOfMass); //send every child to the function for further traversal   

        }


        private void saveTreeView(TreeView tview, StringWriter output)
        {
            long numRigidBodies = new long();
                        
            cRigidBodyUI temp, temp2;

            string str = @"
        public void populateTreeView (TreeView rigidBodyHierarchy, string RBSFileName)
        {
            TreeNode root = new TreeNode(RBSFileName);
            root.ImageIndex = 0;
            root.Name = RBSFileName;
            rigidBodyHierarchy.Nodes.Insert(0, root);";

            output.Write(str);
            output.WriteLine("");

            //Loop through the Parent Nodes
            for (int k = 0; k < tview.Nodes[0].Nodes.Count; k++)
            {
                temp = (cRigidBodyUI)tview.Nodes[0].Nodes[k];

                output.WriteLine("\tcRigidBodyUI body" + numRigidBodies + "= new cRigidBodyUI();");
                output.WriteLine("\t\tbody" + numRigidBodies + ".position.X = " + temp.position.X.ToString() + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".position.Y = " + temp.position.Y.ToString() + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".position.Z = " + temp.position.Z.ToString() + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".orientation.X = " + temp.orientation.X.ToString() + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".orientation.Y = " + temp.orientation.Y.ToString() + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".orientation.Z = " + temp.orientation.Z.ToString() + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".Sphere.Radius = " + temp.Sphere.Radius + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".Sphere.Use_sphere = " + temp.Sphere.Use_sphere.ToString().ToLower() + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".Cylinder.Final_Angle = " + temp.Cylinder.Final_Angle + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".Cylinder.Height = " + temp.Cylinder.Height + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".Cylinder.Initial_Angle = " + temp.Cylinder.Initial_Angle + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".Cylinder.Radius = " + temp.Cylinder.Radius + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".Cylinder.Use_Cylinder = " + temp.Cylinder.Use_Cylinder.ToString().ToLower() + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".Torus.Inner_Radius = " + temp.Torus.Inner_Radius + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".Torus.Outer_Radius = " + temp.Torus.Outer_Radius + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".Torus.Use_Torus = " + temp.Torus.Use_Torus.ToString().ToLower() + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".Mass = " + temp.Mass.ToString() + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".Forces.X = " + temp.Forces.X.ToString() + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".Forces.Y = " + temp.Forces.Y.ToString() + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".Forces.Z = " + temp.Forces.Z.ToString() + ";");                
                output.WriteLine("\t\tbody" + numRigidBodies + ".Torque.X = " + temp.Torque.X.ToString() + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".Torque.Y = " + temp.Torque.Y.ToString() + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".Torque.Z = " + temp.Torque.Z.ToString() + ";");

                output.WriteLine("\t\tbody" + numRigidBodies + ".CenterOfMass.X = " + temp.CenterOfMass.X.ToString() + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".CenterOfMass.Y = " + temp.CenterOfMass.Y.ToString() + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".CenterOfMass.Z = " + temp.CenterOfMass.Z.ToString() + ";");

                
                output.WriteLine("\t\tbody" + numRigidBodies + ".RotXAxis = " + temp.RotXAxis.ToString().ToLower() + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".RotYAxis = " + temp.RotYAxis.ToString().ToLower() + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".RotZAxis = " + temp.RotZAxis.ToString().ToLower() + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".TransXaxis = " + temp.TransXaxis.ToString().ToLower() + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".TransYaxis = " + temp.TransYaxis.ToString().ToLower() + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".TransZaxis = " + temp.TransZaxis.ToString().ToLower() + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".Surface = " + temp.Surface.ToString().ToLower() + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".Name = \"" + temp.Name + "\";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".Text = \"" + temp.Name + "\";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".ImageIndex = 1;");
                output.WriteLine("\t\tbody" + numRigidBodies + ".genCoords.q0 = " + temp.genCoords.q0.ToString() + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".genCoords.q1 = " + temp.genCoords.q1.ToString() + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".genCoords.q2 = " + temp.genCoords.q2.ToString() + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".genCoords.q3 = " + temp.genCoords.q3.ToString() + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".genCoords.q4 = " + temp.genCoords.q4.ToString() + ";");
                output.WriteLine("\t\tbody" + numRigidBodies + ".genCoords.q5 = " + temp.genCoords.q5.ToString() + ";");

                

                    

                temp.bodyNumber = numRigidBodies;
                //output.WriteLine("\t\trigidBodyHierarchy.Nodes[0].Nodes.Insert(rigidBodyHierarchy.Nodes[0].Nodes.Count, body" + numRigidBodies + ");");
                numRigidBodies++;
                str = output.ToString();

                //Display the Text of the Parent Node i.e. temp
                // MessageBox.Show(temp.Text);

                //Now Loop through each of the child nodes in this parent node i.e.temp
                for (int i = 0; i < temp.Nodes.Count; i++)
                {
                    visitChildNodes((cRigidBodyUI)temp.Nodes[i], temp, output, ref numRigidBodies); //send every child to the function for further traversal
                }
            }
            output.WriteLine("");
            output.WriteLine("\t\trigidBodyHierarchy.Nodes[0].Nodes.Insert(rigidBodyHierarchy.Nodes[0].Nodes.Count, body0);");
            output.WriteLine("\t}");
            str = output.ToString();
        }

        private void visitChildNodes(cRigidBodyUI node, cRigidBodyUI father, StringWriter output, ref long numRigidBodies)
        {
            string str;
            output.WriteLine("");
            output.WriteLine("\t\tcRigidBodyUI body" + numRigidBodies + "= new cRigidBodyUI();");
            output.WriteLine("\t\tbody" + numRigidBodies + ".position.X = " + node.position.X.ToString() + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".position.Y = " + node.position.Y.ToString() + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".position.Z = " + node.position.Z.ToString() + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".orientation.X = " + node.orientation.X.ToString() + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".orientation.Y = " + node.orientation.Y.ToString() + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".orientation.Z = " + node.orientation.Z.ToString() + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".Sphere.Radius = " + node.Sphere.Radius + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".Sphere.Use_sphere = " + node.Sphere.Use_sphere.ToString().ToLower() + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".Cylinder.Final_Angle = " + node.Cylinder.Final_Angle + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".Cylinder.Height = " + node.Cylinder.Height + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".Cylinder.Initial_Angle = " + node.Cylinder.Initial_Angle + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".Cylinder.Radius = " + node.Cylinder.Radius + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".Cylinder.Use_Cylinder = " + node.Cylinder.Use_Cylinder.ToString().ToLower() + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".Torus.Inner_Radius = " + node.Torus.Inner_Radius + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".Torus.Outer_Radius = " + node.Torus.Outer_Radius + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".Torus.Use_Torus = " + node.Torus.Use_Torus.ToString().ToLower() + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".Mass = " + node.Mass.ToString() + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".Forces.X = " + node.Forces.X.ToString() + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".Forces.Y = " + node.Forces.Y.ToString() + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".Forces.Z = " + node.Forces.Z.ToString() + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".Torque.X = " + node.Torque.X.ToString() + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".Torque.Y = " + node.Torque.Y.ToString() + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".Torque.Z = " + node.Torque.Z.ToString() + ";");

            output.WriteLine("\t\tbody" + numRigidBodies + ".CenterOfMass.X = " + node.CenterOfMass.X.ToString() + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".CenterOfMass.Y = " + node.CenterOfMass.Y.ToString() + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".CenterOfMass.Z = " + node.CenterOfMass.Z.ToString() + ";");


            output.WriteLine("\t\tbody" + numRigidBodies + ".RotXAxis = " + node.RotXAxis.ToString().ToLower() + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".RotYAxis = " + node.RotYAxis.ToString().ToLower() + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".RotZAxis = " + node.RotZAxis.ToString().ToLower() + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".TransXaxis = " + node.TransXaxis.ToString().ToLower() + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".TransYaxis = " + node.TransYaxis.ToString().ToLower() + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".TransZaxis = " + node.TransZaxis.ToString().ToLower() + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".Surface = " + node.Surface.ToString().ToLower() + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".Name = \"" + node.Name + "\";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".Text = \"" + node.Name + "\";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".ImageIndex = 1;");
            output.WriteLine("\t\tbody" + numRigidBodies + ".genCoords.q0 = " + node.genCoords.q0.ToString() + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".genCoords.q1 = " + node.genCoords.q1.ToString() + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".genCoords.q2 = " + node.genCoords.q2.ToString() + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".genCoords.q3 = " + node.genCoords.q3.ToString() + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".genCoords.q4 = " + node.genCoords.q4.ToString() + ";");
            output.WriteLine("\t\tbody" + numRigidBodies + ".genCoords.q5 = " + node.genCoords.q5.ToString() + ";");
            node.bodyNumber = numRigidBodies;
            output.WriteLine("\t\tbody" + father.bodyNumber + ".Nodes.Insert(body" + father.bodyNumber + ".Nodes.Count + 1, body" + numRigidBodies + ");");

            //targetBody.Nodes.Insert(targetBody.Nodes.Count + 1, bodyCopy);
            str = output.ToString();
            numRigidBodies++;
            //Display the Text of the node
            //MessageBox.Show(node.Text);
            //Loop Through this node and its childs recursively
            for (int j = 0; j < node.Nodes.Count; j++)
                visitChildNodes((cRigidBodyUI)node.Nodes[j], node, output, ref numRigidBodies);
        }


        /// <summary>
        /// FRICTION FORCE NEEDS TO BE DEFINED FOR EACH GENERALIZED COORDINATE
        /// </summary>
        /// <param name="t"></param>
        /// <param name="genCoords"></param>
        /// <returns></returns>
        public Function[] frictionForce(Variable t, params Function[] genCoords)
        {
            Function[] result = new Function[genCoords.Length];

            //for (int i = 0; i < genCoords.Length; i++) { result[i] = (- Math.PI/2.0 - genCoords[i] ) - .2 * Function.D(genCoords[i], t); } //this works only for one cylinder
            for (int i = 0; i < genCoords.Length; i++) { result[i] = -2 * Function.D(genCoords[i], t); }
            return result;
        }

        public Function[,] inertiaMatrixCylinder(double mass, double radius, double height)
        {
            // moment of inertia:
            return new Function[,]{{1/12.0*mass*(3*(radius*radius) + (height*height)), 0 ,0 ,0},
                         {0, .5*mass*(radius*radius), 0 ,0},
                         {0, 0, 1/12.0*mass*(3*(radius*radius) + (height*height)), 0},
                         {0,0,0,0}};
        }

        public Function[,] inertiaMatrixCylinder(Variable mass, double radius, double height)
        {
            // moment of inertia:
            return new Function[,]{{1/12.0*mass*(3*(radius*radius) + (height*height)), 0 ,0 ,0},
                         {0, .5*mass*(radius*radius), 0 ,0},
                         {0, 0, 1/12.0*mass*(3*(radius*radius) + (height*height)), 0},
                         {0,0,0,0}};
        }

        public Function[,] inertiaMatrixTorus(double mass, double largeRadius, double smallRadius)
        {
            return new Function[,]{{1/8.0*mass*(4*largeRadius*largeRadius + 5*smallRadius*smallRadius), 0 ,0 ,0},
                         {0, .25*mass*(4*largeRadius*largeRadius + 3*smallRadius*smallRadius), 0 ,0},
                         {0, 0, 1/8.0*mass*(4*largeRadius*largeRadius + 5*smallRadius*smallRadius), 0},
                         {0,0,0,0}};
        }

        public Function[,] inertiaMatrixTorus(Variable mass, double largeRadius, double smallRadius)
        {
            return new Function[,]{{1/8.0*mass*(4*largeRadius*largeRadius + 5*smallRadius*smallRadius), 0 ,0 ,0},
                         {0, .25*mass*(4*largeRadius*largeRadius + 3*smallRadius*smallRadius), 0 ,0},
                         {0, 0, 1/8.0*mass*(4*largeRadius*largeRadius + 5*smallRadius*smallRadius), 0},
                         {0,0,0,0}};
        }

        public Function[,] inertiaMatrixSphere(double mass, double radius)
        {
            return new Function[,]{ {0.4*mass*radius*radius, 0 ,0 ,0},
                                    {0, 0.4*mass*radius*radius, 0 ,0},
                                    {0, 0, 0.4*mass*radius*radius, 0},
                                    {0,0,0,0}};
        }

        public Function[,] inertiaMatrixSphere(Variable mass, double radius)
        {
            return new Function[,]{ {0.4*mass*radius*radius, 0 ,0 ,0},
                                    {0, 0.4*mass*radius*radius, 0 ,0},
                                    {0, 0, 0.4*mass*radius*radius, 0},
                                    {0,0,0,0}};
        }


    }
}
