﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using DifferentiableFunction;
using System.Drawing;
using System.Windows.Forms;
using System.Drawing.Design;
using System.Windows.Forms.Design;
using System.Globalization;     // CultureInfo

///
/// rigidBodyUI
/// author: Benjamin Hernandez (t-behern@microsoft.com, hbenjamin@itesm.mx)
/// date: 3/30/2009
/// This class will allow the programmer to specify the rigid body trough the  GUI (using the propertyGrid).
/// The elements that conform this class are taken from RigidBodySystem class. According to this class
/// a RigidBody is built from a transformation Matrix, generalized coordinates, generalized forces
/// center of mass, inertia matrix, mass, name and delta time.
/// A general example of how a rigid body is specified can be found in 
/// class private RigidBodySystem gyratory(GraphicsDevice device, Texture2D[] texture) from WindowsFormXNA.cs of
/// this project.

 

namespace WinFormsGraphicsDevice
{
    enum eAttributeChange
    {
        SELECTION,
    };

   
    public class cRigidBodyUI : TreeNode
    {
        #region class cParametricModelLoader

        public class cParametricLoaderConverter : TypeConverter
        {

            public override object ConvertTo(ITypeDescriptorContext context, CultureInfo culture, object value, Type destType)
            {
                if ((destType == typeof(string)))
                {
                    
                    return value.ToString();
                }
                return base.ConvertTo(context, culture, value, destType);
            }
        }

        public class cParametricLoader : UITypeEditor
        {
            private IWindowsFormsEditorService service;
            public string fileName;
                        
            public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context)
            {
                return UITypeEditorEditStyle.Modal;
            }

            public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value)
            {
                OpenFileDialog openFileDialog = new OpenFileDialog();
                if (provider != null)
                    service = (IWindowsFormsEditorService)provider.GetService(typeof(IWindowsFormsEditorService));

                if (service != null)
                {
                    openFileDialog.Filter = "xml model files (*.3dmodel.xml)|*.3dmodel.xml|xml files (*.xml)|*.xml|All files (*.*)|*.*";
                    openFileDialog.AddExtension = true;

                    if (openFileDialog.ShowDialog() == DialogResult.OK)
                    {
                        value = openFileDialog.FileName.ToString();
                        fileName = value.ToString();
                    }

                }
                return value;
            }
            public override string ToString()
            {
                return "...";
            }
        }
        [TypeConverter(typeof(ExpandableObjectConverter))]
        public class cParametricModel 
        {
            //private cParametricLoader loader = new cParametricLoader();
            private string fileName;
            private double width = 1.0;
            private double height = 1.0;
            private double x = 0.0;
            private double y = 0.0;
            private double z = 0.0;

            [EditorAttribute(typeof(cParametricLoader), typeof(UITypeEditor))]
            public string File
            {
                get { return fileName; }
                set { fileName = value; }
            }

            [CategoryAttribute("Model Size")]
            public double Width
            {
                get { return width; }
                set { width = value; }
            }

            [CategoryAttribute("Model Size")]
            public double Height
            {
                get { return height; }
                set { height = value; }
            }

            public override string ToString()
            {
                return "";
            }

        }
        
        #endregion

        #region class cBodyCenterofMass
        [TypeConverter(typeof(ExpandableObjectConverter))]
        public class cBodyCenterofMass
        {
            private double x = 0.0;
            private double y = 0.0;
            private double z = 0.0;

            [DefaultValue(typeof(double), "0.0"), DescriptionAttribute("Center of mass in X axis")]
            public double X
            {
                get
                {
                    return x;
                }
                set
                {
                    x = value;
                }
            }
            [DefaultValue(typeof(double), "0.0"), DescriptionAttribute("Center of mass Y axis")]
            public double Y
            {
                get
                {
                    return y;
                }
                set
                {
                    y = value;
                }
            }
            [DefaultValue(typeof(double), "0.0"), DescriptionAttribute("Center of mass in Z axis")]
            public double Z
            {
                get
                {
                    return z;
                }
                set
                {
                    z = value;
                }
            }

            public override string ToString()
            {
                return x.ToString("F") + "," + y.ToString("F") + "," + z.ToString("F");
                //return str;
            }
        }
        #endregion

        #region class cBodyForce
        [TypeConverter(typeof(ExpandableObjectConverter))]
        public class cBodyForce
        {
            private double x = 0.0;
            private double y = -9.8;
            private double z = 0.0;

            [DefaultValue(typeof(double), "0.0"), DescriptionAttribute("Force in X axis")]
            public double X
            {
                get
                {
                    return x;
                }
                set
                {
                    x = value;
                }
            }
            [DefaultValue(typeof(double), "0.0"), DescriptionAttribute("Force Y axis")]
            public double Y
            {
                get
                {
                    return y;
                }
                set
                {
                    y = value;
                }
            }
            [DefaultValue(typeof(double), "0.0"), DescriptionAttribute("Force in Z axis")]
            public double Z
            {
                get
                {
                    return z;
                }
                set
                {
                    z = value;
                }
            }

            public override string ToString()
            {
                return x.ToString("F") + "," + y.ToString("F") + "," + z.ToString("F");
                //return str;
            }
        }
        #endregion

        #region class cBodyTorque
        [TypeConverter(typeof(ExpandableObjectConverter))]
        public class cBodyTorque
        {
            private double x = 0.0;
            private double y = 0.0;
            private double z = 0.0;

            [DefaultValue(typeof(double), "0.0"), DescriptionAttribute("Torque in X axis")]
            public double X
            {
                get
                {
                    return x;
                }
                set
                {
                    x = value;
                }
            }
            [DefaultValue(typeof(double), "0.0"), DescriptionAttribute("Torque Y axis")]
            public double Y
            {
                get
                {
                    return y;
                }
                set
                {
                    y = value;
                }
            }
            [DefaultValue(typeof(double), "0.0"), DescriptionAttribute("Torque in Z axis")]
            public double Z
            {
                get
                {
                    return z;
                }
                set
                {
                    z = value;
                }
            }

            public override string ToString()
            {
                return x.ToString("F") + "," + y.ToString("F") + "," + z.ToString("F");
                //return str;
            }
        }
        #endregion

        #region class cBodyPosition
        [TypeConverter(typeof(ExpandableObjectConverter))]
        public class cBodyPosition
        {
            private double x = 0.0;
            private double y = 0.0;
            private double z = 0.0;

            [DefaultValue(typeof(double), "0.0"), DescriptionAttribute("Position in X axis")]
            public double X
            {
                get
                {
                    return x;
                }
                set
                {
                    x = value;
                }
            }
            [DefaultValue(typeof(double), "0.0"), DescriptionAttribute("Position in Y axis")]
            public double Y
            {
                get
                {
                    return y;
                }
                set
                {
                    y = value;
                }
            }
            [DefaultValue(typeof(double), "0.0"), DescriptionAttribute("Position in Z axis")]
            public double Z
            {
                get
                {
                    return z;
                }
                set
                {
                    z = value;
                }
            }

            public override string ToString()
            {
                return x.ToString("F") + "," + y.ToString("F") + "," + z.ToString("F");
                //return str;
            }
        }
        #endregion

        #region class cBodyOrientation
        [TypeConverter(typeof(ExpandableObjectConverter))]
        public class cBodyOrientation
        {
            private double x = 0.0;
            private double y = 0.0;
            private double z = 0.0;

            [DefaultValue(typeof(double), "0.0"), DescriptionAttribute("Orientation in X axis. Units: degrees")]
            public double X
            {
                get
                {
                    return x;
                }
                set
                {
                    x = value;
                }
            }
            [DefaultValue(typeof(double), "0.0"), DescriptionAttribute("Orientation in Y axis. Units: degrees")]
            public double Y
            {
                get
                {
                    return y;
                }
                set
                {
                    y = value;
                }
            }
            [DefaultValue(typeof(double), "0.0"), DescriptionAttribute("Orientation in Z axis. Units: degrees")]
            public double Z
            {
                get
                {
                    return z;
                }
                set
                {
                    z = value;
                }
            }

            public override string ToString()
            {
                return x.ToString("F") + "," + y.ToString("F") + "," + z.ToString("F");
                //return str;
            }
        }
        #endregion

        #region class cSphereShape
       
        [TypeConverter(typeof(ExpandableObjectConverter))]
        public class cSphereShape
        {       
            double radius = 0.5;
            bool selected = true;

            [DefaultValue(typeof(bool), "true"), DescriptionAttribute("Selects a torus as a Body shape")]
            public bool Use_sphere
            {
                get
                {
                    return selected;
                }
                set
                {
                    selected = value;
                    
                }
            }

            [DefaultValue(typeof(double), "0.0"), DescriptionAttribute("Sphere radius")]
            public double Radius
            {
                get
                {
                    return radius;
                }
                set
                {
                    radius = value;
                }
            }
            public override string ToString()
            {
                return "";
            }

        }
        #endregion

        #region class cTorusShape
        [TypeConverter(typeof(ExpandableObjectConverter))]
        public class cTorusShape
        {
            double innerRadius = 0.2;
            double outerRadius = 0.8;
            bool selected = false;

            [DefaultValue(typeof(bool), "false"), DescriptionAttribute("Selects a torus as a Body shape")]
            public bool Use_Torus
            {
                get
                {
                    return selected;
                }
                set
                {
                    selected = value;
                }
            }
            [DefaultValue(typeof(double), "0.2"), DescriptionAttribute("Inner radius of the torus")]
            public double Inner_Radius
            {
                get
                {
                    return innerRadius;
                }
                set
                {
                    innerRadius = value;
                }
            }
            [DefaultValue(typeof(double), "0.8"), DescriptionAttribute("Outer radius of the torus")]
            public double Outer_Radius
            {
                get
                {
                    return outerRadius;
                }
                set
                {
                    outerRadius = value;
                }
            }
            public override string ToString()
            {
                return "";
            }
        }
        #endregion

        #region class cCylinderShape
        [TypeConverter(typeof(ExpandableObjectConverter))]
        public class cCylinderShape
        {
            double height = 2.0;
            double radius = 0.3;
            double tetha0 = 0.0;
            double tetha1 = 360;
            bool selected = false;


            [DefaultValue(typeof(bool), "false"), DescriptionAttribute("Selects a cylinder as a Body shape")]
            public bool Use_Cylinder
            {
                get
                {
                    return selected;
                }
                set
                {
                    selected = value;
                }
            }
            [DescriptionAttribute("Radius of the Cylinder")]
            public double Radius
            {
                get
                {
                    
                    return radius;
                }
                set
                {
                    radius= value;
                }
            }
            [DescriptionAttribute("Height of the Cylinder")]
            public double Height
            {
                get
                {
                    return height;
                }
                set
                {
                    height = value;
                }
            }
            [DescriptionAttribute("Initial Angle, use Zero for a closed cylinder. Units: degrees ")]
            public double Initial_Angle
            {
                get
                {
                    return tetha0;
                }
                set
                {
                    tetha0= value;
                }
            }

            [DescriptionAttribute("Final angle, use 2*PI for a closed cylinder. Units: degrees ")]
            public double Final_Angle
            {
                get
                {
                    return tetha1;
                }
                set
                {
                    tetha1 = value;
                }
            }
            public override string ToString()
            {
                return "";
            }
        }
#endregion

        #region GenCoords
        [TypeConverter(typeof(ExpandableObjectConverter))]
        public class cBodyGenCoords
        {
            double q_0=0.0, q_1=0.0, q_2=0.0, q_3=0.0, q_4=0.0, q_5=0.0;
            [DescriptionAttribute("q0"), EditorBrowsable(System.ComponentModel.EditorBrowsableState.Never)]
            public double q0
            {
                get
                {
                    return q_0;
                }
                set
                {
                    q_0 = value;
                }
            }
            [DescriptionAttribute("q1")]
            public double q1
            {
                get
                {
                    return q_1;
                }
                set
                {
                    q_1 = value;
                }
            }
            [DescriptionAttribute("q2")]
            public double q2
            {
                get
                {
                    return q_2;
                }
                set
                {
                    q_2 = value;
                }
            }
            [DescriptionAttribute("q3")]
            public double q3
            {
                get
                {
                    return q_3;
                }
                set
                {
                    q_3 = value;
                }
            }
            [DescriptionAttribute("q4")]
            public double q4
            {
                get
                {
                    return q_4;
                }
                set
                {
                    q_4 = value;
                }
            }
            [DescriptionAttribute("q5")]
            public double q5
            {
                get
                {
                    return q_5;
                }
                set
                {
                    q_5 = value;
                }
            }

            public override string ToString()
            {
                return "";
            }

        }
#endregion 

        #region Removing useless TreeNode properties
        
        

        
        [Browsable (false)]
        public new System.Drawing.Color BackColor
        {
            get
            {
                return base.BackColor;
            }
            set
            {
                base.BackColor = value;
            }
        }

        [Browsable(false)]
        public new System.Drawing.Color ForeColor
        {
            get
            {

                return base.ForeColor;
            }
            set
            {
                base.ForeColor = value;
            }
        }

        [Browsable(false)]
        public new System.Drawing.Font NodeFont
        {
            get
            {

                return base.NodeFont;
            }
            set
            {
                base.NodeFont = value;
            }
        }

        [Browsable(false)]
        public new string ToolTipText
        {
            get
            {
                return base.ToolTipText;
            }
            set
            {
                base.ToolTipText = value;
            }
        }

        [CategoryAttribute("General Properties"), Browsable(false)]
        public new string Text
        {
            get
            {
                return base.Text;
            }
            set
            {
                base.Text = value;
            }
        }
        [CategoryAttribute("General Properties"), Browsable(true)]
        public new string Name
        {
            get
            {
                return base.Name;
            }
            set
            {
                base.Name = value;
                base.Text = value;
            }
        }

        [CategoryAttribute("General Properties"), Browsable(false)]
        public new object Tag
        {
            get
            {
                return base.Tag;
            }
            set
            {
                base.Tag = value;
            }
        }
        [CategoryAttribute("General Properties"), Browsable(false)]
        public new bool Checked
        {
            get
            {
                return base.Checked;
            }
            set
            {
                base.Checked = value;
            }
        }

        [CategoryAttribute("General Properties"), Browsable(false)]
        public new ContextMenu ContextMenu
        {
            get
            {
                return base.ContextMenu;
            }
            set
            {
                base.ContextMenu = value;
            }
        }
        
        [CategoryAttribute("General Properties"), Browsable(false)]
        public new ContextMenuStrip ContextMenuStrip
        {
            get
            {
                return base.ContextMenuStrip;
            }
            set
            {
                base.ContextMenuStrip = value;
            }
        }

        [CategoryAttribute("General Properties"), Browsable(false)]
        public new int ImageIndex
        {
            get
            {
                return base.ImageIndex;
            }
            set
            {
                base.ImageIndex = value;
            }
        }

        [CategoryAttribute("General Properties"), Browsable(false)]
        public new string ImageKey
        {
            get
            {
                return base.ImageKey;
            }
            set
            {
                base.ImageKey = value;
            }
        }
        [CategoryAttribute("General Properties"), Browsable(false)]
        public new int Index
        {
            get
            {
                return base.Index;
            }
          
        }

        [CategoryAttribute("General Properties"), Browsable(false)]
        public new int SelectedImageIndex
        {
            get
            {
                return base.SelectedImageIndex;
            }
            set
            {
                base.SelectedImageIndex = value;
            }
        }
        [CategoryAttribute("General Properties"), Browsable(false)]
        public new string SelectedImageKey
        {
            get
            {
                return base.SelectedImageKey;
            }
            set
            {
                base.SelectedImageKey = value;
            }
        }
        [CategoryAttribute("General Properties"), Browsable(false)]
        public new int StateImageIndex
        {
            get
            {
                return base.StateImageIndex;
            }
            set
            {
                base.StateImageIndex = value;
            }
        }
        [CategoryAttribute("General Properties"), Browsable(false)]
        public new string StateImageKey
        {
            get
            {
                return base.StateImageKey;
            }
            set
            {
                base.StateImageKey = value;
            }
        }
        #endregion
        
        public cBodyPosition position = new cBodyPosition();
        public cBodyOrientation orientation = new cBodyOrientation();

        private cSphereShape sphereShape= new cSphereShape();
        private cTorusShape torusShape = new cTorusShape();
        private cCylinderShape cylinderShape = new cCylinderShape();
        private cParametricModel parametricModel = new cParametricModel();

        public RigidBody rigidBody;
        private List<Function[,]> transform;
        public UnspecifiedFunction q0, q1, q2, q3, q4, q5;

        public Variable vMass;
        private double mass;
        private cBodyForce bodyForce = new cBodyForce();
        private cBodyTorque bodyTorque = new cBodyTorque();
        private cBodyCenterofMass bodyCOM = new cBodyCenterofMass();

        
        public cBodyGenCoords genCoords = new cBodyGenCoords();
        private bool surface = false;
        private bool rotXConstraint = true;
        private bool rotYConstraint = false;
        private bool rotZConstraint = false;
        private bool transXConstraint = false;
        private bool transYConstraint = false;
        private bool transZConstraint = false;


        public long  bodyNumber;
        
        Variable t = new Variable("t");
               
        [CategoryAttribute("General Properties"), DescriptionAttribute("Specifies the position of the body"),]
        public cBodyPosition Position
        {
            get { return position; }
            set { position = value; }
        }
        [CategoryAttribute("General Properties"), DescriptionAttribute("Specifies the orientation of the body"),]
        public cBodyOrientation Orientation
        {

            get { return orientation; }
            set { orientation  = value; }
        }

        [CategoryAttribute("Shapes"), DescriptionAttribute("Type of Shapes")]
        public cSphereShape Sphere
        {
            get {
                if (sphereShape.Use_sphere == true)
                {
                    torusShape.Use_Torus = false;
                    cylinderShape.Use_Cylinder = false;
                }
                    /*
                else if (sphereShape.Use_sphere == false && cylinderShape.Use_Cylinder == false && torusShape.Use_Torus == false)
                {
                    sphereShape.Use_sphere = true;
                }
                     * */
                return sphereShape; }
            set 
            { 
                sphereShape = value;
            }
        }

        [CategoryAttribute("Shapes"), DescriptionAttribute("Type of Shapes")]
        public cTorusShape Torus
        {
            get {
                if (torusShape.Use_Torus == true)
                {
                    sphereShape.Use_sphere = false;
                    cylinderShape.Use_Cylinder = false;

                }
                    /*
                else if (sphereShape.Use_sphere == false && cylinderShape.Use_Cylinder == false && torusShape.Use_Torus == false)
                {
                    torusShape.Use_Torus = true;
                }
                */
                return torusShape; }
            set
            {
                torusShape = value;
                
            }
        }
        [CategoryAttribute("Shapes"), DescriptionAttribute("Type of Shapes")]
        public cCylinderShape Cylinder
        {
            get
            {
                if (cylinderShape.Use_Cylinder == true)
                {
                    torusShape.Use_Torus = false;
                    sphereShape.Use_sphere = false;
                }
                    /*
                else if (sphereShape.Use_sphere == false && cylinderShape.Use_Cylinder == false && torusShape.Use_Torus == false)
                {
                    cylinderShape.Use_Cylinder = true;
                }
                     * */
                return cylinderShape;
            }
            set
            {
                cylinderShape = value;
            }
        }
        [CategoryAttribute("Shape (File)"), DescriptionAttribute("Type of Shapes")]
        public cParametricModel ParametricModel
        {
            get
            {
                return parametricModel;
            }
            set
            {
                parametricModel = value;
            }
        }
        


        [CategoryAttribute("Physical Propierties"), DescriptionAttribute("Mass of the Body")]
        public double Mass
        {
            get
            {
                return mass;
            }
            set
            {
                mass = value;
                
            }
        }

        [CategoryAttribute("Physical Propierties"), DescriptionAttribute("Body forces")]
        public cBodyForce Forces
        {
            get
            {
                return bodyForce;
            }
            set
            {
                bodyForce = value;

            }
        }

        [CategoryAttribute("Physical Propierties"), DescriptionAttribute("Body forces")]
        public cBodyTorque Torque
        {
            get
            {
                return bodyTorque;
            }
            set
            {
                bodyTorque = value;

            }
        }
        [CategoryAttribute("Physical Propierties"), DescriptionAttribute("Center of mass")]
        public cBodyCenterofMass CenterOfMass
        {
            get
            {
                return bodyCOM;
            }
            set
            {
                bodyCOM = value;

            }
        }

        [CategoryAttribute("Transformation Constraints")]
        public bool RotXAxis
        {
            get
            {
                if (rotXConstraint == true)
                {
                    rotYConstraint = rotZConstraint = false;
                    surface = false;
                    transXConstraint = transYConstraint = transZConstraint = false;
                }

                return rotXConstraint;
            }
            set
            {
                rotXConstraint = value;
            }
        }
        [CategoryAttribute("Transformation Constraints")]
        public bool RotYAxis
        {
            get
            {
                if (rotYConstraint == true)
                {
                    rotXConstraint = rotZConstraint = false;
                    surface = false;
                    transXConstraint = transYConstraint = transZConstraint = false;
                }

                return rotYConstraint;
            }
            set
            {
                rotYConstraint = value;
            }
        }
        [CategoryAttribute("Transformation Constraints")]
        public bool RotZAxis
        {
            get
            {
                if (rotZConstraint == true)
                {
                    rotXConstraint = rotYConstraint = false;
                    surface = false;
                    transXConstraint = transYConstraint = transZConstraint = false;
                }

                return rotZConstraint;
            }
            set
            {
                rotZConstraint = value;
            }
        }
        
        [CategoryAttribute("Transformation Constraints"), Browsable(false) ]
        public bool TransXaxis
        {
            get
            {
                if (transXConstraint == true)
                {
                    rotXConstraint = rotYConstraint = rotZConstraint = false;
                    transYConstraint = transZConstraint = false;
                    surface = false;
                }
                return transXConstraint;
            }
            set
            {
                transXConstraint = value;
            }
        }
        
        [CategoryAttribute("Transformation Constraints"), Browsable(false)]
        public bool TransYaxis
        {
            get
            {
                if (transYConstraint == true)
                {
                    rotXConstraint = rotYConstraint = rotZConstraint = false;
                    transXConstraint = transZConstraint = false;
                    surface = false;
                }
                return transYConstraint;
            }
            set
            {
                transYConstraint = value;
            }
        }

        [CategoryAttribute("Transformation Constraints"), Browsable(false)]
        public bool TransZaxis
        {
            get
            {
                if (transZConstraint == true)
                {
                    rotXConstraint = rotYConstraint = rotZConstraint = false;
                    transXConstraint = transYConstraint = false;
                    surface = false;
                }
                return transZConstraint;
            }
            set
            {
                transZConstraint = value;
            }
        }
        
        [CategoryAttribute("Transformation Constraints")]
        public bool Surface
        {
            get
            {
                if (surface == true)
                {
                    rotXConstraint = rotYConstraint = rotZConstraint = false;
                    transXConstraint = transYConstraint = transZConstraint = false;
                }
                return surface;
            }
            set
            {
                surface = value;
            }
        }

        [CategoryAttribute("Generalized Coordinates"), DescriptionAttribute("Set initial conditions")]
        public cBodyGenCoords Coordinates
        {
            get
            {
                return genCoords;
            }
            set
            {
                genCoords = value;
            }
        }

        public cRigidBodyUI()
        {
            mass = 1.0;
        }

        public cRigidBodyUI(string text, int imageIndex, int selectedImageIndex) : base(text, imageIndex, selectedImageIndex)
        {
            //shapesId = shapes.SPHERE;
            mass = 1.0;
            //massFunction = new Function(mass);
            //massFunction = mass;
            

        }

        public cRigidBodyUI(string text, int imageIndex, int selectedImageIndex, cRigidBodyUI[] children) : base(text, imageIndex, selectedImageIndex, children)
        {
            mass = 1.0;
            bodyNumber = 0;
        }

        public void setGUIValues(cRigidBodyUI body)
        {
            position = body.position;
            orientation = body.orientation;
            sphereShape = body.sphereShape;
            cylinderShape = body.cylinderShape;
            torusShape = body.torusShape;
            mass = body.mass;
            Forces = body.Forces;
            Torque = body.Torque;
            rotXConstraint = body.rotXConstraint;
            rotYConstraint = body.rotYConstraint;
            rotZConstraint = body.rotZConstraint;
            transXConstraint = body.transXConstraint;
            transYConstraint = body.transYConstraint;
            transZConstraint = body.transZConstraint;

            surface = body.surface;
            Name = body.Name;

            bodyCOM = body.bodyCOM;
        }
    }
}
